#!/usr/bin/perl
##*****************************************************************************
##
##     File Name:	AnonymousLogin.pl
##         Owner:	InfoPlace
##     Copyright:	InfoPlace Inc., Pleasanton, CA, USA, 1996.
##     				Not to be copied, modified or redistributed without written
##     				permission of InfoPlace.
##
##
##*****************************************************************************
##                      R E V I S I O N   H I S T O R Y
##*****************************************************************************
##
##	  Date		Time	Author	Description
##	--------	-----	------	---------------------------------------------
##	09/27/96	13:17	N		Original version
##
##*****************************************************************************

# This cgi will redirect to StartSession.pl with appropriate userid

# to set the current directory ...
require "pwd.pl";
require Config;

# Use osname for portability switches (doubled to cheaply avoid -w warning)
my $osname = $Config::Config{'osname'} || $Config::Config{'osname'};

if ($osname eq "MSWin32") {
# under NT the Perl CGI script is not run from the directory in
# which the cgi script resides (i.e., current dir is not the cgi directory)
    my($thisFilePath) = $0;
    $thisFilePath =~ s/(.*)\\[\w\-]+\.[\w\-]+/$1\\/;
    if ($thisFilePath !~ /.+\\/) {
        $thisFilePath = ".\\";   # if no path available assume current directory
    }
    # now set the include path one directory levels higher
    my($requirePath) = $thisFilePath;
    $requirePath =~ s/(.*)\\[\w\-]+\\/$1/;

    push(@INC, "$requirePath\\cgi-bin\\");
    push(@INC, "$requirePath\\code\\");

    &chdir("$requirePath\\cgi-bin\\");
}
else { #for Unix etc. where Perl is invoked from the script's directory
    push(@INC, "../code/");
}
# --------------------------------------------------------------------------------
require "SessionUtils.pl";
require "CGIUtility.pl";

require "ShowCategories.pl";

# --------------------------------------------------------------------------------
# Execute ...
#

# Read the input (from the form)
$lineread = &ReadQueryString();

# Create an associative list of (key value) pair
# of the input
%userAssocRecord = ();
&MakeAssociativeList($lineread, *userAssocRecord);

# Identify arguments that are mandatory.
@kMandatoryArguments = ("db");

# List of all arguments.
@kAllArguments = ("db", "url", "close", "publish");

# Check if all the mandatory arguments are specified
@missingArguments = ();
&CheckMandatoryFields(*kMandatoryArguments, *userAssocRecord, *missingArguments);
&CheckMissingArguments(*missingArguments, $kDocErrPagePath, $kErrMissingArgumentsTemplate);

$db = $userAssocRecord{$kAllArguments[0]};
$url = $userAssocRecord{$kAllArguments[1]};
$close = $userAssocRecord{$kAllArguments[2]};
$publish = $userAssocRecord{$kAllArguments[3]};
$user = "anonymous";

$dbpath = "";

%dbassoc = ();
%dburlassoc = ();
&ReadDatabaseNames(\%dbassoc, \%dburlassoc);
$dbpath = $dbassoc{$db};
$dburl = $dburlassoc{$db};
$result = $errNone;
if (!$dbpath || $dbpath eq "" || (!(-d $dbpath))) {
    $result = $errNoDB;
}
if ($result == $errNone) {
    # sets all globals appropriately for the given infobase ...
	&SetAllGlobals($dbpath, $dburl);

	if (-f $kBlockSystem) {
		my($templateFile) = $kDocDocPagePath . $kDocBlockSystemTemplate;
		&GenerateHTMLPage($templateFile, *gAssocList, $kDocDocPagePath);    
		exit 1;
	}

	if (open(VER, "<${kVersionFile}")) {
		my($ver) = <VER>;
		chop($ver);
		if (!&MemberOf($ver, *kProductVersionSupported)) {
			my($templateFile) = $kDocDocPagePath . $kDocSystemVersionTemplate;
			$gAssocList{"INFOBASE"} = $db;
			$gAssocList{"INFOBASEVERSION"} = $ver;
			$gAssocList{"VERSIONSUPPORTED"} = join(", ", @kProductVersionSupported);
			&GenerateHTMLPage($templateFile, *gAssocList, $kDocDocPagePath);    
			exit 1;
		}
	}
	else {
		# allow old infobases ...
		&LogMsg("Error: Failed to open version file: ${kVersionFile} : $!");
	}
}
if ($result == $errNone) {
    $result = &UMVerifyUser($user, "", 1);
}
if ($result == $errNone) {
	# Start a session now...
	local ($sessionID);
	$result = &UMEndObsoleteSessions($user); # delete any obsolete sessions first
	$result = &UMNewSession($user, *sessionID);
	my($anonyCookieAddition) = "anonymous#${url}#${close}";
	$gSessionCookie = "${kLoadedSessionIDCookie}=" . &EncodeCookieValue("${sessionID}#${dbpath}#${dburl}#${anonyCookieAddition}") .
    				  ';' ;
	#add path to cookie
	my($cookiePath) = $ENV{ 'SCRIPT_NAME' };
	if($cookiePath =~ /(.*)\/[\w\-]+\/[\w\-\.]+/) {
		$cookiePath =~ s/(.*)\/[\w\-]+\/[\w\-\.]+/$1\//; #remove cgi-bin/StartSession.pl from URL
	}
	else {
		$cookiePath = '/';
	}
	$gSessionCookie = $gSessionCookie . 'path=' . $cookiePath . ';';

	if ($publish) {
        # set up the template directory
		my($templatePath) = $kIBAnonymousTemplatePath;
		$templatePath = $kAnonymousTemplatePath if (!(-d $templatePath));
		$gAssocList{"InfoBase"} = &ReadInfoBaseName($kDocDBRootPath); 		
		&ShowCategories($kRootCategoryID, 0, *gAssocList, $templatePath . 'XU0.htm', $templatePath, 1);		
		exit 0;
	}
	$templateFile = $kDocPagePath . $kStartPage;
	$gAssocList{"DBName"} = $db;
	&GenerateHTMLPage($templateFile, *gAssocList,  $kDocPagePath);
	exit 0;
}
$templateFile = $kDocDocPagePath . $kDocAnonymousLoginTemplate;
$gAssocList{'INFOBASE'} = $db;

if ($result == $errOPFailed) {
      $gAssocList{"NOCONFIG"} =  1;
}
elsif ($result == $errEntryNotFound) { # there is no anonymous users for the given infobase
	$gAssocList{"NOANONYMOUS"} =  1;
}
elsif ($result == $errNoDB) {
	$gAssocList{"NODB"} =  1;
}
&GenerateHTMLPage($templateFile, *gAssocList, $kDocDocPagePath);
exit 0;
