#!/usr/bin/perl
##*****************************************************************************
##
##     File Name:	DeleteClearDatabase.pl
##         Owner:	InfoPlace
##     Copyright:	InfoPlace Inc., Pleasanton, CA, USA, 1996.
##     				Not to be copied, modified or redistributed without written
##     				permission of InfoPlace.
##
##
##*****************************************************************************
##                      R E V I S I O N   H I S T O R Y
##*****************************************************************************
##
##	  Date		Time	Author	Description
##	--------	-----	------	---------------------------------------------
##	09/15/96	13:27	N		Original version
##
##*****************************************************************************
# Reset the database

# to set the current directory ...
require "pwd.pl";
require Config;

# Use osname for portability switches (doubled to cheaply avoid -w warning)
my $osname = $Config::Config{'osname'} || $Config::Config{'osname'};

if ($osname eq "MSWin32") {
# under NT the Perl CGI script is not run from the directory in
# which the cgi script resides (i.e., current dir is not the cgi directory)
    my($thisFilePath) = $0;
    $thisFilePath =~ s/(.*)\\[\w\-]+\.[\w\-]+/$1\\/;
    if ($thisFilePath !~ /.+\\/) {
        $thisFilePath = ".\\";   # if no path available assume current directory
    }
    # now set the include path one directory levels higher
    my($requirePath) = $thisFilePath;
    $requirePath =~ s/(.*)\\[\w\-]+\\/$1/;

    push(@INC, "$requirePath\\cgi-bin\\");
    push(@INC, "$requirePath\\code\\");

    &chdir("$requirePath\\cgi-bin\\");
}
else { #for Unix etc. where Perl is invoked from the script's directory
    push(@INC, "../code/");
}
# --------------------------------------------------------------------------------
require "VerifySession.pl";

require "DocDB.pl";
# Delete a directory tree. Assumes permissions are set for delete.
sub DeleteDirRecurse { 
	my($dirpath) = @_;
    my(@files);
	unless (opendir(FILES, $dirpath)) {
		 print("DeleteDirRecurse:: open $dirpath failed\n");
		 return;
	}
    @files = readdir(FILES);
    closedir(FILES);
    my($i);
    my($filename);
    for ($i = 0; $i < $#files + 1; $i++) {
		$filename = $dirpath . "/". $files[$i];
		#print "looking at: $filename...\n";
		if (-d $filename) {# directories
			if(($files[$i] ne '.') && ($files[$i] ne '..')) { # not self or parent
				#print "DeleteDirRecurse:: DeleteDirRecurse($filename)\n";
				&DeleteDirRecurse($filename);
				#now delete dir
				#print "DeleteDirRecurse:: Now rmdir $filename\n";
				rmdir $filename;
			}#inner if
		}#if directory
		else { #if a file
			#print "DeleteDirRecurse:: unlink $filename\n";
			unlink $filename;
		}#else
	}#for
	#print "DeleteDirRecurse:: Now rmdir $dirpath\n";
	rmdir $dirpath;
}#DeleteDirRecurse

# --------------------------------------------------------------------------------
# Execute ...
#

# verify user acccess first
&VerifyUserAccess($kSuperAdminMode);

# Read the query string
$lineread = &ReadQueryString();

# Create an associative list of (key value) pair
# of the input
%userAssocRecord = ();
&MakeAssociativeList($lineread, *userAssocRecord);

# Identify arguments that are mandatory.
@kMandatoryArguments = ("db", "action");

# List of all arguments.
@kAllArguments = ("db", "action");

# Check if all the mandatory arguments are specified
@missingArguments = ();
&CheckMandatoryFields(*kMandatoryArguments, *userAssocRecord, *missingArguments);
&CheckMissingArguments(*missingArguments, $kDocErrPagePath, $kErrMissingArgumentsTemplate);

$db = $userAssocRecord{$kAllArguments[0]};
$action = $userAssocRecord{$kAllArguments[1]};  
$dbdelete = ($action eq "delete") ? 1 : 0;
$dbclear = ($action eq "clear") ? 1 : 0;
$dbpath = "";

%dbassoc = ();
%dburlassoc = ();
&ReadDatabaseNames(\%dbassoc, \%dburlassoc);
$dbpath = $dbassoc{$db};
$dburl = $dburlassoc{$db};
$result = $errNone;
$msg = "";
if (!$dbpath || $dbpath eq "" || (!(-d $dbpath))) {
	&RedirectURL("../cgi-bin/StaleObject.pl?obj=Infobase");
	exit 0;		
    $result = $errNoDB;
    $msg = "'$db' InfoBase directory not found.<br>";
	&LogMsg("'$db' InfoBase directory: $dbpath not found.<br>");
}
if($result == $errNone) {
	&SetAllGlobals($dbpath, $dburl);
	# first reset the log file
	&ResetLogFile();

	$result = 1; # error case ...
	# reset the DBs
	if ($dbdelete || $dbclear) {
		$result = &DeleteAllDBs();
	}
	if ($dbclear) {
		$result = &DefineAllDBs();
	}
	if ($dbdelete) { 
		# remove the entry from config/InfoBases file if required
		# delete the infobase directory
		$dbpath = $dbassoc{$db};
		chop($dbpath);
		# remove the whole db directory ...
		#system("rm -rf $dbpath"); # only works on UNIX systems
		&DeleteDirRecurse($dbpath);

		my($iConfigChanged) = 0;
		my(%newdbassoc) = ();
		my($dbNum) = -1; #an invalid value
		&ReadSpecifiedConfigFile($kInfobasesConfigFile, \%newdbassoc);
		foreach $key (keys(%newdbassoc)) {
			if(	($key =~ /^directory([0-9]+)$/) && #directory
				($newdbassoc{$key} eq $dbassoc{$db})) { #matches the one to be deleted
				$dbNum = $1;
				delete $newdbassoc{$key};
				delete $newdbassoc{'url' . $dbNum};
				$iConfigChanged = 1;
			}#if current db to be deleted
		}#foreach
		if($iConfigChanged) {
			# rewrite to the InfoBases file, if needed
			&WriteSpecifiedConfigFile($kInfobasesConfigFile, \%newdbassoc);
		}
	}#if delete
}
if ($result == $errNone) {
    $templateFile = $kDocMsgPagePath .  $kMsgAllAdminOpSuccessfullTemplate;
    local(%assocList) = ();
	$assocList{"CloseOnLoad"} = "true";# close the window after loading
    &GenerateHTMLPage($templateFile, *assocList, $kDocMsgPagePath);
}
else {
    $msg = $msg . "Failed to clear the database '$db'";
    $templateFile = $kDocErrPagePath .  $kErrInternalErrorTemplate;
    local(%assocList) = ("Body", "Failed to clear/delete database '$dbname'<BR>$msg");
    $assocList{"Body"} =  $msg;
    &GenerateHTMLPage($templateFile, *assocList, $kDocErrPagePath);
}

exit 0;
