#!/usr/bin/perl
##*****************************************************************************
##
##     File Name:	EditGlossary.pl
##         Owner:	InfoPlace
##     Copyright:	InfoPlace Inc., Pleasanton, CA, USA, 1996.
##     				Not to be copied, modified or redistributed without written
##     				permission of InfoPlace.
##
##
##*****************************************************************************
##                      R E V I S I O N   H I S T O R Y
##*****************************************************************************
##
##	  Date		Time	Author	Description
##	--------	-----	------	---------------------------------------------
##	09/15/96	13:26	N		Original version
##
##*****************************************************************************

# The Submit button from the Edit Form for a Glossary Term will invoke this cgi.
# This will output a new read only detailed view of the Glossary term.

# to set the current directory ...
require "pwd.pl";
require Config;

# Use osname for portability switches (doubled to cheaply avoid -w warning)
my $osname = $Config::Config{'osname'} || $Config::Config{'osname'};

if ($osname eq "MSWin32") {
# under NT the Perl CGI script is not run from the directory in
# which the cgi script resides (i.e., current dir is not the cgi directory)
    my($thisFilePath) = $0;
    $thisFilePath =~ s/(.*)\\[\w\-]+\.[\w\-]+/$1\\/;
    if ($thisFilePath !~ /.+\\/) {
        $thisFilePath = ".\\";   # if no path available assume current directory
    }
    # now set the include path one directory levels higher
    my($requirePath) = $thisFilePath;
    $requirePath =~ s/(.*)\\[\w\-]+\\/$1/;

    push(@INC, "$requirePath\\cgi-bin\\");
    push(@INC, "$requirePath\\code\\");

    &chdir("$requirePath\\cgi-bin\\");
}
else { #for Unix etc. where Perl is invoked from the script's directory
    push(@INC, "../code/");
}
# --------------------------------------------------------------------------------
require "VerifySession.pl";

require "DocAPI.pl";
require "Glossary.pl";
require "DocUtils.pl";

# --------------------------------------------------------------------------------
# Execute ...
#

# verify user acccess first
&VerifyUserAccess($kEditMode);

# Read the input (from the form)
$lineread = &ReadQueryString();

# Create an associative list of (key value) pair
# of the input
%userAssocRecord = ();
&MakeAssociativeList($lineread, *userAssocRecord);

# Identify arguments that are mandatory.
@kMandatoryArguments = ($kGlossaryDBFields[$kGlossaryTermIndex], $kGlossaryDBFields[$kGlossaryDefinitionIndex],
						"old" . $kGlossaryDBFields[$kGlossaryTermIndex]);

# List of all arguments.
@kAllArguments = ($kGlossaryDBFields[$kGlossaryTermIndex], $kGlossaryDBFields[$kGlossaryDefinitionIndex],
				  "old" . $kGlossaryDBFields[$kGlossaryTermIndex]);

# Check if all the mandatory arguments are specified
@missingArguments = ();
&CheckMandatoryFields(*kMandatoryArguments, *userAssocRecord, *missingArguments);
&CheckMissingArguments(*missingArguments, $kDocErrPagePath, $kErrMissingArgumentsTemplate);

$term = $userAssocRecord{$kAllArguments[0]};
$def = $userAssocRecord{$kAllArguments[1]};
$oldTerm = $userAssocRecord{$kAllArguments[2]};

if (!&IsNameInProperCharSet($term)) {
	$gAssocList{"NAMENOTINCHARSET"} = 1;
	$gAssocList{"EditTerm"} = $term;
	$templateFile = $kDocDocPagePath . $kDocEditGlossaryTemplate;
	$gAssocList{$kGlossaryDBFields[$kGlossaryDefinitionIndex]} = $def;
	$gAssocList{$kGlossaryDBFields[$kGlossaryTermIndex]} = $oldTerm;
	$gAssocList{'editsuccess'} = 0; #needed for JavaScript only for edit saves
	&GenerateHTMLPage($templateFile, *gAssocList, $kDocDocPagePath);
	exit 0;
}

@gRecord = ();
$gRecord[$kGlossaryTermIndex] = $term;
$gRecord[$kGlossaryDefinitionIndex] = $def;

$result = &OPGetGlossary($oldTerm, *glossary, *kGlossaryGetAllFieldsMask);
if($result != $errNone) { # handle error
	&RedirectURL("../cgi-bin/StaleObject.pl?obj=Glossary%20Term");
	exit 0;
}

$result = &OPSetGlossary($oldTerm, *gRecord);
if ($result == $errNone) {
#	local($edit) = "true";
#	&ShowGlossaryTerms($edit, *gAssocList);
   	$templateFile = $kDocDocPagePath . $kDocEditGlossaryTemplate;
	$gAssocList{$kGlossaryDBFields[$kGlossaryDefinitionIndex]} = $def;
	$gAssocList{$kGlossaryDBFields[$kGlossaryTermIndex]} = $term;
	$gAssocList{'editsuccess'} = 1; #needed for JavaScript only for edit saves
	&GenerateHTMLPage($templateFile, *gAssocList, $kDocDocPagePath);
	exit 0;
}
if ($result == $errDuplicateGlossary) {
	$gAssocList{"errDuplicateGlossary"} = 1;
}
$templateFile = $kDocDocPagePath . $kDocEditGlossaryTemplate;
$gAssocList{"EDITERROR"} = 1;
$gAssocList{'editsuccess'} = 0; #needed for JavaScript only for edit saves
$gAssocList{$kGlossaryDBFields[$kGlossaryDefinitionIndex]} = $def;
$gAssocList{$kGlossaryDBFields[$kGlossaryTermIndex]} = $term;
$gAssocList{"OrigTerm"} = $oldTerm;
&GenerateHTMLPage($templateFile, *gAssocList, $kDocDocPagePath);

exit 0;
