#!/usr/bin/perl
##*****************************************************************************
##
##     File Name:	ExportData.pl
##         Owner:	InfoPlace
##     Copyright:	InfoPlace Inc., Pleasanton, CA, USA, 1996.
##     				Not to be copied, modified or redistributed without written
##     				permission of InfoPlace.
##
##
##*****************************************************************************
##                      R E V I S I O N   H I S T O R Y
##*****************************************************************************
##
##	  Date		Time	Author	Description
##	--------	-----	------	---------------------------------------------
##	09/15/96	13:27	N		Original version
##
##*****************************************************************************

# Reset the database and populate the database with the given export


# to set the current directory ...
require "pwd.pl";
require Config;

# Use osname for portability switches (doubled to cheaply avoid -w warning)
my $osname = $Config::Config{'osname'} || $Config::Config{'osname'};

if ($osname eq "MSWin32") {
# under NT the Perl CGI script is not run from the directory in
# which the cgi script resides (i.e., current dir is not the cgi directory)
    my($thisFilePath) = $0;
    $thisFilePath =~ s/(.*)\\[\w\-]+\.[\w\-]+/$1\\/;
    if ($thisFilePath !~ /.+\\/) {
        $thisFilePath = ".\\";   # if no path available assume current directory
    }
    # now set the include path one directory levels higher
    my($requirePath) = $thisFilePath;
    $requirePath =~ s/(.*)\\[\w\-]+\\/$1/;

    push(@INC, "$requirePath\\cgi-bin\\");
    push(@INC, "$requirePath\\code\\");

    &chdir("$requirePath\\cgi-bin\\");
}
else { #for Unix etc. where Perl is invoked from the script's directory
    push(@INC, "../code/");
}
# --------------------------------------------------------------------------------
require "VerifySession.pl";

require "DocDB.pl";
require "Export.pl";

# --------------------------------------------------------------------------------
# Execute ...
#

# verify user acccess first
&VerifyUserAccess($kAdminMode);

# Read the input (from the form)
$lineread = &ReadQueryString();

# Create an associative list of (key value) pair
# of the input
%userAssocRecord = ();
&MakeAssociativeList($lineread, *userAssocRecord);

# Identify arguments that are mandatory.
@kMandatoryArguments = ("samplename", "savetype");

# List of all arguments.
@kAllArguments = ("samplename", "savetype", "documents", "categories", "glossary", "users", "groups");

# Check if all the mandatory arguments are specified
@missingArguments = ();
&CheckMandatoryFields(*kMandatoryArguments, *userAssocRecord, *missingArguments);
&CheckMissingArguments(*missingArguments, $kDocErrPagePath, $kErrMissingArgumentsTemplate);


# now save the database ...
$exportName = $userAssocRecord{$kAllArguments[0]};
$exportPath = $kDocExportBasePath . $exportName;

$savetype = $userAssocRecord{$kAllArguments[1]};
local($saveall) = 0;
if ($savetype eq "all") {
    $saveall = 1;
    if (-d $exportPath) {
	$templateFile = $kDocErrPagePath . $kErrInternalErrorTemplate;
	$gAassocList{"Body"} = "The export directory \"$exportName\" already exists. Try with a different name.";
	&GenerateHTMLPage($templateFile, *gAssocList, $kDocErrPagePath);
	exit 1;
    }
}

# first create the directory, if not there ...
if (!(-d $exportPath)) {
    umask 0000;
    if (!mkdir($exportPath, $kDBDefaultPermission)) {
		$templateFile = $kDocDocPagePath . $kDocExportTemplate;
		$gAssocList{'EXPORTERROR'} = 1;
		$gAssocList{'ExportPath'} = $exportPath;
		$gAssocList{'Infobase'} = &ReadInfoBaseName($kDocDBRootPath);
		$gAssocList{'ExportDir'} = $kDocExportBasePath;
		$gAssocList{'Name'} = $exportName;
		&GenerateHTMLPage($templateFile, *gAssocList, $kDocDocPagePath);
		return 1;
    }
}

$exportPath = $exportPath . $kDBPathNameSeparator;
$categoryFile = $exportPath . "Categories";
$documentsFile = $exportPath . "Documents";
$glossaryFile = $exportPath . "Glossary";
$usersFile = $exportPath . "Users";
$userProfilesFile = $exportPath . "UserProfiles";
$groupsFile = $exportPath . "Groups";

if ($saveall || $userAssocRecord{$kAllArguments[3]}) {
    &WriteCategories($categoryFile, $kRootCategoryID);
}
if ($saveall || $userAssocRecord{$kAllArguments[2]}) {
    &WriteDocuments($documentsFile, $kRootCategoryID);
}
if ($saveall || $userAssocRecord{$kAllArguments[4]}) {
    &WriteGlossaries($glossaryFile);
}
if ($saveall || $userAssocRecord{$kAllArguments[5]}) {
    &WriteUsers($usersFile);
    &WriteUsersProfiles($userProfilesFile);
}
if ($saveall || $userAssocRecord{$kAllArguments[6]}) {
    &WriteGroups($groupsFile);
}

$templateFile = $kDocMsgPagePath . $kMsgIBAdminOpSuccessfullTemplate;
&GenerateHTMLPage($templateFile, *gAssocList, $kDocMsgPagePath);

exit 0;
