#!/usr/bin/perl
##*****************************************************************************
##
##     File Name:	FindDocument.pl
##         Owner:	InfoPlace
##     Copyright:	InfoPlace Inc., Pleasanton, CA, USA, 1996.
##     				Not to be copied, modified or redistributed without written
##     				permission of InfoPlace.
##
##
##*****************************************************************************
##                      R E V I S I O N   H I S T O R Y
##*****************************************************************************
##
##	  Date		Time	Author	Description
##	--------	-----	------	---------------------------------------------
##	09/15/96	13:07	N		What you are doing...
##	09/15/96	13:05	N		Original version
##
##*****************************************************************************

# This will be invoked from the Find form for documents.
# This cgi will take the criteria and display the resultant records.


# to set the current directory ...
require "pwd.pl";
require Config;

# Use osname for portability switches (doubled to cheaply avoid -w warning)
my $osname = $Config::Config{'osname'} || $Config::Config{'osname'};

if ($osname eq "MSWin32") {
# under NT the Perl CGI script is not run from the directory in
# which the cgi script resides (i.e., current dir is not the cgi directory)
    my($thisFilePath) = $0;
    $thisFilePath =~ s/(.*)\\[\w\-]+\.[\w\-]+/$1\\/;
    if ($thisFilePath !~ /.+\\/) {
        $thisFilePath = ".\\";   # if no path available assume current directory
    }
    # now set the include path one directory levels higher
    my($requirePath) = $thisFilePath;
    $requirePath =~ s/(.*)\\[\w\-]+\\/$1/;

    push(@INC, "$requirePath\\cgi-bin\\");
    push(@INC, "$requirePath\\code\\");

    &chdir("$requirePath\\cgi-bin\\");
}
else { #for Unix etc. where Perl is invoked from the script's directory
    push(@INC, "../code/");
}
# --------------------------------------------------------------------------------
require "VerifySession.pl";

require "DocAPI.pl";
require "OPFind.pl";
require "Listing.pl";
require "Search.pl";
require "UMProfileAPI.pl";

# --------------------------------------------------------------------------------
# Execute ...
#

# Read from stdin
$lineread = &ReadQueryString();

# Create an associative list of (key value) pair
# of the input
%userAssocRecord = ();
&MakeAssociativeList($lineread, *userAssocRecord);

# Identify arguments that are mandatory.
@kMandatoryArguments = ("namecriteria", "authorcriteria", "createcriteria",
						"modifycriteria", "versioncriteria", "apptype", "keywordscriteria");

# List of all arguments.
@kAllArguments = ("title", "namecriteria", "author", "authorcriteria",
				  "createtime", "createcriteria", "modifytime", "modifycriteria", 
				  "version", "versioncriteria", "apptype",
				  "keywords", "keywordscriteria", "categories", "categorycriteria",
				  "query", "modificationdate", "creationdate", "description", "descriptioncriteria");

# Check if all the mandatory arguments are specified
@missingArguments = ();
&CheckMandatoryFields(*kMandatoryArguments, *userAssocRecord, *missingArguments);
&CheckMissingArguments(*missingArguments, $kDocErrPagePath, $kErrMissingArgumentsTemplate);


@documents = ();
@document = ();
$document[$kDocNameIndex] = $userAssocRecord{$kAllArguments[0]};
$document[$kDocAuthorIndex] = $userAssocRecord{$kAllArguments[2]};
$document[$kDocCreationTimeIndex] = $userAssocRecord{$kAllArguments[4]};
# $document[$kDocSizeIndex] = 0;
$document[$kDocModificationTimeIndex] = $userAssocRecord{$kAllArguments[6]};
$document[$kDocVersionIndex] = $userAssocRecord{$kAllArguments[8]};
$appType = $userAssocRecord{$kAllArguments[10]};
if ($appType ne "Unknown") {
	$document[$kDocAppTypeIndex] = $appType;
}
$document[$kDocKeywordsIndex] = $userAssocRecord{$kAllArguments[11]};
$document[$kDocCategoryListIndex] = $userAssocRecord{$kAllArguments[13]};
$document[$kDocDescriptionIndex] = $userAssocRecord{$kAllArguments[18]};
$query = $userAssocRecord{$kAllArguments[15]};

@criteria = ();
$criteria[$kDocNameIndex] = &GetStringCriteria($userAssocRecord{$kAllArguments[1]});
$criteria[$kDocAuthorIndex] = &GetStringCriteria($userAssocRecord{$kAllArguments[3]});
$criteria[$kDocCreationTimeIndex] = &GetNumberCriteria($userAssocRecord{$kAllArguments[5]});
$criteria[$kDocSizeIndex] = -1; #&GetNumberCriteria("Greater Than");
$criteria[$kDocModificationTimeIndex] = &GetNumberCriteria($userAssocRecord{$kAllArguments[7]});
$criteria[$kDocVersionIndex] = &GetNumberCriteria($userAssocRecord{$kAllArguments[9]});
$criteria[$kDocAppTypeIndex] = &GetStringCriteria("Equal");
$criteria[$kDocKeywordsIndex] = &GetStringCriteria($userAssocRecord{$kAllArguments[12]});
$criteria[$kDocDescriptionIndex] = &GetStringCriteria($userAssocRecord{$kAllArguments[19]});
$criteria[$kDocCategoryListIndex] = &GetStringCriteria($userAssocRecord{$kAllArguments[14]});

$kDocPropertyTitleName = "Title";
$kDocPropertyAuthorName = "Author";
$kDocPropertyDocTypeName = "Document Type";
$kDocPropertyCategoriesName = "Categories";
$kDocPropertyDescName = "Description";
$kDocPropertyKeywordsName = "Keywords";
$kDocPropertyVersionName = "Version";
$kDocPropertyAddedName = "Added On";
$kDocPropertyEditedName = "Last Edited";

@usercriteria = ();
$uidx = 0;
$usercriteria[$uidx++] = "$kDocPropertyTitleName " . &MapComparisonOperator($userAssocRecord{$kAllArguments[1]}, $kDocPropertyTitleName) . "</FONT></TD><TD valign=\"top\"><FONT SIZE=-1>" .  $document[$kDocNameIndex]  if $document[$kDocNameIndex] ne "";
$usercriteria[$uidx++] = "$kDocPropertyAuthorName " . &MapComparisonOperator($userAssocRecord{$kAllArguments[3]}, $kDocPropertyAuthorName) . "</FONT></TD><TD valign=\"top\"><FONT SIZE=-1>" . $document[$kDocAuthorIndex]  if $document[$kDocAuthorIndex] ne "";
$usercriteria[$uidx++] = "$kDocPropertyDocTypeName Is </FONT></TD><TD valign=\"top\"><FONT SIZE=-1>" . $document[$kDocAppTypeIndex] if $document[$kDocAppTypeIndex] ne "";
if ($document[$kDocCategoryListIndex] ne "") {
	my(@cats) = split($kValueSeperator, $document[$kDocCategoryListIndex]);
	my($catStr) = &GetCategoryPath($cats[0], 1, 1);
	for($i=1; $i <= $#cats; $i++) {
		$catID = @cats[$i];
		$catStr = $catStr . " <B>or</B> " . &GetCategoryPath($catID, 1, 1);
	}#for
	$usercriteria[$uidx++] = "$kDocPropertyCategoriesName In </FONT></TD><TD valign=\"top\"><FONT SIZE=-1>" . $catStr;
}
$usercriteria[$uidx++] = "$kDocPropertyDescName " . $userAssocRecord{$kAllArguments[19]} . "</FONT></TD><TD valign=\"top\"><FONT SIZE=-1>" . $document[$kDocDescriptionIndex] if $document[$kDocDescriptionIndex] ne "";
if ($document[$kDocKeywordsIndex] ne "") {
	my(@keys) = split($kValueSeperator, $document[$kDocKeywordsIndex]);
	my(@modifiedKeys) = ();
	foreach $keyword (@keys) {
		$keyword =~ s/^(\s)+//; #remove leading white spaces in each key word
		push(@modifiedKeys, $keyword);
	}
	$document[$kDocKeywordsIndex] = join($kValueSeperator, @modifiedKeys);
	my($newkeys) = join(" <B>or</B> ", @keys);
	$usercriteria[$uidx++] = "$kDocPropertyKeywordsName " . $userAssocRecord{$kAllArguments[12]} . "</FONT></TD><TD valign=\"top\"><FONT SIZE=-1>" . $newkeys;
}
$usercriteria[$uidx++] = "$kDocPropertyVersionName " . &MapComparisonOperator($userAssocRecord{$kAllArguments[9]}, kDocPropertyVersionName) . "</FONT></TD><TD valign=\"top\"><FONT SIZE=-1>" . $document[$kDocVersionIndex] if $document[$kDocVersionIndex] ne "";
$usercriteria[$uidx++] = "$kDocPropertyAddedName " . &MapComparisonOperator($userAssocRecord{$kAllArguments[5]}, $kDocPropertyAddedName) . "</FONT></TD><TD valign=\"top\"><FONT SIZE=-1>" . $userAssocRecord{$kAllArguments[17]} if $userAssocRecord{$kAllArguments[17]} ne "";
$usercriteria[$uidx++] = "$kDocPropertyEditedName " . &MapComparisonOperator($userAssocRecord{$kAllArguments[7]}, $kDocPropertyEditedName) . "</FONT></TD><TD valign=\"top\"><FONT SIZE=-1>" . $userAssocRecord{$kAllArguments[16]} if $userAssocRecord{$kAllArguments[16]} ne "";

#$fullusercriteria = join("<BR> and ", @usercriteria);
$fullusercriteria = join("</FONT></TD></TR>\n<TR BGCOLOR=#dfdfdf><TD valign=\"top\" Align=right><FONT SIZE=-1><B>and</B></FONT></TD><TD valign=\"top\" Align=left><FONT SIZE=-1>", @usercriteria);
if($fullusercriteria eq "") {
	$fullusercriteria = 'In</FONT></TD><TD><FONT>All Categories';
}
$gAssocList{"SearchCriteria"} = "${fullusercriteria}";
my($userID) = $gAssocList{$kUserIDCookie};
$gAssocList{"DocViewType"} = &UMGetDocViewTypePrefFromUserID($userID);
$gAssocList{"DocumentParent"} = "";
$result = &OPFindDocument(*document, *criteria, *documents);
@finalList = ();
if($result == 0 && $query) {
    $query =~ tr/\t/ /;  #replace tabs by spaces
    $query = join(' ', split(' ', $query));
    if ($query) {
	  my(@docs, @docWeights);
	  $result = !&SEFindMatchedDocs($query, \@docs, \@docWeights);
	  foreach $docid (@docs) {
	    push(@finalList, $docid) if (&MemberOf($docid, *documents));
	  }
    }
}
if ($result == 0) {
    &ShowFindResult(*gAssocList, ($query) ? *finalList : *documents, $userID);
}
else {
    my($errPage) = $kDocErrPagePath . $kErrInternalErrorTemplate;
    $gAssocList{"Body"} = "Find Document failed!";
    &GenerateHTMLPage($errPage, *gAssocList, $kDocErrPagePath);
}

exit 0;

sub MapComparisonOperator() {
	my($logicalOpOrig, $property) = @_;
	$logicalOp = $logicalOpOrig;
	$logicalOp =~ tr/A-Z/a-z/; # make it lower case
	if(($property eq $kDocPropertyTitleName) || 
	   ($property eq $kDocPropertyAuthorName)) {
		if($logicalOp eq "equal") {
			return "Is";
		}
	}
	elsif($property eq $kDocPropertyVersionName) {
		if($logicalOp eq "equal") {
			return "Equal To";
		}
	}
	elsif(($property eq $kDocPropertyAddedName) || 
		  ($property eq $kDocPropertyEditedName)) {
		if($logicalOp eq "less than") {
			return "Before";
		}
		elsif($logicalOp eq "greater than") {
			return "After";
		}
	}
	return $logicalOpOrig;
}#MapComparisonOperator