#!/usr/bin/perl
##*****************************************************************************
##
##     File Name:	ImportData.pl
##         Owner:	InfoPlace
##     Copyright:	InfoPlace Inc., Pleasanton, CA, USA, 1996.
##     				Not to be copied, modified or redistributed without written
##     				permission of InfoPlace.
##
##
##*****************************************************************************
##                      R E V I S I O N   H I S T O R Y
##*****************************************************************************
##
##	  Date		Time	Author	Description
##	--------	-----	------	---------------------------------------------
##	09/15/96	13:27	N		Original version
##
##*****************************************************************************

# Reset the database and populate the database with the given import

# to set the current directory ...
require "pwd.pl";
require Config;

# Use osname for portability switches (doubled to cheaply avoid -w warning)
my $osname = $Config::Config{'osname'} || $Config::Config{'osname'};

if ($osname eq "MSWin32") {
# under NT the Perl CGI script is not run from the directory in
# which the cgi script resides (i.e., current dir is not the cgi directory)
    my($thisFilePath) = $0;
    $thisFilePath =~ s/(.*)\\[\w\-]+\.[\w\-]+/$1\\/;
    if ($thisFilePath !~ /.+\\/) {
        $thisFilePath = ".\\";   # if no path available assume current directory
    }
    # now set the include path one directory levels higher
    my($requirePath) = $thisFilePath;
    $requirePath =~ s/(.*)\\[\w\-]+\\/$1/;

    push(@INC, "$requirePath\\cgi-bin\\");
    push(@INC, "$requirePath\\code\\");

    &chdir("$requirePath\\cgi-bin\\");
}
else { #for Unix etc. where Perl is invoked from the script's directory
    push(@INC, "../code/");
}
# --------------------------------------------------------------------------------
require "VerifySession.pl";

require "DocDB.pl";
require "Import.pl";

# --------------------------------------------------------------------------------
# Execute ...
#

# verify user acccess first
&VerifyUserAccess($kAdminMode);

# Read the input (from the form)
$lineread = &ReadQueryString();

# Create an associative list of (key value) pair
# of the input
%userAssocRecord = ();
&MakeAssociativeList($lineread, *userAssocRecord);

# Identify arguments that are mandatory.
@kMandatoryArguments = ("samplename", "loadtype");

# List of all arguments.
@kAllArguments = ("samplename", "loadtype", "documents", "categories", "glossary", "users", "groups");

# Check if all the mandatory arguments are specified
@missingArguments = ();
&CheckMandatoryFields(*kMandatoryArguments, *userAssocRecord, *missingArguments);
&CheckMissingArguments(*missingArguments, $kDocErrPagePath, $kErrMissingArgumentsTemplate);


# first reset the log file

$loadtype = $userAssocRecord{$kAllArguments[1]};
local($loadall) = 0;
# reset the DBs, if loadtype == "all"
if ($loadtype eq "all") {
    &ResetLogFile();
    &DeleteAllDBs();
    $result = &DefineAllDBs();
    if ($result != $errNone) {
		$templateFile = $kDocErrPagePath . $kErrInternalErrorTemplate;
		$gAssocList{"Body"} = "Failed to create new databases.";
		&GenerateHTMLPage($templateFile, *gAssocList, $kDocErrPagePath);
		exit 1;
    }
    $loadall = 1;
}

# now populate the database ...
$importName = $userAssocRecord{$kAllArguments[0]};
$importPath = $kDocExportBasePath . $importName . $kDBPathNameSeparator;

if (!(-d $importPath)) {
    $templateFile = $kDocErrPagePath . $kErrInternalErrorTemplate;
    $gAssocList{"Body"} =  "The import directory \"$importName\" does not exists.";
    &GenerateHTMLPage($templateFile, *gAssocList, $kDocErrPagePath);
    exit 1;
}
$categoryFile = $importPath . "Categories";
$documentsFile = $importPath . "Documents";
$glossaryFile = $importPath . "Glossary";
$usersFile = $importPath . "Users";
$userProfilesFile = $importPath . "UserProfiles";
$groupsFile = $importPath . "Groups";

if ($loadall || $userAssocRecord{$kAllArguments[3]}) {
    if (-f $categoryFile) {
	&ReadCategories($categoryFile, $kRootCategoryID);
    }
}
if ($loadall || $userAssocRecord{$kAllArguments[2]}) {
    if (-f $documentsFile) {
	&ReadDocuments($documentsFile, $kRootCategoryID);
    }
}
if ($loadall || $userAssocRecord{$kAllArguments[4]}) {
    if (-f $glossaryFile) {
	&ReadGlossaries($glossaryFile);
    }
}
if ($loadall || $userAssocRecord{$kAllArguments[5]}) {
    if (-f $usersFile) {
	&ReadUsers($usersFile);
    }
    if (-f $userProfilesFile) {
	&ReadUserProfiles($userProfilesFile);
    }
}
if ($loadall || $userAssocRecord{$kAllArguments[6]}) {
    if (-f $groupsFile) {
	&ReadGroups($groupsFile);
    }
}

if ($loadall) {
    $templateFile = $kDocDocPagePath . $kDocGotoLoginPromptTemplate;
    &GenerateHTMLPage($templateFile, *gAssocList, $kDocDocPagePath);
}
else {
    $templateFile = $kDocMsgPagePath . $kMsgIBAdminOpSuccessfullTemplate;
    &GenerateHTMLPage($templateFile, *gAssocList, $kDocMsgPagePath);
}
exit 0;
