#!/usr/bin/perl
##*****************************************************************************
##
##     File Name:	StartSession.pl
##         Owner:	InfoPlace
##     Copyright:	InfoPlace Inc., Pleasanton, CA, USA, 1996.
##     				Not to be copied, modified or redistributed without written
##     				permission of InfoPlace.
##
##
##*****************************************************************************
##                      R E V I S I O N   H I S T O R Y
##*****************************************************************************
##
##	  Date		Time	Author	Description
##	--------	-----	------	---------------------------------------------
##	09/27/96	13:17	N		Original version
##
##*****************************************************************************

# This cgi will check the validity of the userId and matching password


# to set the current directory ...
require "pwd.pl";
require Config;

# Use osname for portability switches (doubled to cheaply avoid -w warning)
my $osname = $Config::Config{'osname'} || $Config::Config{'osname'};

if ($osname eq "MSWin32") {
# under NT the Perl CGI script is not run from the directory in
# which the cgi script resides (i.e., current dir is not the cgi directory)
    my($thisFilePath) = $0;
    $thisFilePath =~ s/(.*)\\[\w\-]+\.[\w\-]+/$1\\/;
    if ($thisFilePath !~ /.+\\/) {
        $thisFilePath = ".\\";   # if no path available assume current directory
    }
    # now set the include path one directory levels higher
    my($requirePath) = $thisFilePath;
    $requirePath =~ s/(.*)\\[\w\-]+\\/$1/;

    push(@INC, "$requirePath\\cgi-bin\\");
    push(@INC, "$requirePath\\code\\");
    push(@INC, "../code/");
    &chdir("$requirePath\\cgi-bin\\");
}
else { #for Unix etc. where Perl is invoked from the script's directory
    push(@INC, "../code/");
}
# --------------------------------------------------------------------------------
require "SessionUtils.pl";
require "CGIUtility.pl";

# --------------------------------------------------------------------------------
# Execute ...
#

# Read the query string
$lineread = &ReadQueryString();

# Create an associative list of (key value) pair
# of the input
%userAssocRecord = ();
&MakeAssociativeList($lineread, *userAssocRecord);

# Identify arguments that are mandatory.
@kMandatoryArguments = ("username", "db");

# List of all arguments.
@kAllArguments = ("username", "password", "db");

# Check if all the mandatory arguments are specified
@missingArguments = ();
&CheckMandatoryFields(*kMandatoryArguments, *userAssocRecord, *missingArguments);
&CheckMissingArguments(*missingArguments, $kDocErrPagePath, $kErrMissingArgumentsTemplate);

$user = $userAssocRecord{$kAllArguments[0]};
$db = $userAssocRecord{$kAllArguments[2]};
$dbpath = "";

%dbassoc = ();
%dburlassoc = ();
&ReadDatabaseNames(\%dbassoc, \%dburlassoc);
$dbpath = $dbassoc{$db};
$dburl = $dburlassoc{$db};
$result = $errNone;
if (!$dbpath || $dbpath eq "" || (!(-d $dbpath))) {
    $result = $errNoDB;
}
if ($result == $errNone) {
    # sets all globals appropriately for the given infobase ...
	&SetAllGlobals($dbpath, $dburl);
	if (-f $kBlockSystem) {
		my($templateFile) = $kDocDocPagePath . $kDocBlockSystemTemplate;
		&GenerateHTMLPage($templateFile, *gAssocList, $kDocDocPagePath);    
		exit 1;
	}

	if (open(VER, "<${kVersionFile}")) {
		my($ver) = <VER>;
		chop($ver);
		if (!&MemberOf($ver, *kProductVersionSupported)) {
			my($templateFile) = $kDocDocPagePath . $kDocSystemVersionTemplate;
			$gAssocList{"INFOBASE"} = $db;
			$gAssocList{"INFOBASEVERSION"} = $ver;
			$gAssocList{"VERSIONSUPPORTED"} = join(", ", @kProductVersionSupported);
			&GenerateHTMLPage($templateFile, *gAssocList, $kDocDocPagePath);    
			exit 1;
		}
	}
	else {
		# allow old infobases ...
		&LogMsg("Error: Failed to open version file: ${kVersionFile} : $!");
	}
}
# UMAPI.pl uses $kDocDBPath
require "UMAPI.pl";
require "UMProfileAPI.pl";

if ($result == $errNone) {
    # $result = &UMVerifyUser($user, $password);
    # instead add the user to the USERDB, if not already present
    local(@tempRecord) = ($user, "", 3, "", "", "");# 3 for type modifiable
    local(@tempProfileRecord) = ($user, $user, $user, "", "");
    $result = &UMAddUser(*tempRecord);#adds only if not present in DB
    $result = $result || &UMAddUserProfile(*tempProfileRecord);
    if($result == $errDuplicateRecord) {
        $result = $errNone; #reset if expected error
	}
}

# verify if cookie is on ....
my(%cookieJar) = ();
&GetRawCookies(\%cookieJar);
if ($cookieJar{"Test"} ne "1") {
	# cookie is not supported ...
	$templateFile = $kDocDocPagePath . $kDocCookieSupportTemplate;
	&GenerateHTMLPage($templateFile, *gAssocList, $kDocDocPagePath);
}

if ($result == $errNone) {
	# Start a session now...
	local ($sessionID);
	$result = &UMEndObsoleteSessions($user); # delete any obsolete sessions first
	$result = &UMNewSession($user, *sessionID);
	$gSessionCookie = "${kLoadedSessionIDCookie}=" . &EncodeCookieValue("${sessionID}#${dbpath}#${dburl}") .
    				  ';' ;
	# add path to cookie
	my($cookiePath) = $ENV{ 'SCRIPT_NAME' };
	if($cookiePath =~ /(.*)\/[\w\-]+\/[\w\-\.]+/) {
		$cookiePath =~ s/(.*)\/[\w\-]+\/[\w\-\.]+/$1\//; #remove cgi-bin/StartSession.pl from URL
	}
	else {
		$cookiePath = '/';
	}
	$gSessionCookie = $gSessionCookie . 'path=' . $cookiePath . ';';

	# add userid/dbname cookie of current login
	$gLoginCookie = "${kPreviousLoginCookie}=" . &EncodeCookieValue("${user}#${db}") .
		';path=' . $cookiePath . ';expires= Fri, 31-Dec-99 23:59:59 GMT;';

	$templateFile = $kDocPagePath . $kStartPage;
	$gAssocList{"DBName"} = $db;
	&GenerateHTMLPage($templateFile, *gAssocList, $kDocPagePath);
}
elsif ($result == $errOPFailed) {
      $templateFile = $kDocDocPagePath . $kDocLoginTemplate;
      local($errMsg) = "Configuration File Not found. Cannot locate Database!";
      &LogMsg($errMsg);
      $gAssocList{"BadDatabase"} =  1;
      $dbselect = &GetDBOptionList(*dbassoc, $db);
      $gAssocList{'DBSELECTIONS'} = $dbselect;
      $gAssocList{'DB'} = 1;
      $gAssocList{'USERID'} = $user;
      &GenerateHTMLPage($templateFile, *gAssocList, $kDocDocPagePath);
}
elsif ($result == $errEntryNotFound) {
	$templateFile = $kDocDocPagePath . $kDocLoginTemplate;
	local($errMsg) = "Invalid User!";
	&LogMsg($errMsg);
	$gAssocList{"BadUserName"} =  1;
	$gAssocList{"username"} =  $user;
	$dbselect = &GetDBOptionList(*dbassoc, $db);
	$gAssocList{'DBSELECTIONS'} = $dbselect;
	$gAssocList{'DB'} = 1;
	$gAssocList{'USERID'} = $user;
	&GenerateHTMLPage($templateFile, *gAssocList, $kDocDocPagePath);
}
elsif ($result == $errInvalidPassword) {
	$templateFile = $kDocDocPagePath . $kDocLoginTemplate;
	local($errMsg) = "Invalid Password!";
	&LogMsg($errMsg);
	$gAssocList{"BadPassword"} =  1;
	$dbselect = &GetDBOptionList(*dbassoc, $db);
	$gAssocList{'DBSELECTIONS'} = $dbselect;
	$gAssocList{'DB'} = 1;
	$gAssocList{'USERID'} = $user;
	&GenerateHTMLPage($templateFile, *gAssocList, $kDocDocPagePath);
}
elsif ($result == $errNoDB) {
	$templateFile = $kDocDocPagePath . $kDocLoginTemplate;
	local($errMsg) = "Invalid Database!";
	&LogMsg($errMsg);
	$gAssocList{"BadDatabase"} =  1;
	$dbselect = &GetDBOptionList(*dbassoc, $db);
	$gAssocList{'DBSELECTIONS'} = $dbselect;
	$gAssocList{'DB'} = 1;
	$gAssocList{'USERID'} = $user;
	&GenerateHTMLPage($templateFile, *gAssocList, $kDocDocPagePath);
}
exit 0;
