#!/usr/bin/perl
##*****************************************************************************
##
##     File Name:	ListCategories.pl
##         Owner:	InfoPlace
##     Copyright:	InfoPlace Inc., Pleasanton, CA, USA, 1996.
##     				Not to be copied, modified or redistributed without written
##     				permission of InfoPlace.
##
##
##*****************************************************************************
##                      R E V I S I O N   H I S T O R Y
##*****************************************************************************
##
##	  Date		Time	Author	Description
##	--------	-----	------	---------------------------------------------
##	09/15/96	13:17	N		Original version
##
##*****************************************************************************

# This cgi will list all the categories either in a hierarchical view 

# to set the current directory ...
require "pwd.pl";
require Config;

# Use osname for portability switches (doubled to cheaply avoid -w warning)
my $osname = $Config::Config{'osname'} || $Config::Config{'osname'};

if ($osname eq "MSWin32") {
# under NT the Perl CGI script is not run from the directory in
# which the cgi script resides (i.e., current dir is not the cgi directory)
    my($thisFilePath) = $0;
    $thisFilePath =~ s/(.*)\\[\w\-]+\.[\w\-]+/$1\\/;
    if ($thisFilePath !~ /.+\\/) {
        $thisFilePath = ".\\";   # if no path available assume current directory
    }
    # now set the include path one directory levels higher
    my($requirePath) = $thisFilePath;
    $requirePath =~ s/(.*)\\[\w\-]+\\/$1/;

    push(@INC, "$requirePath\\cgi-bin\\");
    push(@INC, "$requirePath\\code\\");

    &chdir("$requirePath\\cgi-bin\\");
}
else { #for Unix etc. where Perl is invoked from the script's directory
    push(@INC, "../code/");
}
# --------------------------------------------------------------------------------
require "VerifySession.pl";

require "DocAPI.pl";
require "UMProfileAPI.pl";
require "Listing.pl";

# --------------------------------------------------------------------------------
# Execute ...
#

sub GetDocumentsListing
{
    local($userID, *docsdata, *assocList) = @_;
    my($result);
    local(@db, @docs);
    $result = &DBOpen($kDocumentDB, $kDocDBPath, $kDBReadAccess, *db);
    if ($result == $errNone) {
	local(@document) = ();
	$result = &DBReadFirstRecord(*db, *document);
	while (!$result) {
	    push(@docs, &SListMake(*document, $kFieldSeperator));
	    $result = &DBReadNextRecord(*db, *document);
	}
	my($count) = $#docs + 1; 
	my($columnCount) = &PrepareDocColumnsForListing($userID, *assocList);
	if ($count) {
	    local(%newAssocList) = ();
	    local(@sortedDocs) = &SortRecordsOnField(*docs, $kDocNameIndex + 1);
	    $columnCount = &PrepareDocColumnsForListing($userID, *newAssocList);
	    my($whatsNewTime) = &UMComputeUsersWhatsNewTime($userID);
	    &AddDocumentsToTableCells(*sortedDocs, $count, 0, *newAssocList, $whatsNewTime);
	    $docsdata = $docsdata . &ProcessHTMLFromTemplate($kDocDocPagePath . "DocTable.htm", $columnCount*$count, $columnCount, *newAssocList, $kDocDocPagePath); 	
	    $assocList{'DOCTABLEEXISTS'} = 1;
	    $assocList{'NoDocuments'} = "";
	}
	else {
	    $assocList{'NoDocuments'} = $docsdata;
	}
    }
    &DBClose(*db);
}

# --------------------------------------------------------------------------------

# Read the query string
$lineread = &ReadQueryString();

# Create an associative list of (key value) pair
# of the input
%userAssocRecord = ();
&MakeAssociativeList($lineread, *userAssocRecord);

# Identify arguments that are mandatory.
@kMandatoryArguments = ();

# List of all arguments.
@kAllArguments = ("OpenCategory");

# Check if all the mandatory arguments are specified
@missingArguments = ();
&CheckMandatoryFields(*kMandatoryArguments, *userAssocRecord, *missingArguments);
&CheckMissingArguments(*missingArguments, $kDocErrPagePath, $kErrMissingArgumentsTemplate);

# First get the userid ...
$userID = $gAssocList{$kUserIDCookie};
$gAssocList{"DocViewType"} = &UMGetDocViewTypePrefFromUserID($userID);

$openCategory = $userAssocRecord{$kAllArguments[0]};
$gAssocList{"CategoryID"} = $openCategory;

$templateFile = $kDocDocPagePath . $kDocTOCTemplate;
my($indentLevel) = 0;
my($data) = "";
my(@category) = ();
my(@openCategories) = ($openCategory);
&OPGetCategory($kRootCategoryID, \@category);
my($children) = $category[$kCategoryChildListIndex];
my($count) = &SListLength($children, $kValueSeperator);
my(@categoryList) = &SListUnmake($children, $kValueSeperator);
for ($i = 0; $i < $count; $i++) {
	my($element) = $categoryList[$i];
	&OPGetCategory($element, \@category);
	$categoryList[$i] = &SListMake(\@category, $kFieldSeperator);
}
@categoryList = &SortRecordsOnField(\@categoryList, 2);
for ($i = 0; $i < $count; $i++) {
	my($element) = &SListNthElement($categoryList[$i], 0, $kFieldSeperator);
	&RecurseThruCategories($element, $indentLevel, \$data, \@openCategories, $userID, *gAssocList);
}
if ($count == 0) {
	$data = $data . "There are no categories or documents in this Infobase.";
}
if (!$gAssocList{'DOCTABLEEXISTS'}) {
	$gAssocList{'NODOCTABLEEXISTS'} = 1;
}
$gAssocList{"Body"} = $data;
&GenerateHTMLPage($templateFile, *gAssocList, $kDocDocPagePath);

exit 0;
