#!/usr/bin/perl
##*****************************************************************************
##
##     File Name:	WhatsNew.pl
##         Owner:	InfoPlace
##     Copyright:	InfoPlace Inc., Pleasanton, CA, USA, 1996.
##     			Not to be copied, modified or redistributed without written
##     			permission of InfoPlace.
##
##
##*****************************************************************************
##                      R E V I S I O N   H I S T O R Y
##*****************************************************************************
##
##	  Date		Time	Author	Description
##	--------	-----	------	---------------------------------------------
##	11/15/96	13:26	N		Original version
##
##*****************************************************************************



# to set the current directory ...
require "pwd.pl";
require Config;

# Use osname for portability switches (doubled to cheaply avoid -w warning)
my $osname = $Config::Config{'osname'} || $Config::Config{'osname'};

if ($osname eq "MSWin32") {
# under NT the Perl CGI script is not run from the directory in
# which the cgi script resides (i.e., current dir is not the cgi directory)
    my($thisFilePath) = $0;
    $thisFilePath =~ s/(.*)\\[\w\-]+\.[\w\-]+/$1\\/;
    if ($thisFilePath !~ /.+\\/) {
        $thisFilePath = ".\\";   # if no path available assume current directory
    }
    # now set the include path one directory levels higher
    my($requirePath) = $thisFilePath;
    $requirePath =~ s/(.*)\\[\w\-]+\\/$1/;

    push(@INC, "$requirePath\\cgi-bin\\");
    push(@INC, "$requirePath\\code\\");

    &chdir("$requirePath\\cgi-bin\\");
}
else { #for Unix etc. where Perl is invoked from the script's directory
    push(@INC, "../code/");
}
# --------------------------------------------------------------------------------
require "VerifySession.pl";

require "UMProfileAPI.pl";
require "Listing.pl";

# --------------------------------------------------------------------------------
# Function
#

sub SortRecordDesc
{
	# $a and $b are pre-defined variables for sort, a pre-built function.
	my($aa, $bb);
	$aa = &SListNthElement($a, $fieldIndex - 1, $kFieldSeperator);
	$bb = &SListNthElement($b, $fieldIndex - 1, $kFieldSeperator);
	$aa =~ tr/A-Z/a-z/;
	$bb =~ tr/A-Z/a-z/;
	my($mynull) = 0;
	return ($aa > $bb) ? -1 : (($aa < $bb) ? 1 : $mynull);
}

sub SortRecordsOnFieldDesc
{
	local(*categoryList, $fieldIndex) = @_;
	return sort SortRecordDesc @categoryList;
}

# --------------------------------------------------------------------------------
# Execute ...
#

$lineread = &ReadQueryString();

# Create an associative list of (key value) pair
# of the input
%userAssocRecord = ();
&MakeAssociativeList($lineread, *userAssocRecord);

# Identify arguments that are mandatory.
@kMandatoryArguments = ();

# List of all arguments.
@kAllArguments = ("whatsnew");

# Check if all the mandatory arguments are specified
@missingArguments = ();
&CheckMandatoryFields(*kMandatoryArguments, *userAssocRecord, *missingArguments);
&CheckMissingArguments(*missingArguments, $kDocErrPagePath, $kErrMissingArgumentsTemplate);

$whatsnewdays = $userAssocRecord{$kAllArguments[0]};

$templateFile = $kDocDocPagePath . $kDocWhatsNewTemplate;

# First get the whats' new time (in hours) from the user profile
$userID = $gAssocList{$kUserIDCookie};
$gAssocList{"DocViewType"} = &UMGetDocViewTypePrefFromUserID($userID);
my($timeSince) = defined($whatsnewdays) ? &UMComputeUsersWhatsNewTime($userID, $whatsnewdays*24) : &UMComputeUsersWhatsNewTime($userID);
$gAssocList{"SinceWhatsNew"} = &GetTimeFormatted($timeSince);

# Now, iterate thru the docs for the right ones
local(@newDocs) = ();
local(@modDocs) = ();

my($result);
local(@db) = ();
$result = &DBOpen($kDocumentDB, $kDocDBPath, $kDBReadAccess, *db);
if ($result == $errNone) {
    local(@document) = ();
    $result = &DBReadFirstRecord(*db, *document);
    while (!$result) {
		my($modTime) = $document[$kDocModificationTimeIndex];
		my($newTime) = $document[$kDocCreationTimeIndex];
		if ($newTime >= $timeSince) {
			push(@newDocs, &SListMake(*document, $kFieldSeperator));
		}
		elsif ($modTime >= $timeSince) {
			push(@modDocs, &SListMake(*document, $kFieldSeperator));
		}
		$result = &DBReadNextRecord(*db, *document);
    }
    my($newCount) = $#newDocs + 1;
    my($modCount) = $#modDocs + 1; 
    my($columnCount) = &PrepareDocColumnsForListing($userID, *gAssocList);
    local($totalCount) = $modCount + $newCount;
    if ($newCount) {
		local(%newAssocList) = ();
		local(@sortedNewDocs) = &SortRecordsOnFieldDesc(*newDocs, $kDocCreationTimeIndex + 1);
		$columnCount = &PrepareDocColumnsForListing($userID, *newAssocList);
		&AddDocumentsToTableCells(*sortedNewDocs, $newCount, 0, *newAssocList, 0);
		$gAssocList{"NEWDOCS"} = &ProcessHTMLFromTemplate($kDocDocPagePath . "DocTable.htm", $columnCount*$newCount, $columnCount, *newAssocList, $kDocDocPagePath); 	
    }
    if ($modCount) {
		local(%modAssocList) = ();
		local(@sortedModDocs) = &SortRecordsOnFieldDesc(*modDocs, $kDocModificationTimeIndex + 1);
		$columnCount = &PrepareDocColumnsForListing($userID, *modAssocList);
		&AddDocumentsToTableCells(*sortedModDocs, $modCount, 0, *modAssocList, 0);
		$gAssocList{"MODDOCS"} = &ProcessHTMLFromTemplate($kDocDocPagePath . "DocTable.htm", $columnCount*$modCount, $columnCount, *modAssocList, $kDocDocPagePath); 
    }
    $gAssocList{"CategoryParent"} = "";
    $gAssocList{"DocumentParent"} = "";
    if ($totalCount) {
		$gAssocList{"NoDocuments"} = "";
     	$gAssocList{"TABLEEXISTS"} = 1;	
    }
    else {
		$gAssocList{"NoDocuments"} = "Nothing has changed";
     	$gAssocList{"TABLEEXISTS"} = 0;
    }
	$gAssocList{"CategoryName"} = "";
    &GenerateHTMLPage($templateFile, *gAssocList, $kDocDocPagePath);
}
else {
    local($errPage) = $kDocErrPagePath . $kErrInternalErrorTemplate;	
    $gAssocList{"Body"} = "Document DB open failed.";
    &GenerateHTMLPage($errPage, *gAssocList, $kDocErrPagePath);
}
&DBClose(*db);
exit 0;
