#!/usr/bin/perl
##*****************************************************************************
##
##     File Name:	FindDocument.pl
##         Owner:	InfoPlace
##     Copyright:	InfoPlace Inc., Pleasanton, CA, USA, 1996.
##     				Not to be copied, modified or redistributed without written
##     				permission of InfoPlace.
##
##
##*****************************************************************************
##                      R E V I S I O N   H I S T O R Y
##*****************************************************************************
##
##	  Date		Time	Author	Description
##	--------	-----	------	---------------------------------------------
##	09/15/96	13:07	N		What you are doing...
##	09/15/96	13:05	N		Original version
##
##*****************************************************************************

# This will be invoked from the Find form for documents.
# This cgi will take the criteria and display the resultant records.


# to set the current directory ...
require "pwd.pl";
require Config;

# Use osname for portability switches (doubled to cheaply avoid -w warning)
my $osname = $Config::Config{'osname'} || $Config::Config{'osname'};

if ($osname eq "MSWin32") {
# under NT the Perl CGI script is not run from the directory in
# which the cgi script resides (i.e., current dir is not the cgi directory)
    my($thisFilePath) = $0;
    $thisFilePath =~ s/(.*)\\[\w\-]+\.[\w\-]+/$1\\/;
    if ($thisFilePath !~ /.+\\/) {
        $thisFilePath = ".\\";   # if no path available assume current directory
    }
    # now set the include path one directory levels higher
    my($requirePath) = $thisFilePath;
    $requirePath =~ s/(.*)\\[\w\-]+\\/$1/;

    push(@INC, "$requirePath\\cgi-bin\\");
    push(@INC, "$requirePath\\code\\");

    &chdir("$requirePath\\cgi-bin\\");
}
else { #for Unix etc. where Perl is invoked from the script's directory
    push(@INC, "../code/");
}
# --------------------------------------------------------------------------------
require "VerifySession.pl";

require "DocAPI.pl";
require "OPFind.pl";
require "Listing.pl";
require "UMProfileAPI.pl";
require "ShowDocuments.pl";

#------------------------------------------------------------
# List result of search ...
sub ShowFindResult
{
    local(*assocList, *documents, $userID) = @_;
    # set up the template directory
	my($templatePath) = $kIBAnonymousTemplatePath;
	$templatePath = $kAnonymousTemplatePath if (!(-d $templatePath)); 
	$templateFile = $templatePath . 'XU6.htm';

    my($totalCells, $numCellsPerRow) = (0, 1);
    my($result);

   if ($#documents +1 == 0) {
		$assocList{"Result"} = "No documents matched the given search criteria.";
		$assocList{"NoDocuments"} = "";
		$assocList{"TABLEEXISTS"} = 0;
    }
    else {
		$assocList{"Result"} = "";
		# Look at user profile to determine columns
		local ($columns) = "";
		$result = &UMGetDocColumnsFromProfile($userID, *columns);
		if ($columns eq "") {
			$columns = $kDefaultDocColumnNames;
		}
		my(@columnList) = split("\,", $columns);
		foreach $c (@columnList) {
			$assocList{"COLUMN" . $c} = 1;
		}
		$numCellsPerRow = $#columnList + 1;
		my($docList) = join($kValueSeperator, @documents);
		&GenDocTableCells($docList, *assocList, "");
		$totalCells = $numCellsPerRow * ($#documents +1);
		$assocList{"Result"} = ($#documents + 1) . " documents found.";
		$assocList{"NoDocuments"} = "";
		$assocList{"TABLEEXISTS"} = 1;
    }
    &GenerateHTMLFromTemplate($templateFile, $totalCells, $numCellsPerRow, *assocList, $templatePath);
}

# --------------------------------------------------------------------------------
# Execute ...
#

# Read from stdin
$lineread = &ReadQueryString();

# Create an associative list of (key value) pair
# of the input
%userAssocRecord = ();
&MakeAssociativeList($lineread, *userAssocRecord);

# Identify arguments that are mandatory.
@kMandatoryArguments = ("text");

# List of all arguments.
@kAllArguments = ("text");

# Check if all the mandatory arguments are specified
@missingArguments = ();
&CheckMandatoryFields(*kMandatoryArguments, *userAssocRecord, *missingArguments);
&CheckMissingArguments(*missingArguments, $kDocErrPagePath, $kErrMissingArgumentsTemplate);

$text = $userAssocRecord{$kAllArguments[0]};
@documents = ();
@document = ();
$document[$kDocNameIndex] = $text;
$document[$kDocAuthorIndex] = $text;
$document[$kDocKeywordsIndex] = $text;
$document[$kDocDescriptionIndex] = $text;

@criteria = ();
$criteria[$kDocNameIndex] = &GetStringCriteria("Contains");
$criteria[$kDocAuthorIndex] = &GetStringCriteria("Contains");
$criteria[$kDocKeywordsIndex] = &GetStringCriteria("Contains");
$criteria[$kDocDescriptionIndex] = &GetStringCriteria("Contains");

$gAssocList{"SearchCriteria"} = $text;

my($userID) = $gAssocList{$kUserIDCookie};
$gAssocList{"DocViewType"} = &UMGetDocViewTypePrefFromUserID($userID);

$gAssocList{"DocumentParent"} = "";
$result = &OPFindDocument(*document, *criteria, *documents, 1);
if ($result == 0) {
    &ShowFindResult(*gAssocList, *documents, $userID);
}
else {
    my($errPage) = $kDocErrPagePath . $kErrInternalErrorTemplate;
    $gAssocList{"Body"} = "Find Document failed!";
    &GenerateHTMLPage($errPage, *gAssocList, $kDocErrPagePath);
}

exit 0;
