#
#    Last change:  NAG   8 Jan 98   11:47 am
##*****************************************************************************
##
##     File Name:	CGIUtility.pl
##         Owner:	InfoPlace
##     Copyright:	InfoPlace Inc., Pleasanton, CA, USA, 1996.
##     				Not to be copied, modified or redistributed without written
##     				permission of InfoPlace.
##
##
##*****************************************************************************
##                      R E V I S I O N   H I S T O R Y
##*****************************************************************************
##
##	  Date		Time	Author	Description
##	--------	-----	------	---------------------------------------------
##	09/15/96	12:50	N		Original version
##
##*****************************************************************************
require "FileLock.pl";
require "LogMsg.pl";

$kTemplatePrefix = '\[';
$kTemplatePostfix = '\]';
$kNoEscTemplatePrefix = '[';
$kNoEscTemplatePostfix = ']';


# --------------------------------------------------------------------------------
# EncryptString( $str )
# Given a string, does simple one way encryption and returns encrypted string.
# Args:
#	$password 		(string): the password of the user.
#							
#	Return Value(string):  encrypted string

sub EncryptString {
	my($str) = @_;
	my($key) = "docorg"; #key for encryption
	my($cryptedstr) = &ipcrypt($str, $key);
	$cryptedstr =~ s/\~\#\~/*IPDBDELIM*/g; # remove '~#~' substrings
	$cryptedstr =~ s/\n//g; #remove new lines
	return $cryptedstr;
}

sub ipcrypt {
	my($str, $key) = @_;
	my($numericstr)= 0;
	my(@strarr) = unpack('C*', $str);
	for($i=0; $i <= $#strarr; $i++) {
		$numericstr = ($numericstr * 10) + $strarr[$i];
	}
	my($numerickey) = 0;
	my(@keyarr) = unpack('C*', $key);
	for($i=0; $i <= $#keyarr; $i++) {
		$numerickey = ($numerickey * 10) + $keyarr[$i];
	}
	return pack('u*', ($numericstr*(($numerickey+$numericstr)%1000))); #uuencoded string
}#ipcrypt
# --------------------------------------------------------------------------------
sub SortText
{
    # $a and $b are pre-defined variables for sort, a pre-built function.
    my($aa, $bb) = ($a, $b);
    $aa =~ tr/A-Z/a-z/;
    $bb =~ tr/A-Z/a-z/;
    return ($aa cmp $bb);
}

sub MemberOf
{
    local($element, *list) = @_;
    foreach $x (@list) {
	if ($element eq $x) {
	    return 1;
	}
    }
    return 0;
}

sub ReadQueryString {
	my($query);
	my($requestMethod) = $ENV{ 'REQUEST_METHOD' };

	if ($requestMethod eq 'GET') {
		my($queryString) = $ENV{ 'QUERY_STRING' };
		$query = $queryString;
	}
	elsif (($requestMethod eq 'POST') || ($requestMethod eq 'PUT')) {
		my($contentLength) = $ENV{ 'CONTENT_LENGTH' };
		# read from STDIN
		read(STDIN, $query, $contentLength);
	}
	else { #error condition
		 # just read from stdin, works out for debugging
		 $query = <STDIN>;
		 # &TerminateExpansion("Unknown Method: $requestMethod");
	}
	return $query;
}

@kMonthName = ("Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec");
# get time is the following format "MMM DD, YYYY HH:MM"
sub GetTimeFormatted
{
	my($time) = @_;
	my(@t) = localtime($time);
	return sprintf('%s %02d, %d %02d:%02d', $kMonthName[$t[4]], $t[3], $t[5], $t[2], $t[1]);
}

# Merge second assoc-list into the first one
sub MergeAssocList
{
	local(*primary, *secondary) = @_;
	my(@pair);
	while ((@pair = each(%secondary)) != ()) {
		$primary{$pair[0]} = $pair[1];
	}
}

# Given an array of keyword and an array of values, returns an assoc-list.
sub GenerateAssocList
{
	local(*keys, *values) = @_;
	my($count) = $#keys + 1;
	my($i);
	my(@result) = ();
	for ($i = 0; $i < $count; $i++) {
		push(@result, $keys[$i]);
		push(@result, $values[$i]);
	}
	return @result;
}

# Given an array of keyword and an array of values, fill-in the assoc-list.
sub FillInAssocList
{
	local(*keys, *values, *assocList) = @_;
	my($count) = $#keys + 1;
	my($i);
	for ($i = 0; $i < $count; $i++) {
		$assocList{$keys[$i]} = $values[$i];
	}
}


# Utility function to terminate template expansion on error.
sub TerminateExpansion
{
	my($msg) = @_;
	&OutputHTMLHeader("Error");
	&OutputHTML($msg);
	&TerminateCGI();
}

# Read the given file completely and return the string, if the file could
# not be found or read, returns undefined value
sub ReadFileIntoString
{
	my($file) = @_;
	my($data);
	if (!open(XFILE, "<$file")) {
		&LogFileOpenFailMsg($file);
		return $data;
	}
	local($/);
	$data = <XFILE>;
	close(XFILE);
	return $data;
}

sub WriteStringIntoFile
{
	my($data, $file) = @_;
	if (!open(XFILE, ">$file")) {
		&LogFileOpenFailMsg($file);
	}
	print XFILE $data;
	close(XFILE);
}

sub ProcessIncludes
{
    my($data, $path) = @_;
    my($count)  = 0;
    my($kDeep) = 10;
    while ($data =~ s/$kTemplatePrefix#include\s*"(.*)"\s*$kTemplatePostfix/&ReadFileIntoString($path . $1)/ge) {
	   $count++;
	   if ($count > $kDeep) {
	       &LogMsg("ProcessInclude $kDeep for file: ${path}$1, possible recursion.");
	       return $data;
	   }
    }
    return $data;
}

sub EncodeForURL
{
	my($argStr) = @_;
	$argStr =~s/([^a-zA-Z0-9_.-])/uc sprintf("%%%02x",ord($1))/eg;
	return $argStr;
}

sub CheckCondition
{
  local($condition, $match, *assocList) = @_;
  my($value) = $assocList{$condition};
  if (defined($value)) {
    if ($value) {
		return $match;
	}
  }
  return "";
}

sub ProcessConditionals
{
  local($data, *assocList) = @_;
  my($name, @conditionList, $preBody, $body, $rest, $resultbody, $resultdata);

  if($data =~ /(.*?)${kTemplatePrefix}IF-TRUE\s*(\w+)\s*$kTemplatePostfix(.*)/s) {
	$preBody = $1; $name = $2; $rest = $3;
	if($rest =~ /(.*?)${kTemplatePrefix}\/IF-TRUE\s*$name\s*$kTemplatePostfix(.*)/s) {
	  $body = $1; $rest = $2;
	  if (&CheckCondition($name, 1, *assocList)) {
		$resultbody = &ProcessConditionals($body, *assocList);
	  }#if checkcondition
	  if ($rest) {
		$resultdata = &ProcessConditionals($rest, *assocList);
	  }
	  return $preBody . $resultbody . $resultdata;
	}
  }
  return $data;
}


# The following two routines process Option Templates in Select HTML blocks.
# syntax: <select..> [OptionTemplate tagname] ..  [/OptionTemplate tagname]
# the number of options is specified by the assoclist{tagname} entry.
# Any "Option#" sting within the template is substitued by the Option number starting at 0.

sub AddOptionEntries {
	local($name, $optionString,*assocList) = @_;
	my($numOptions) = $assocList{$name};
	my($i, $optionlist) = (0, '');
	for ($i = 0; $i < $numOptions; $i++) {
		my($curOption) = $optionString;
		$curOption =~ s/Option\#/"${i}"/ieg; 
		$optionlist = $optionlist . $curOption;
	}
	return $optionlist;
}

sub ProcessOptionTemplates
{
  local($data, *assocList) = @_;



  while ($data =~ /${kTemplatePrefix}OptionTemplate\s*(.*)\s*$kTemplatePostfix/g) {
    push(@conditionList, $1);
  }
  $name = pop(@conditionList);
  while  ($name)  {
    $data =~ s/${kTemplatePrefix}OptionTemplate\s*$name\s*$kTemplatePostfix(.|\n)*?$kTemplatePrefix\/OptionTemplate\s*$name\s*$kTemplatePostfix/&AddOptionEntries($name, $1,*assocList)/ge;
    $name = pop(@conditionList);
  }
  return $data;
}

# General function for generating a HTML page with unlimited "substitutions"
sub GenerateHTMLPage {
	local($fileName, *assocList, $includePath) = @_;
	my($page) = &GetGenerateHTMLPage($fileName, *assocList, $includePath);

	# write out the html file ...
	$eol = "\015\012";
	print "Content-Type: text/html\n";

	if ($gSessionCookie) {
		print "Set-Cookie: " , $gSessionCookie,  $eol;
	}
	if ($gLoginCookie) {
		print "Set-Cookie: " , $gLoginCookie,  $eol;
	}
	print "\n\n";
	print $page;
	exit 0;
}

# 
sub OutputHTMLPage {
	my($page) = @_;

	# write out the html ...
	$eol = "\015\012";
	print "Content-Type: text/html\n";

	if ($gSessionCookie) {
		print "Set-Cookie: " , $gSessionCookie,  $eol;
	}
	if ($gLoginCookie) {
		print "Set-Cookie: " , $gLoginCookie,  $eol;
	}
	print "\n\n";
	print $page;
	exit 0;
}

# 
sub GetGenerateHTMLPage {
	local($fileName, *assocList, $includePath) = @_;
	my($page) = &ReadFileIntoString($fileName);
	if (!defined($page)) {
		&TerminateExpansion("Could not open HTML page \"$fileName\": $! \n");
	}
	$page = &ProcessIncludes($page, $includePath);
	$page = &ProcessConditionals($page, *assocList);
	$page = &ProcessOptionTemplates($page, *assocList);
	
	# do the replacements ...
	$page =~  s/$kTemplatePrefix(\w+?)$kTemplatePostfix/(defined($assocList{$1}) ? $assocList{$1}: "$kNoEscTemplatePrefix$1$kNoEscTemplatePostfix")/ieg;

	return $page;
}

sub AddWithGivenBase
{
	my($x, $base) = @_;
	$x++;
	if ($x == $base) {
		$x = 0;
	}
	return $x;
}

sub ProcessTableTemplate
{
    local($page, $totalCells, $numCellsPerRow, *assocList, $doCellSubstitutions) = @_;
    # set up cells for Table Header Row, if needed:
    if ($page =~ /.*\<HEADCELLTEMPLATE\>((.|\n)*)\<\/HEADCELLTEMPLATE\>.*/) {
	my($headerCellStr) = $1;
	my($cell, $headerRowStr) = (0, "");
      innerLoop: 
	for ($cell = 0; $cell < $numCellsPerRow; $cell++) {
	    my($newCellStr) = $headerCellStr;
	    last innerLoop if($cell == $totalCells); # break out of inner for loop
	    $newCellStr =~ s/Cell\#/"${cell}"/ieg;
	    $headerRowStr = $headerRowStr . $newCellStr;
	}
	#now put the header string into the page stream
	$page =~ s/.*\<HEADCELLTEMPLATE\>((.|\n)*)\<\/HEADCELLTEMPLATE\>.*/$headerRowStr/ie;	
    }
    # set up cells per row:
    my($rowTemplateStr, $rowStrPrefix, $cellStr, $rowStrSuffix);
    if ($page =~ /.*\<ROWTEMPLATE\>((.|\n)*)\<\/ROWTEMPLATE\>.*/) {
	$rowTemplateStr = $1;
	if ($rowTemplateStr =~ /((.|\n)*)\<CELLTEMPLATE\>((.|\n)*)\<\/CELLTEMPLATE\>((.|\n)*)/) {
	    $rowStrPrefix = $1;
	    $cellStr = $3 ;
	    $rowStrSuffix = $5;
	}
	else {
	    $rowStrPrefix = $rowTemplateStr;
	    $cellStr = '' ;
	    $rowStrSuffix = '';
	}
    }
    my($tableStr, $rowStr, $row, $cell);
    $tableStr = ""; $rowStr = "";
    my($numRows) = $totalCells/$numCellsPerRow; 
    my($rowColorIndex) = 0;
    my($rowColor);
#    my($darkColor) = "bfbfbf";
#    my($lightColor) = "d2d2d2";
    my($darkColor) = "dfdfdf";
    my($lightColor) = "eeeeee";
    for ($row = 0; $row < $numRows; $row++) {
	if ($cellStr) {
	  innerLoop: 
	    for ($cell = 0; $cell < $numCellsPerRow; $cell++) {
		my($newCellStr) = $cellStr;
		my($newCell) = ($row * $numCellsPerRow) + $cell; #index of the cell in the table
		last innerLoop if($newCell == $totalCells); # break out of inner for loop
		$newCellStr =~ s/Cell\#/"${newCell}"/ieg;
		$rowStr = $rowStr . $newCellStr;
	    }
	}
	my($tempRowStrPrefix) = $rowStrPrefix;
	my($tempRowStrSuffix) = $rowStrSuffix;
	$rowColor = ($rowColorIndex) ? $darkColor : $lightColor;
	$tempRowStrPrefix =~ s/Row\#/"Row${row}"/ieg;
	$tempRowStrPrefix =~ s/${kTemplatePrefix}Color$kTemplatePostfix/$rowColor/g;
        $tempRowStrSuffix =~ s/Row\#/"Row${row}"/ieg;
        $tableStr = $tableStr . $tempRowStrPrefix . $rowStr . $tempRowStrSuffix;
        $rowStr = "";
        $rowColorIndex = &AddWithGivenBase($rowColorIndex, 2);
    }
    # now put the table string into the page stream
    $page =~ s/.*\<ROWTEMPLATE\>((.|\n)*)\<\/ROWTEMPLATE\>.*/$tableStr/ie;

    if ($doCellSubstitutions) {
       # do all replacements for cell values
#       my(@valuePair) = each(%assocList);
	$page =~  s/$kTemplatePrefix(\w+?)$kTemplatePostfix/(defined($assocList{$1}) ? $assocList{$1}: "$kNoEscTemplatePrefix$1$kNoEscTemplatePostfix")/ieg;
#       while ($#valuePair >= 0) {
#           $page =~  s/$kTemplatePrefix$valuePair[0]$kTemplatePostfix/$valuePair[1]/ieg;
#       	   @valuePair = each(%assocList);
#       }
   }
   return $page;
}


sub ProcessHTMLFromTemplate {
    local($fileName, $totalCells, $numCellsPerRow, *assocList, $includePath) = @_;
    my($found) = 0;
    my($page) = &ReadFileIntoString($fileName);
    if (!defined($page)) {
	&TerminateExpansion("Could not open HTML page $fileName: $! \n");
    }
    $page = &ProcessIncludes($page, $includePath);
    $page = &ProcessConditionals($page, *assocList);
    $page = &ProcessOptionTemplates($page, *assocList);
    $page = &ProcessTableTemplate($page, $totalCells, $numCellsPerRow, *assocList, 0);
    
    # do all replacements for normal values
#    my(@valuePair) = each(%assocList);
	$page =~  s/$kTemplatePrefix(\w+?)$kTemplatePostfix/(defined($assocList{$1}) ? $assocList{$1}: "$kNoEscTemplatePrefix$1$kNoEscTemplatePostfix")/ieg;
#    while ($#valuePair >= 0) {
#        $page =~  s/$kTemplatePrefix$valuePair[0]$kTemplatePostfix/$valuePair[1]/ieg;
#	@valuePair = each(%assocList);
#    }
    return $page;
}

# A function for generating a HTML page from a template page w/ Table
# The number of rows($numRows) and cells per row($numCellsPerRow) are
# specified as parameters.
# Each cell may contain strings in the form [substituteKey]. This function also
# substitutes these keys with values specified in the last parameter.
# The @assocList is used to specify the values with keywords

sub GenerateHTMLFromTemplate {
    local($fileName, $totalCells, $numCellsPerRow, *assocList, $includePath) = @_;
    my($page) = &ProcessHTMLFromTemplate($fileName, $totalCells, $numCellsPerRow, *assocList, $includePath);

    # write out the html file ...
    $eol = "\015\012";
    print "Content-Type: text/html\n";

	if ($gSessionCookie) {
		print "Set-Cookie: " , $gSessionCookie,  $eol;
	}
	if ($gLoginCookie) {
		print "Set-Cookie: " , $gLoginCookie,  $eol;
	}
    print "\n\n";
    print $page;
    exit 0;
}


# Used for parsing the input line from user to CGI scripts
sub MakeAssociativeList
{
	local($line, *assoc) =@_;
	my(@record) = split(/&/, $line);
	foreach $field (@record) {
		my(@pair) = split(/=/, $field);
		if (!($pair[1] eq "")) {
			$pair[0] =~ y/\+/ /; # replace all + by SPACE
			$pair[0] =~ s/%([\da-f]{1,2})/pack(C, hex($1))/eig;
			$pair[1] =~ y/\+/ /; # replace all + by SPACE
			$pair[1] =~ s/%([\da-f]{1,2})/pack(C, hex($1))/eig;
			$pair[1] = join(' ', split(' ', $pair[1])); # remove whitespaces
			$pair[1] =~ s/\n/ /g; # replace newlines by space
			#   $pair[1] =~ s/~/^/g; # replace '~' by '^', as '~' is field separator
			#if (length($pair[1]) > 255) {
			#   $pair[1] = substr($pair[1], 0, 255); # truncate strings to 255 characters
		    #}
			#print "pair[0] =",$pair[0], " pair[1] = ", $pair[1], " \n";
			if ($assoc{$pair[0]} eq "") {
				$assoc{$pair[0]} = $pair[1];
			}
			else { #for multi valued Select options: make it comma seperated list
				$assoc{$pair[0]} = $assoc{$pair[0]} . "," . $pair[1];
			}
		}
	}
	return 0;
}

sub CheckMandatoryFields {
	local(*mandatoryFields, *fieldAssoc, *missingFields) = @_;
	my($index) = 0;
	my($mField);
	foreach $mField (@mandatoryFields) {
		if( !defined($fieldAssoc{$mField}) ) {
			$missingFields[$index++] = $mField;
		}
	}
}

sub CheckMissingArguments
{
	local(*missingArguments, $path, $template) = @_;
	if ( $#missingArguments + 1) { #non-zero
		my($args) = join(',', @missingArguments);
		my($errPage) = $path . $template;
		local(%assocList) = ("Body", $args);
		&GenerateHTMLPage($errPage, *assocList, $path);
		exit 1;
	}
}

sub UserErrorDisplay {
	local($errString, *valueList) = @_;
	my($valueStr) = join(',', @valueList);
	&OutputHTML("<H2>Error: ${errString} $valueStr</H2>");
	&TerminateCGI();
}

sub OutputHTMLHeader
{
	my($title) = @_;
	# Head part
	print "Content-Type: text/html\n\n<HTML>\n\n<HEAD> <TITLE> $title </TITLE> </HEAD> 
			<BODY>
			<p>";
}

sub OutputHTML
{
	my ($body) = @_;
	# Action part
	print $body;
}

sub OutputHTMLTail
{
	my($tail) = @_;
	# Tail part
	print "${tail}<p>
			</BODY> </HTML>";
}

sub TerminateCGI
{
   &OutputHTMLTail();
   exit 0;
}

sub OutputHTMLMsg
{
    my($title, $msg) = @_;
    &OutputHTMLHeader($title);
    print "<B>$msg</B>\n";
    &OutputHTMLTail("");
}

# returns the day and date for yesterday "Wed April 24" etc.
#sub GetYesterdayStr {
#	require "ctime.pl";
#	my($timeStr) = time - (1*(24*60*60)); #substract one day for yesterday
#	my($dayStr) =  &ctime($timeStr), "\n";
#	print $dayStr, "\n";
#	$dayStr = substr($dayStr, 0, 10); # now truncate to just day/date
#       print $dayStr, "\n";

#    return &dayStr;
#}

# The following will return 0 if valid format for e-mail is provided as input
# If E-mail string is not of valid form, return value is 1.
sub VerifyEmailFormat {
	my($eMailStr) = @_;
	return 0; #always return Okay
}

sub RedirectURL
{
	my($url) = @_;
	print "Location: ${url}\n\n";
}

sub GenerateSessionID
{
    my($seed) = @_;
    my($kConst1) = time();
    my($kConst2) = $$;
    my($id) = sprintf "%012u%012u", $kConst1 + $kConst1 / $seed, $kConst2 * $seed;
    return $id;
}

1;
