#!/share/perl5/bin/perl
##*****************************************************************************
##
##     File Name:	CreateInfobase.pl
##         Owner:	InfoPlace
##     Copyright:	InfoPlace Inc., Milpitas, CA, USA, 1998.
##     			Not to be copied, modified or redistributed without written
##     			permission of InfoPlace.
##
##
##*****************************************************************************
##                      R E V I S I O N   H I S T O R Y
##*****************************************************************************
##
##	  Date		Time	Author	Description
##	--------	-----	------	---------------------------------------------
##	07/14/98	13:17	N		Original version
##
##*****************************************************************************


require "DocDB.pl";
require "DocUtils.pl";

# --------------------------------------------------------------------------------
# Write version file
sub WriteVersionFile
{
	if (!open(VERSIONFILE, ">$kVersionFile")) {
		&LogFileOpenFailMsg($kVersionFile);
		&LogMsg("Version file could be opened !!!");
	}
	else {
		&GetWriteLockOnFile("VERSIONFILE") || &LogMsg("WriteVersionFile: failed on write lock on file $kVersionFile: $!");
		print VERSIONFILE "${kProductVersion}\n${kProductFullVersion}\n";
		close(VERSIONFILE);
		&UnlockFile("VERSIONFILE") || &LogMsg("WriteVersionFile: failed on unlock on file $kVersionFile: $!");
	}	
}

# --------------------------------------------------------------------------------
# Write version file
sub CreateInfobase
{
	my($ibname, $ibdesc, $ibpath, $iburl) = @_;
	if (!&IsNameInProperCharSet($ibname)) {
		$gAssocList{"NAMENOTINCHARSET"} = 1;
		$gAssocList{"AddInfobase"} = $ibname;
		return 2;
	}
	if (!$ibpath) {
		$ibpath = $kDefaultBaseIBPath;
	}
	if (!$iburl) {
		$iburl = $kDefaultBaseIBURL;
	}
	$msg = "";
	local(%dbassoc) = ();
	local(%dburlassoc) = ();
	&ReadDatabaseNames(\%dbassoc, \%dburlassoc);
	my(@keys) = keys %dbassoc;

	my($numDB) = &ReadLastDBDirIndex() + 1; 
	my($dbDirName) = "DB" . $numDB;
	my($completePath) = $ibpath . $dbDirName;

	if (($result == $errNone) && ($dbassoc{$ibname})) {
		$result = 1;
		$gAssocList{"errDuplicateInfobase"} = 1;
		$gAssocList{"ADDERROR"} = 1;
	}

	if ($result == $errNone) { #if no duplicate exists
		while (-d $completePath) { #if directory exists bump the number
			$numDB++;
			$dbDirName = "DB" . $numDB;
			$completePath = $ibpath . $dbDirName;
		}
		
		$result = 1;
		umask 0000;
		if (mkdir($completePath, $kDBDefaultPermission) == 1) { # success
			$result = $errNone;
		}
		else {
			$gAssocList{"ADDERROR"} = 1;
			$gAssocList{"errPermission"} = 1;
		}
		if ($result == $errNone) {
			mkdir("${completePath}/db", $kDBDefaultPermission);
			mkdir("${completePath}/docs", $kDBDefaultPermission);
			my($url) = "${iburl}${dbDirName}/";
			my($db) = "${completePath}/";
			
			# set all globals to point to new infobase ...
			&SetAllGlobals($db, $url);
			
			$result = &DefineAllDBs();
			if ($result == $errNone) {
				# write defaults config file ...
				&WriteInfobaseConfigFile();
				# write version file ...
				&WriteVersionFile();

				my(%assoc) = ();
				# add to the InfoBases config file
				&ReadSpecifiedConfigFile($kInfobasesConfigFile, \%assoc);
				$assoc{"lastindex"} = $numDB;
				#append directory if it is not in a default location
				if($completePath !~ /^$kDefaultBaseIBPath/) {
					$assoc{"directory" . $numDB} = $db;
					$assoc{"url" . $numDB} = $url;
				}#if
				&WriteSpecifiedConfigFile($kInfobasesConfigFile, \%assoc);
				# add name, description to the infobase.config file
				%assoc = ();
				$assoc{"name"} = $ibname;
				$assoc{"description"} = $ibdesc;
				&WriteSpecifiedConfigFile($kDocDBRootPath . $kIBInfobaseFileName, \%assoc);
			}#if DBs created
		}#if mkdir succeeded
	}#if no duplicates
	return $result;
}

