#!/usr/bin/perl
##*****************************************************************************
##
##     File Name:	DBAdd.pl
##         Owner:	InfoPlace
##     Copyright:	InfoPlace Inc., Pleasanton, CA, USA, 1996.
##     				Not to be copied, modified or redistributed without written
##     				permission of InfoPlace.
##
##
##*****************************************************************************
##                      R E V I S I O N   H I S T O R Y
##*****************************************************************************
##
##	  Date		Time	Author	Description
##	--------	-----	------	---------------------------------------------
##	09/15/96	12:50	N		Original version
##
##*****************************************************************************


require "FileLock.pl";
require "DBUtility.pl";

# --------------------------------------------------------------------------------
# DBAppendRecord(@DBInfo, @record)
# Appends the record  to the database.
# Args:
#	@DBInfo 		(list): the data contained in the list returned by DBOpen
#	@record			(list): list of field values for the record
#							
#	Return Value	(integer): 0 if append successful, 1 if bad DBInfo, 
#                              -1 if DB busy (timed out)

sub DBAppendRecord
{
	local(*db, *record) = @_;
	if ($db[$kDBMagicNumberIndex] != $kDBMagicNumber) {
		return 1;
	}
	if ($db[$kDBAccessModeIndex] != $kDBWriteAccess) {
		&LogMsg("DBAppendRecord::No write access to DB\n");
		return 1;
	}
#	if ($db[$kDBFieldCountIndex] != $#record + 1) {
#		&LogMsg("DBAppendRecord::Improper record, does not match DB record\n");
#		return 1;
#	}
	# check which datafile depending on bucket count ...
	local ($datafile) = $db[$kDBDirectoryIndex] . $kDBDataFile;
	if ($db[$kDBBucketCountIndex]) {
		$datafile = $datafile . (&HashKey($record[0]) & ($db[$kDBBucketCountIndex] - 1));
	}
	umask 0000;
	if (!open(DATAFILE, ">>$datafile")) {
	    &LogFileOpenFailMsg($datafile);
	    return -1;
	}
	my($size) = $#record + 1;
	my($i) = 0;
	for (; $i < $size;) {
		$record[$i] =~ s/\n/ /g;
		$record[$i] =~ s/\r/ /g;
		$record[$i++] =~ s/$kFieldSeparator/\^/g;
	}
	my($line) = join($kFieldSeparator, @record);

	# write lock file
	&GetWriteLockOnFile("DATAFILE") || &LogMsg("DBAppendRecord::GetWriteLockOnFile failed on file handle $datafile: $!\n");

	print DATAFILE "$line\n";

	close(DATAFILE);
    # unlock the file
	&UnlockFile("DATAFILE") || &LogMsg("DBAppendRecord::UnlockFile failed on file handle $datafile: $!\n");
	return 0;
}

# --------------------------------------------------------------------------------
# DBAppendRecordIfNotExists(@DBInfo, @record)
# Appends the record  to the database, if not already there
# Args:
#	@DBInfo 		(list): the data contained in the list returned by DBOpen
#	@record			(list): list of field values for the record
#							
#	Return Value	(integer): 0 if append successful, 1 if bad DBInfo, 
#                              -1 if DB busy (timed out), 2 if record already exists

sub DBAppendRecordIfNotExists
{
	local(*db, *record) = @_;
	if ($db[$kDBMagicNumberIndex] != $kDBMagicNumber) {
		return 1;
	}
	if ($db[$kDBAccessModeIndex] != $kDBWriteAccess) {
		&LogMsg("DBAppendRecordIfNotExists::No write access to DB\n");
		return 1;
	}
#	if ($db[$kDBFieldCountIndex] != $#record + 1) {
#		&LogMsg("DBAppendRecordIfNotExists::Improper record, does not match DB record\n");
#		return 1;
#	}
	# check which datafile depending on bucket count ...
	local ($datafile) = $db[$kDBDirectoryIndex] . $kDBDataFile;
	local($key) = $record[0];
	if ($db[$kDBBucketCountIndex]) {
		$datafile = $datafile . (&HashKey($key) & ($db[$kDBBucketCountIndex] - 1));
	}
	local($times) = 0;
	while (!open(DATAFILE, "+<$datafile")) {
	    if ($times) {
		&LogFileOpenFailMsg($datafile);
		return -1;
	    }
	    umask 0000;
	    open(DATAFILE, ">>$datafile");
	    close(DATAFILE);
	    $times++;
	}
	# write lock file
	&GetWriteLockOnFile("DATAFILE") || &LogMsg("DBAppendRecordIfNotExists::GetWriteLockOnFile failed on file handle $datafile: $!\n");
	local($line);

	$key =~ s/([^a-zA-Z0-9])/\\$1/g;

	while (!eof(DATAFILE)) {
		$line = <DATAFILE>;
		chop($line);
		if ($line =~  /^${key}${kFieldSeparator}.*/) {
		    close(DATAFILE);
	         # unlock the file
			&UnlockFile("DATAFILE") || &LogMsg("DBAppendRecordIfNotExists::UnlockFile failed on file handle $datafile: $!\n");
			return 2;
		}
	}
    my($size) = $#record + 1;
    my($i) = 0;
	for (; $i < $size;) {
		$record[$i] =~ s/\n/ /g;
		$record[$i] =~ s/\r/ /g;
		$record[$i++] =~ s/$kFieldSeparator/\^/g;
	}
	$line = join($kFieldSeparator, @record);
	print DATAFILE "$line\n";

	close(DATAFILE);
    # unlock the file
	&UnlockFile("DATAFILE") || &LogMsg("DBAppendRecordIfNotExists::UnlockFile failed on file handle $datafile: $!\n");

	return 0;
}
