# $Id: DBCreate.pl,v 1.1 1998/05/04 19:50:02 naga Exp $
# Copyright InfoPlace Inc., Milpitas, CA, USA, 1996-1998
# Not to be copied, modified or redistributed without written
# permission of InfoPlace.
#
# --------------------------------------------------------------------------------

require "FileLock.pl";
require "DBUtility.pl";

# --------------------------------------------------------------------------------
# DBCreate($name, $pathName, @fieldNameRecord, @fieldTypeRecord, $maxSize, $doHash)
# Creates a database in the form of a directory.
# Args:
#	$name 		(string): the name of the DB directory
#	$pathName 	(string): the directory where the DB is created
#	@fieldNameRecord	(list): list of field names
#	@fieldTypeRecord	(list): list of field types
#	$maxSize	(integer): maximum number of records estimated to be in the DB
#	$doHash		(boolean): whether to use Hashing for the record files
#
#	Return Value	(boolean): 0 if creation successful, 1 if not.

sub DBCreate
{
	local($name, $pathName, *fieldNameRecord, *fieldTypeRecord, $maxSize, $doHash) = @_;
	local($dbName);
	local($fieldCount) = $#fieldNameRecord + 1;
	# Check if field names and its type are equal in length
	if ($fieldCount != $#fieldTypeRecord + 1) {
		&LogMsg("DBCreate::Fail to create DB, the field-names and field-types donot match:\n");
		local($fieldNames) = join($kFieldSeperator, @fieldNameRecord);
		local($fieldTypes) = join($kFieldSeperator, @fieldTypeRecord);
		&LogMsg("FieldNames: $fieldNames \nFieldTypes: $fieldTypes \n");
		return 1;
	}
	# ----------------------------------------
	# First create DB directory ...
	umask 0000;
	if (!mkdir($pathName . $name, $kDBDefaultPermission)) {
		local($fname) = $pathName . $name;
		&LogMsg("DBCreate::Fail to create DB directory $fname: $!\n");
		return 1;
	}
	local($dbDir) = $pathName . $name . $kDBPathNameSeparator;
	# ----------------------------------------
	# Create Data file(s) ...
	local($datafile) = $dbDir . $kDBDataFile;
	local($bucketCount) = 0;
	if ($doHash == 0) {
		if (!open(DATAFILE, ">$datafile")) {
			&LogMsg("DBCreate::Fail to create DB data file $datafile: $!\n");
			return 1;		
		}
		close(DATAFILE);
	}
	else {
		$bucketCount = $kDBMinBucketCount;
		while ($bucketCount < $kDBMaxBucketCount && $maxSize/$bucketCount > $kDBPerBucketCount) {
			$bucketCount *= 2;
		}
		# The bucket-files are created on demand ...
	}
	# ----------------------------------------
	# Create metadata file ...
	local($metafile) = $dbDir . $kDBMetaDataFile;
	if (!open(METAFILE, ">$metafile")) {
		&LogMsg("DBCreate::Fail to create DB metadata file: $!\n");
		return 1;		
	}
	local($fieldNames) = join($kFieldSeperator, @fieldNameRecord);
	local($fieldTypes) = join($kFieldSeperator, @fieldTypeRecord);
	printf(METAFILE "%d\n%d\n%s\n%s\n", $doHash ? $bucketCount : 0, $#fieldNameRecord + 1, 
					$fieldNames, $fieldTypes);
	close(METAFILE);
	return 0;
}

# --------------------------------------------------------------------------------
# DBDelete($name, $pathName)
# Deletes the database.
# Args:
#	$name 			(string): the name of the DB directory
#	$pathName 		(string): the directory where the DB is created
#
#	Return Value	(boolean): 0 if deletion successful, 1 if not.
#

sub DBDelete
{
	local($name, $pathName) = @_;
	local($dbDir) = $pathName.$name;
	system("rm -rf $dbDir");  # ?????????????????? ??????????????????
	return 0;

#	local($dbDir) = $pathName.$name.$kDBPathNameSeparator;
#	unlink <$dbDir*>;
#	return (rmdir $dbDir) ? 0 : 1;
}

1;
