#!/usr/bin/perl
##*****************************************************************************
##
##     File Name:	DBIterate.pl
##         Owner:	InfoPlace
##     Copyright:	InfoPlace Inc., Pleasanton, CA, USA, 1996.
##     				Not to be copied, modified or redistributed without written
##     				permission of InfoPlace.
##
##
##*****************************************************************************
##                      R E V I S I O N   H I S T O R Y
##*****************************************************************************
##
##	  Date		Time	Author	Description
##	--------	-----	------	---------------------------------------------
##	09/15/96	12:51	N		Original version
##
##*****************************************************************************


require "FileLock.pl";
require "DBUtility.pl";


# --------------------------------------------------------------------------------
# DBReadFirstRecord(@DBInfo, @record)
# Returns the first record from the database. Since the DB is not ordered, the
# first record could be any record.  Useful for iterating through all the records
# in the DB in conjunction with DBReadNextRecord() described later.
# Args:
#	@DBInfo 		(list): the data contained in the list returned by DBOpen
#	@record			(list): used to return the list of field values for the record
#							
#	Return Value	(integer): 0 if read successful, 1 if bad DBInfo, 
#                              -1 if DB busy (timed out), 2 if no more records.

sub DBReadFirstRecord
{
    local(*db, *record) = @_;
    if ($db[$kDBMagicNumberIndex] != $kDBMagicNumber) {
	return 1;
    }
    local($recordfile) = $db[$kDBDirectoryIndex] . $kDBDataFile;
    local($bucketIndex) = 0;
    local($datafile) = $recordfile;
    do {
	$db[$kDBBucketIndex] = $bucketIndex;
	if ($db[$kDBBucketCountIndex]) {
	    $datafile = $recordfile . $bucketIndex;
	}
	if (open("READDATAFILE", "<$datafile")) {
	    local($/) = "\n";

	    # read lock file
	    &GetReadLockOnFile("READDATAFILE") || &LogMsg("DBReadFirstRecord::GetReadLockOnFile failed on file handle $datafile: $!\n");
	    if (!eof("READDATAFILE")) {
	    	local ($line);
			$line = <READDATAFILE>;
	    	chop($line);
	    	@record = split(/$kFieldSeperator/, $line);
			return 0;
	    }
	    else {
			close("READDATAFILE");
			# unlock the file
			&UnlockFile("READDATAFILE") || &LogMsg("DBReadFirstRecord::UnlockFile failed on file handle $datafile: $!\n");
	    }
	}
	$bucketIndex++;
    } until (!($bucketIndex < $db[$kDBBucketCountIndex]));
    return 2;
}

# --------------------------------------------------------------------------------
# Dbreadnextrecord(@DBInfo, @record)
# Returns the next record from the database. Since the DB is not ordered, the
# next record could be any record.  Useful for iterating through all the records
# in the DB in conjunction with DBReadFirstRecord() described earlier.
# Args:
#	@DBInfo 		(list): the data contained in the list returned by DBOpen
#	@record			(list): used to return the list of field values for the record
#							
#	Return Value	(integer): 0 if read successful, 1 if bad DBInfo, 
#                              -1 if DB busy (timed out), 2 if no more records.

sub DBReadNextRecord
{
    local(*db, *record) = @_;
    if ($db[$kDBMagicNumberIndex] != $kDBMagicNumber) {
	return 1;
    }
    local($bucketIndex) = $db[$kDBBucketIndex];
    do {
	local($/) = "\n";
	if (!eof("READDATAFILE")) {
	    local ($line);
		$line = <READDATAFILE>;
	    chop($line);
	    @record = split(/$kFieldSeperator/, $line);
	    return 0;
	}
	else {
	    close("READDATAFILE");
	    # unlock the file
	    &UnlockFile("READDATAFILE") || &LogMsg("DBReadNextRecord::UnlockFile failed on file handle: $!\n");

	    if ($db[$kDBBucketCountIndex]) {
		local($result) = 0;
		do {
		    $db[$kDBBucketIndex] = ++$bucketIndex;
		    if ($bucketIndex < $db[$kDBBucketCountIndex]) {
			local ($datafile) = $db[$kDBDirectoryIndex] . $kDBDataFile;
			$db[$kDBBucketIndex] = $bucketIndex;
			$datafile = $datafile . $bucketIndex;
			$result = open(READDATAFILE, "<$datafile");
		    }
		} until ($result || !($bucketIndex < $db[$kDBBucketCountIndex]));
	    }
	}
    } until (!($bucketIndex < $db[$kDBBucketCountIndex]));
    return 2;
}

1;
