#!/usr/bin/perl
##*****************************************************************************
##
##     File Name:	DBLookUp.pl
##         Owner:	InfoPlace
##     Copyright:	InfoPlace Inc., Pleasanton, CA, USA, 1996.
##     				Not to be copied, modified or redistributed without written
##     				permission of InfoPlace.
##
##
##*****************************************************************************
##                      R E V I S I O N   H I S T O R Y
##*****************************************************************************
##
##	  Date		Time	Author	Description
##	--------	-----	------	---------------------------------------------
##	09/15/96	12:51	N		Original version
##
##*****************************************************************************


require "FileLock.pl";
require "DBUtility.pl";

# --------------------------------------------------------------------------------
# DBRetreiveRecord(@DBInfo, $key, @record)
# Search the database for a record with the key value specified.
#
# Args:
#	@DBInfo 		(list): the data contained in the list returned by DBOpen
#	$key			(string): the key value for the record, where key is the first
#							  field of the record.
#	@record			(list): used to return the list of field values for the record
#							
#	Return Value	(integer): 0 if retreive successful, 1 if bad DBInfo, 
#                              -1 if DB busy (timed out), 2 if not found

sub DBRetreiveRecord
{
    local(*db, $key, *record) = @_;
    if ($db[$kDBMagicNumberIndex] != $kDBMagicNumber) {
	return 1;
    }
    local($bucketIndex) = -1;
    local($datafile) = $db[$kDBDirectoryIndex] . $kDBDataFile;
    if ($db[$kDBBucketCountIndex]) {
	$datafile = $datafile . (&HashKey($key) & ($db[$kDBBucketCountIndex] - 1));
    }
    # Now open the data file and read record lines until the record we seek is found
    if (!open(DATAFILE, "<$datafile")) {
	@record = ();
	return 2;
    }
    local($/) = "\n";
    # read lock file
    &GetReadLockOnFile("DATAFILE") || &LogMsg("DBRetreiveRecord::GetReadLockOnFile failed on file handle: $!\n");
	
	$key =~ s/([^a-zA-Z0-9])/\\$1/g;
    while (!eof(DATAFILE)) {
	$recordline = <DATAFILE>;
	chop($recordline);

	if ($recordline =~  /^${key}${kFieldSeparator}.*/) {
	     @record = split(/$kFieldSeparator/, $recordline);
	     if (!defined($record[($db[$kDBFieldCountIndex] - 1)]) ) {
		 # retain the length of record
		 $record[($db[$kDBFieldCountIndex] - 1)] = "";
	     }
	     close(DATAFILE);
	     &UnlockFile("DATAFILE") || &LogMsg("DBRetreiveRecord::UnlockFile failed on file handle: $!\n");
	     return 0;
	 }
    }
    # If record not found ...
    close(DATAFILE);
    # unlock the file
    &UnlockFile("DATAFILE") || &LogMsg("DBRetreiveRecord::UnlockFile failed on file handle: $!\n");
    @record = ();
    return 2;	
}

# --------------------------------------------------------------------------------
# DBRecordExists(@DBInfo, $key)
# Search the database for the existence of a record with the key value specified.
#
# Args:
#	@DBInfo 		(list): the data contained in the list returned by DBOpen
#	$key			(string): the key value for the record, where key is the first
#							  field of the record.
#							
#	Return Value	(integer): 0 if record with key found, 1 if bad DBInfo, 
#                              -1 if DB busy (timed out), 2 if not found.

sub DBRecordExists
{
    local(*db, $key, *record) = @_;
    if ($db[$kDBMagicNumberIndex] != $kDBMagicNumber) {
		return 1;
    }
    local($bucketIndex) = -1;
    local($datafile) = $db[$kDBDirectoryIndex] . $kDBDataFile;
    if ($db[$kDBBucketCountIndex]) {
		$datafile = $datafile . (&HashKey($key) & ($db[$kDBBucketCountIndex] - 1));
    }
    # Now open the data file and read record lines until the record we seek is found
    if (!open(DATAFILE, "<$datafile")) {
		@record = ();
		return 2;
    }
    local($/) = "\n";
    # read lock file
    &GetReadLockOnFile("DATAFILE") || &LogMsg("DBRecordExists::GetReadLockOnFile failed on file handle $datafile: $!\n");
	
	$key =~ s/([^a-zA-Z0-9])/\\$1/g;
    while (!eof(DATAFILE)) {
	   $recordline = <DATAFILE>;
       chop($recordline);

	if ($recordline =~  /^${key}${kFieldSeparator}.*/) {
	    close(DATAFILE);
	    &UnlockFile("DATAFILE") || &LogMsg("DBRecordExists::UnlockFile failed on file handle $datafile: $!\n");
	    return 0;
	}
     }
     close(DATAFILE);
     # If record not found ...
     # unlock the file
     &UnlockFile("DATAFILE") || &LogMsg("DBRecordExists::UnlockFile failed on file handle $datafile: $!\n");

     @record = ();
     return 2;	
}

1;
