#!/usr/bin/perl
##*****************************************************************************
##
##     File Name:	DBOpen.pl
##         Owner:	InfoPlace
##     Copyright:	InfoPlace Inc., Pleasanton, CA, USA, 1996.
##     				Not to be copied, modified or redistributed without written
##     				permission of InfoPlace.
##
##
##*****************************************************************************
##                      R E V I S I O N   H I S T O R Y
##*****************************************************************************
##
##	  Date		Time	Author	Description
##	--------	-----	------	---------------------------------------------
##	09/15/96	12:51	N		Original version
##
##*****************************************************************************


require "FileLock.pl";
require "DBUtility.pl";

# --------------------------------------------------------------------------------
# DBOpen($name, $pathName, $mode, @DBInfo)
# Opens the database with read or write mode specified by $mode.
# Args:
#	$name 			(string): the name of the DB directory
#	$pathName 		(string): the directory where the DB is created
#	$mode			(pre-defined constants):  
#							$kDBWriteAccess specifies write access
#							$kDBReadAccess specifies read only access
#	@DBInfo			(list): a list for output that specifies the DB specific data 
#						 that can be used to access the DB using the other APIs.
#							
#	Return Value	(boolean): 0 if open successful, 1 if not.

sub DBOpen
{
	local($name, $pathName, $mode, *db) = @_;
	local($dbDir) = $pathName . $name . $kDBPathNameSeparator;
	local($metafile) = $dbDir . $kDBMetaDataFile;

	# clear the db array, if not empty ??

	local($metafile) = $dbDir . $kDBMetaDataFile;
	if (!open(METAFILE, "<$metafile")) {
		&LogMsg("DBOpen::Fail to open DB metadata file $metafile: $!\n");
		return 1;		
	}
	local($/) = "\n";
	$db[$kDBMagicNumberIndex] = $kDBMagicNumber;
	$db[$kDBDirectoryIndex] = $dbDir;
    $db[$kDBAccessModeIndex] = $mode;  

	# read lock file
	&GetReadLockOnFile("METAFILE") || &LogMsg("DBOpen::GetReadLockOnFile failed on file handle $metafile: $!\n");

	local($/) = "\n";
	$db[$kDBBucketCountIndex] = eval(<METAFILE>); 
	$db[$kDBFieldCountIndex] = eval(<METAFILE>);
	local($fnames);
	$fnames = <METAFILE>;
	chop($fnames);
    $db[$kDBFieldNamesIndex] = $fnames;
	local($ftypes);
	$ftypes = <METAFILE>;
	chop($ftypes);
    $db[$kDBFieldTypesIndex] = $ftypes;

    close(METAFILE);
	# unlock the file
	&UnlockFile("METAFILE") || &LogMsg("DBOpen::UnlockFile failed on file handle $metafile: $!\n");

	$db[$kDBBucketIndex] = 0;
	return 0;	
}

# --------------------------------------------------------------------------------
# DBClose(@DBInfo)
# Closes the database
# Args:
#	@DBInfo 		(list): the data contained in the list returned by DBOpen
#							
#	Return Value	(boolean): 0 if close successful, 1 if not.

sub DBClose
{
	local(*db) = @_;
	if ($db[$kDBMagicNumberIndex] != $kDBMagicNumber) {
		return 1;
	}
	$db[$kDBMagicNumberIndex] = 0;
	return 0;
}

1;
