#!/usr/bin/perl
##*****************************************************************************
##
##     File Name:	DBPrint.pl
##         Owner:	InfoPlace
##     Copyright:	InfoPlace Inc., Pleasanton, CA, USA, 1996.
##     				Not to be copied, modified or redistributed without written
##     				permission of InfoPlace.
##
##
##*****************************************************************************
##                      R E V I S I O N   H I S T O R Y
##*****************************************************************************
##
##	  Date		Time	Author	Description
##	--------	-----	------	---------------------------------------------
##	09/15/96	12:51	N		Original version
##
##*****************************************************************************


require "FileLock.pl";
require "DBUtility.pl";

# --------------------------------------------------------------------------------
# DBPrint(@DBInfo)
# Prints the db records to stdout
# Args:
#	@DBInfo 		(list): the data contained in the list returned by DBOpen
#	$maxCount 		(integer): the maximum number of records to be printed (optional)
#							   if not specified or 0 then prints all records.
#
#	Return Value	(boolean): 0 if print successful, 1 if not.

sub DBPrint
{
	local(*db, $maxCount) = @_;
	if ($db[$kDBMagicNumberIndex] != $kDBMagicNumber) {
		&LogMsg("DBPrint::Bad DB parameter!!\n");
		return 1;
	}
	if (!$maxCount) {
		$maxCount = $kDBMaxBucketCount*$kDBPerBucketCount;
	}
	local(@record) = ();

	local($value) = &DBReadFirstRecord(*db, *record);
	local($count) = 0;
	while (($count <= $maxCount) && !$value) {
		$count++;
		local($str) = join(' ', @record);
		print "$count.  $str\n";
		$value = &DBReadNextRecord(*db, *record);
	}
	print "\n\nTotal records: $count\n";
	return 0;
}

# --------------------------------------------------------------------------------
# DBPrintRecords(@DBInfo, @keys)
# Prints the db records associated with the keys given to stdout
# Args:
#	@DBInfo 	(list): the data contained in the list returned by DBOpen
#	@keys 		(list): a list of keys
#
#	Return Value	(boolean): 0 if print successful, 1 if not.

sub DBPrintRecords
{
	local(*db, *keys) = @_;
	if ($db[$kDBMagicNumberIndex] != $kDBMagicNumber) {
		&LogMsg("DBPrintRecords::Bad DB parameter!!\n");
		return 1;
	}
	if (!$#keys) {
		&LogMsg("DBPrintRecords::No keys specified!!\n");
		return 1;
	}
	local(@record) = ();

	local($count) = 0;
	foreach $key ($keylist) {
		$count++;
		local($value) = &DBRetreiveRecord(*db, *record);
		if ($value) {
			local($str) = join(' ', @record);
			print "$count.  $str\n";
		}
		else {
			print "No record found for key: ${key}\n";
		}
	}
	print "\n\nTotal records found: $count\n";
	return 0;
}

1;
