#!/usr/bin/perl
##*****************************************************************************
##
##     File Name:	DBRemove.pl
##         Owner:	InfoPlace
##     Copyright:	InfoPlace Inc., Pleasanton, CA, USA, 1996.
##     				Not to be copied, modified or redistributed without written
##     				permission of InfoPlace.
##
##
##*****************************************************************************
##                      R E V I S I O N   H I S T O R Y
##*****************************************************************************
##
##	  Date		Time	Author	Description
##	--------	-----	------	---------------------------------------------
##	09/15/96	12:51	N		Original version
##
##*****************************************************************************


require "FileLock.pl";
require "DBUtility.pl";


# --------------------------------------------------------------------------------
# DBReplaceRecord(@DBInfo, $key, @record)
# Replaces the record with the specified key,in the database with the
# given record. 
# Args:
#	@DBInfo 		(list): the data contained in the list returned by DBOpen
#	$key			(string): the key value for the record to be replaced, where 
#							  key is the first field of the record.
#	@record			(list): list of field values for the new record
#							
#	Return Value	(integer): 0 if replace successful, 1 if bad DBInfo, 
#                              -1 if DB busy (timed out), 2 if not found

sub DBReplaceRecord
{
	local(*db, $key, *record) = @_;
	if ($db[$kDBMagicNumberIndex] != $kDBMagicNumber) {
		return 1;
	}
	local($datafile) = $db[$kDBDirectoryIndex] . $kDBDataFile;
	if ($db[$kDBBucketCountIndex]) {
		$datafile = $datafile . (&HashKey($key) & ($db[$kDBBucketCountIndex] - 1));
	}
    # Now open the data file and read record lines until the record we seek is found
	if (!open(REPLACEFILE, "+<$datafile")) {
	    return 2;
	}
	# write lock file
	&GetWriteLockOnFile("REPLACEFILE") || &LogMsg("DBReplaceRecord::GetWriteLockOnFile failed on file handle $datafile: $!\n");

	local($data);
	local($/);
	$data = <REPLACEFILE>;
	local($initSize) = length($data);

	my($size) = $#record + 1;
	my($i) = 0;
	for (; $i < $size;) {
		$record[$i] =~ s/\n/ /g;
		$record[$i] =~ s/\r/ /g;
		$record[$i++] =~ s/$kFieldSeparator/\^/g;
	}
	local($str) = join($kFieldSeparator, @record);

	$key =~ s/([^a-zA-Z0-9])/\\$1/g;

	$source = "^${key}${kFieldSeparator}.*\n";
	$target = "$str\n";
	local($done) = 0;
	if ($data =~ s/$source/$target/e) {
		$done = 1;
	}
	else {
		$source = "\n${key}${kFieldSeparator}.*\n";
		$target = "\n$str\n";
		if ($data =~ s/$source/$target/e) {
		    $done = 1;
		}
	}
	if ($done) {
		seek(REPLACEFILE, 0, 0);
		print REPLACEFILE $data;
		local($finalSize) = length($data);
		if ($initSize > $finalSize) {
			truncate(REPLACEFILE, $finalSize);
		}
	}
	close(REPLACEFILE);
    # unlock the file
	&UnlockFile("REPLACEFILE") || &LogMsg("DBReplaceRecord::UnlockFile failed on file handle $datafile: $!\n");

	return ($done) ? 0 : 2;
}


# --------------------------------------------------------------------------------
# DBRemoveRecord(@DBInfo, $key)
# Remove the record with the specified key,in the database.
# Args:
#	@DBInfo 		(list): the data contained in the list returned by DBOpen
#	$key			(string): the key value for the record to be removed, where 
#							  key is the first field of the record.
#							
#	Return Value	(integer): 0 if remove successful, 1 if bad DBInfo, 
#                              -1 if DB busy (timed out), 2 if not found

sub DBRemoveRecord
{
	local(*db, $key) = @_;
	local(@record);
	if ($db[$kDBMagicNumberIndex] != $kDBMagicNumber) {
		return 1;
	}
	local($datafile) = $db[$kDBDirectoryIndex] . $kDBDataFile;
	if ($db[$kDBBucketCountIndex]) {
		$datafile = $datafile . (&HashKey($key) & ($db[$kDBBucketCountIndex] - 1));
	}
    # Now open the data file and read record lines until the record we seek is found
	if (!open(REMOVEFILE, "+<$datafile")) {
	    return 2;
	}
	# write lock file
	&GetWriteLockOnFile("REMOVEFILE") || &LogMsg("DBRemoveRecord::GetWriteLockOnFile failed on file handle $datafile: $!\n");

	local($data);
	local($/);
	$data = <REMOVEFILE>;

	$key =~ s/([^a-zA-Z0-9])/\\$1/g;

	$source = "^${key}${kFieldSeparator}.*\n";
	$target = "";
	local($done) = 0;
	if ($data =~ s/$source/$target/e) {
		$done = 1;
	}
	else {
		$source = "\n${key}${kFieldSeparator}.*\n";
		$target = "\n";
		if ($data =~ s/$source/$target/e) {
		    $done = 1;
		}
	}
	if ($done) {
		seek(REMOVEFILE, 0, 0);
		print REMOVEFILE $data;
		local($finalSize) = length($data);
		truncate(REMOVEFILE, $finalSize);
	}
	close(REMOVEFILE);
    # unlock the file
	&UnlockFile("REMOVEFILE") || &LogMsg("DBRemoveRecord::UnlockFile failed on file handle $datafile: $!\n");

	return ($done) ? 0 : 2;
}

