#!/usr/bin/perl
##*****************************************************************************
##
##     File Name:	DBUtility.pl
##         Owner:	InfoPlace
##     Copyright:	InfoPlace Inc., Pleasanton, CA, USA, 1996.
##     				Not to be copied, modified or redistributed without written
##     				permission of InfoPlace.
##
##
##*****************************************************************************
##                      R E V I S I O N   H I S T O R Y
##*****************************************************************************
##
##	  Date		Time	Author	Description
##	--------	-----	------	---------------------------------------------
##	09/15/96	12:51	N		Original version
##
##*****************************************************************************


require "LogMsg.pl";
require "DBGlobals.pl";

# --------------------------------------------------------------------------------
# DBFieldValue(@DBInfo, @record, $field)
# Given a record and a field name, return the value of the field.
# Args:
#	@DBInfo 		(list): the data contained in the list returned by DBOpen
#	@record			(list): the record returned by DBRead/DBRetreive
#	$field			(string): the name of the field
#
#	Return Value	(scalar): value of the field or undef (if field not found)

sub DBFieldValue
{
	local(*db, *record, $field) = @_;
	if ($db[$kDBMagicNumberIndex] != $kDBMagicNumber) {
		return undef;
	}
	local(@fieldNames) = ();
	if (&DBFieldNames(*db, *fieldNames) == 0) {
		for (local($i) = 0; $i <= $#fieldNames; $i++) {
			if ($fieldNames[$i] eq $field) { # String compare
				return $record[$i];
			}
		}
	}
	return undef;
}

# --------------------------------------------------------------------------------
# DBFieldNames(@DBInfo, @fieldNames)
# Returns the names of the record fields for a DB
# Args:
#	@DBInfo 		(list): the data contained in the list returned by DBOpen
#	@fieldNames		(list of strings): the names of the fields
#
#	Return Value	(boolean): 0 if DBInfo is valid else 1.

sub DBFieldNames
{
	local(*db, *fieldNames) = @_;
	if ($db[$kDBMagicNumberIndex] != $kDBMagicNumber) {
		return 1;
	}
	@fieldNames = split(/$kFieldSeperator/, $db[$kDBFieldNamesIndex]);
	return 0;
}

# --------------------------------------------------------------------------------
# DBFieldTypes(@DBInfo, @fieldTypes)
# Returns the  of the record fields for a DB
# Args:
#	@DBInfo 		(list): the data contained in the list returned by DBOpen
#	@fieldTypes		(list of strings): the types of the fields
#
#	Return Value	(boolean): 0 if DBInfo is valid else 1.

sub DBFieldTypes
{
	local(*db, *fieldTypes) = @_;
	if ($db[$kDBMagicNumberIndex] != $kDBMagicNumber) {
		return 1;
	}
	@fieldTypes = split(/$kFieldSeperator/, $db[$kDBFieldTypesIndex]);
	return 0;
}

# --------------------------------------------------------------------------------
# Hash a given string and return a integer value
sub HashKey
{
	local($key) = @_;
	local($value) = 0;
	local($size) = length($key);
	for (local($i) = 0; $i < $size; $i++) {
		local ($char) = vec($key, $i, 8);
		$char |= 0x20;
		$value += $char*($i+1) + $char*6421;
	}
	return $value;
}

1;
