# $Id: DocDB.pl,v 1.1 1998/05/04 19:50:02 naga Exp $
# Copyright InfoPlace Inc., Milpitas, CA, USA, 1996-1998
# Not to be copied, modified or redistributed without written
# permission of InfoPlace.
#
# --------------------------------------------------------------------------------

require "DBCreate.pl";
require "DBOpen.pl";
require "DBAdd.pl";
require "UMDB.pl";

# --------------------------------------------------------------------------------
# Define CategoryDB - This keeps the category records
#
sub DefineCategoryDB
{
	if (&DBCreate($kCategoryDB, $kDocDBPath, \@kCategoryDBFields, \@kCategoryDBFieldsTypes, $kMaxCategoryCount, 1)) {
		&LogMsg("Error: Failed to create the DB: $kCategoryDB");
    return 1;
	}
    return 0;
}

# --------------------------------------------------------------------------------
# Define CategoryMapDB - This keeps the category name - id map
#
sub DefineCategoryMapDB
{
	if (&DBCreate($kCategoryMapDB, $kDocDBPath, \@kCategoryMapDBFields, \@kCategoryMapDBFieldsTypes, $kMaxCategoryCount, 1)) {
		&LogMsg("Error: Failed to create the DB: $kCategoryDB");
    return 1;
	}
    return 0;
}

# --------------------------------------------------------------------------------
# Initialize CategoryDB with a root category
#
sub InitCategoryDB
{
	my($result);
	my(@db) = ();
	$result = &DBOpen($kCategoryDB, $kDocDBPath, $kDBWriteAccess, \@db);
	if ($result == $errNone) {
		my($t) = time();
		my(@record) = ($kRootCategoryID, $kRootCategoryName, "", "", "", "", $t, $t, 0);
		$result = &DBAppendRecord(\@db, \@record);
		&DBClose(\@db);
	}
	if ($result == $errNone) {
		$result = &DBOpen($kCategoryMapDB, $kDocDBPath, $kDBWriteAccess, \@db);
		if ($result == $errNone) {
			my($t) = time();
			my(@record) = ($kRootCategoryName, $kRootCategoryID);
			$result = &DBAppendRecord(\@db, \@record);
			&DBClose(\@db);
		}
	}
	return $result;
}

# --------------------------------------------------------------------------------
# Initialize MasterCatID to 1 in its file
#
sub InitMasterCatID
{
	if (!open(IDFILE, ">$kMasterCatIDFile")) {
		&LogMsg("Error: Failed to create the file for master category-id index");
		return 1;
	}
	my($id) = $kRootCategoryID + 1;
	print IDFILE "${id}\n";
	close(IDFILE);
	return 0;

}

# --------------------------------------------------------------------------------
# Initialize MasterDocID to 1 in its file
#
sub InitMasterDocID
{
	if (!open(IDFILE, ">$kMasterDocIDFile")) {
		&LogMsg("Error: Failed to create the file for master doc-id index");
		return 1;
	}
	print IDFILE "1\n";
	close(IDFILE);
	return 0;

}

# --------------------------------------------------------------------------------
# Define DocumentDB - This keeps the document records
#
sub DefineDocumentDB
{
	if (&DBCreate($kDocumentDB, $kDocDBPath, \@kDocumentDBFields, \@kDocumentDBFieldsTypes, $kMaxDocumentCount, 1)) {
		&LogMsg("Error: Failed to create the DB: $kDocumentDB");
    return 1;
	}
    return 0;
}

# --------------------------------------------------------------------------------
# Define GlossaryDB - This keeps the glossary records
#
sub DefineGlossaryDB
{
    if (&DBCreate($kGlossaryDB, $kDocDBPath, \@kGlossaryDBFields, \@kGlossaryDBFieldsTypes, $kMaxGlossaryCount, 1)) {
	&LogMsg("Error: Failed to create the DB: $kGlossaryDB");
	return 1;
    }
    return 0;
}

# --------------------------------------------------------------------------------
# Define URLIDMapDB - This keeps the URLIDMap records
#
sub DefineURLIDMapDB
{
    if (&DBCreate($kURLIDMapDB, $kDocDBPath, \@kURLIDMapDBFields, \@kURLIDMapDBFieldsTypes, $kMaxURLIDMapCount, 1)) {
	&LogMsg("Error: Failed to create the DB: $kURLIDMapDB");
	return 1;
    }
    return 0;
}

# --------------------------------------------------------------------------------
# Define IDDocsMapDB - This keeps the IDDocsMap records
#
sub DefineIDDocsMapDB
{
    if (&DBCreate($kIDDocsMapDB, $kDocDBPath, \@kIDDocsMapDBFields, \@kIDDocsMapDBFieldsTypes, $kMaxIDDocsMapCount, 1)) {
	&LogMsg("Error: Failed to create the DB: $kIDDocsMapDB");
	return 1;
    }
    return 0;
}

# --------------------------------------------------------------------------------
# Initialize MasterDocID to 1 in its file
#
sub InitMasterURLID
{
    if (!open(IDFILE, ">$kMasterURLIDFile")) {
	&LogMsg("Error: Failed to create the file for master url-id index");
	return 1;
    }
    print IDFILE "1\n";
    close(IDFILE);
    return 0;
}

# --------------------------------------------------------------------------------
# Create Lock files
#
sub CreateLockFiles
{
	if (!open(LFILE, ">$kCatDocLockFile")) {
		&LogMsg("Error: Failed to create the lock file for - $kCatDocLockFile");
		return 1;
	}
	close(LFILE);
	if (!open(LFILE, ">$kGlossaryLockFile")) {
		&LogMsg("Error: Failed to create the lock file for - $kGlossaryLockFile");
		return 1;
	}
	close(LFILE);
	return 0;

}

# --------------------------------------------------------------------------------
# Defines all DBs
#
sub DefineAllDBs
{
   my($result);
   $result = &DefineCategoryDB();
   $result = $result ||	&DefineCategoryMapDB();
   $result = $result ||	&DefineDocumentDB();
   $result = $result ||	&DefineGlossaryDB();
   $result = $result ||	&DefineURLIDMapDB();
   $result = $result ||	&DefineIDDocsMapDB();
   $result = $result ||	&InitCategoryDB();
   $result = $result ||	&InitMasterCatID();
   $result = $result ||	&InitMasterDocID();
   $result = $result ||	&InitMasterURLID();
   $result = $result ||	&CreateLockFiles();
   $result = $result || &DefineAllUMDBs();
   return $result;
}

# --------------------------------------------------------------------------------
# Delete all DBs
#
sub DeleteAllDBs
{
    my($result);
    $result = &DBDelete($kCategoryDB, $kDocDBPath);
    $result = $result || &DBDelete($kCategoryMapDB, $kDocDBPath);
    $result = $result || &DBDelete($kDocumentDB, $kDocDBPath);
    $result = $result || &DBDelete($kGlossaryDB, $kDocDBPath);
    $result = $result || &DBDelete($kURLIDMapDB, $kDocDBPath);
    $result = $result || &DBDelete($kIDDocsMapDB, $kDocDBPath);
    $result = $result || &DeleteAllUMDBs();
    return $result;
}
1;
