# $Id: Export.pl,v 1.2 1998/07/09 02:24:28 naga Exp $
# Copyright InfoPlace Inc., Milpitas, CA, USA, 1996-1998
# Not to be copied, modified or redistributed without written
# permission of InfoPlace.
#
# --------------------------------------------------------------------------------

require "DocAPI.pl";
require "DocUtils.pl";
require "UMAPI.pl";

# --------------------------------------------------------------------------------
sub WriteVersion
{
	my($fp) = @_;
	my($tvalue) = &GetTimeStamp();
	print $fp "# Export Version: ${kExportVersion} -- DO NOT EDIT THIS LINE\n";
	print $fp "# Date:  ${tvalue}\n";
}

# --------------------------------------------------------------------------------
# 
sub WriteCategories
{
    my($file, $catid) = @_;
    open(CFILE, ">$file") || &LogFileOpenFailMsg($file);
	&WriteVersion("CFILE");
    print CFILE "# The format is:\n# CATEGORY-NAME<TAB>PARENT-CATEGORY-NAME<TAB>DESCRIPTION#\n# CATEGORY-NAME field is mandatory.\n# Any other fields can have a no value, but the tabs need to be there.\n#\n";
    my(@db) = ();
    my($result) = &DBOpen($kCategoryDB, $kDocDBPath, $kDBReadAccess, \@db);
	if ($result == $errNone) {
		if ($catid == $kRootCategoryID) {
			my(@record) = ();
			$result = &DBRetreiveRecord(\@db, $catid, \@record);
			my(@childlist) = split($kValueSeparator, $record[$kCategoryChildListIndex]);
			foreach $child (@childlist) {
				$result = &ExportCategories(\@db, "CFILE", $child, $child, "");
			}
		}
		else {
			$result = &ExportCategories(\@db, "CFILE", $catid, $catid, "");
		}
	}
    &DBClose(\@db);
    close(CFILE);
    return $result;	
}

# --------------------------------------------------------------------------------
sub ExportCategories
{
	local(*db, $fp, $catid, $rootid, $path) = @_;
	my(@record) = ();
	$result = &DBRetreiveRecord(*db, $catid, \@record);
	my($parent) = $path;
	my($name) = $record[$kCategoryNameIndex];
	print $fp $name, $kImportExportFieldSeparator, 
	$parent, $kImportExportFieldSeparator, 
	$record[$kCategoryDescIndex], "\n";
	my(@childlist) = split($kValueSeparator, $record[$kCategoryChildListIndex]);
	if ($path ne "") {
		$path = $path . $kCatagoryPathSeparator;
	}
	$path = $path . $name;
	foreach $child (@childlist) {
		$result = &ExportCategories(*db, $fp, $child, $rootid, $path);
	}
	return $result;
}



# --------------------------------------------------------------------------------
sub WriteDocuments
{
	my($file, $catid) = @_;
	my($result);
	open(DFILE, ">$file") || &LogFileOpenFailMsg($file);
	&WriteVersion("DFILE");
	print DFILE "# The format is:\n# DOCUMENT-NAME<TAB>PARENT-CATEGORY-NAMES<TAB>AUTHOR-NAME<TAB>DOC-URL<TAB>DOC-TYPE<TAB>DESCRIPTION<TAB>KEYWORDS\n#\n# DOCUMENT-NAME, DOC-URL, DOC-TYPE, PARENT-CATEGORY-NAMES fields are mandatory.\n# Any other fields can have a no value, but the tabs need to be there.\n#\n";
	my(@docdb, @catdb) = ((), ());
	$result = &DBOpen($kDocumentDB, $kDocDBPath, $kDBReadAccess, \@docdb);
	$result = &DBOpen($kCategoryDB, $kDocDBPath, $kDBReadAccess, \@catdb);
	my(%exportedDocs) = ();
	if ($result == $errNone) {
		if ($catid == $kRootCategoryID) {
			my(@record) = ();
			$result = &DBRetreiveRecord(\@catdb, $catid, \@record);
			my(@childlist) = split($kValueSeparator, $record[$kCategoryChildListIndex]);
			foreach $child (@childlist) {
				$result = &ExportDocuments(\@docdb, \@catdb, "DFILE", $child, $catid, "", \%exportedDocs);
			}
		}
		else {
			$result = &ExportDocuments(\@docdb, \@catdb, "DFILE", $catid, $catid, "", \%exportedDocs);
		}
	}
}

# --------------------------------------------------------------------------------
sub ExportDocuments
{
	local(*docdb, *catdb, $fp, $catid, $rootid, $path, *exportedDocs) = @_;	
	my(@record) = ();
	$result = &DBRetreiveRecord(*catdb, $catid, \@record);
	my($name) = $record[$kCategoryNameIndex];
	if ($path ne "") {
		$path = $path . $kCatagoryPathSeparator;
	}
	$path = $path . $name;
	my(@catlist) = split($kValueSeparator, $record[$kCategoryChildListIndex]);
	my(@doclist) = split($kValueSeparator, $record[$kCategoryDocIDListIndex]);
	foreach $cat (@catlist) {
		&ExportDocuments(*docdb, *catdb, $fp, $cat, $rootid, $path, *exportedDocs);
	}
	my($rootfullpath) = "";
	my(@rootrecord) = ();
	foreach $doc (@doclist) {
		if (!$exportedDocs{$doc}) {
			$result = &DBRetreiveRecord(*docdb, $doc, \@record);
			my($plist) = "";
			my(@parents) = split($kValueSeparator, $record[$kDocCategoryListIndex]);
			if ($#parents > 0) { # document has more than one parents ...
				my($separator) = "";
				foreach $parent (@parents) {
					if ($parent != $catid) {
						if ($rootfullpath eq "") {
							$rootfullpath = &GetCategoryPath($rootid, 1, 0);
							&OPGetCategory($rootid, \@rootrecord);
						}
						my($parentfullpath) = &GetCategoryPath($parent, 1, 0);
						if ($parentfullpath =~ /^($rootfullpath)(.*)/) {
							my($matchedpath) = $2;
							my($rootName) = $rootrecord[$kCategoryNameIndex];
							$plist = $plist . $separator . $rootName . $matchedpath;
							$separator = $kImportExportValueSeparator;
						}
					}
					else {
						$plist = $plist . $separator . $path;
						$separator = $kImportExportValueSeparator;
					}
				}
			}
			else {
				$plist = $path;
			}
			print $fp $record[$kDocNameIndex], $kImportExportFieldSeparator, 
			$plist, $kImportExportFieldSeparator,
			$record[$kDocAuthorIndex], $kImportExportFieldSeparator,
			$record[$kDocURLIndex], $kImportExportFieldSeparator,
			$record[$kDocAppTypeIndex], $kImportExportFieldSeparator,
			$record[$kDocDescriptionIndex], $kImportExportFieldSeparator,
			$record[$kDocKeywordsIndex], "\n";
			$exportedDocs{$doc} = 1;
		}
	}
	return $result;
}

# --------------------------------------------------------------------------------
sub WriteGlossaries
{
	my($file) = @_;
	open(GFILE, ">$file") || &LogFileOpenFailMsg($file);
	&WriteVersion("GFILE");
	print GFILE "# The format is:\n# TERM<TAB>DEFINITION\n#\n# All fields are mandatory.\n";
	my($result);
	my(@db) = ();
	$result = &DBOpen($kGlossaryDB, $kDocDBPath, $kDBReadAccess, \@db);
	if ($result == $errNone) {
		my(@glossary) = ();
		$result = &DBReadFirstRecord(\@db, \@glossary);
		while (!$result) {
			print GFILE $glossary[$kGlossaryTermIndex], $kImportExportFieldSeparator, 
			$glossary[$kGlossaryDefinitionIndex], "\n";
			$result = &DBReadNextRecord(\@db, \@glossary);
		}
	}
	&DBClose(\@db);
	close(GFILE);
	return $result;	
}

# --------------------------------------------------------------------------------
sub WriteUsers
{
    my($file) = @_;
    open(UFILE, ">$file") || &LogFileOpenFailMsg($file);
	&WriteVersion("UFILE");
    print UFILE "# The format is:\n# USERID<TAB>PASSWORD<TAB>PREVILAGE\n#\n# All fields are mandatory.\n# Previlage: can be either:\n#   1 for read-only access\n#   3 for read and modify access\n#   7 for all above + admin access\n";
    my($result);
    my(@db) = ();
    $result = &DBOpen($kUserDB, $kDocDBPath, $kDBReadAccess, \@db);
    if ($result == $errNone) {
		my(@userRecord) = ();
		$result = &DBReadFirstRecord(\@db, \@userRecord);
		while (!$result) {
			if ($userRecord[$kUserUserIDIndex] ne $kAdminUserID) {
				print UFILE $userRecord[$kUserUserIDIndex], $kImportExportFieldSeparator, 
				$userRecord[$kUserPasswordIndex], $kImportExportFieldSeparator, 
				$userRecord[$kUserTypeIndex], "\n";
			}
			$result = &DBReadNextRecord(\@db, \@userRecord);
		}
    }
    &DBClose(\@db);
    close(UFILE);
    return $result;	
}

# --------------------------------------------------------------------------------
sub WriteUsersProfiles
{
    my($file) = @_;
    open(UFILE, ">$file") || &LogFileOpenFailMsg($file);
	&WriteVersion("UFILE");
    print UFILE "# The format is:\n# USERID<TAB>FULL-NAME<TAB>EMAIL\n#\n# All fields are mandatory.\n";
    my($result);
    my(@db) = ();
    $result = &DBOpen($kUserProfileDB, $kDocDBPath, $kDBReadAccess, \@db);
    if ($result == $errNone) {
		my(@uProfile) = ();
		$result = &DBReadFirstRecord(\@db, \@uProfile);
		while (!$result) {
			if ($uProfile[$kUserProfileUserIDIndex] ne $kAdminUserID) {
				print UFILE $uProfile[$kUserProfileUserIDIndex], $kImportExportFieldSeparator, 
				$uProfile[$kUserProfileNameIndex], $kImportExportFieldSeparator, 
				$uProfile[$kUserProfileEmailIndex],"\n";
			}
			$result = &DBReadNextRecord(\@db, \@uProfile);
		}
    }
    &DBClose(\@db);
    close(UFILE);
    return $result;	
}


# --------------------------------------------------------------------------------
sub WriteGroups
{
    my($file) = @_;
    open(GFILE, ">$file") || &LogFileOpenFailMsg($file);
	&WriteVersion("GFILE");
    print GFILE "# The format is:\n# GROUP-NAME<TAB>LIST-OF-USERIDS-COMMA-SEPERATED\n#\n# All fields are mandatory.\n";
    my($result);
    my(@db) = ();
    $result = &DBOpen($kGroupDB, $kDocDBPath, $kDBReadAccess, \@db);
    if ($result == $errNone) {
		my(@groupRecord) = ();
		$result = &DBReadFirstRecord(\@db, \@groupRecord);
		while (!$result) {
			if ($groupRecord[$kGroupNameIndex] ne $kAdminGroupID) {
				print GFILE $groupRecord[$kGroupNameIndex], $kImportExportFieldSeparator, 
				$groupRecord[$kGroupUserIDListIndex], "\n";
			}
			$result = &DBReadNextRecord(\@db, \@groupRecord);
		}
    }
    &DBClose(\@db);
    close(GFILE);
    return $result;	
}

1;
