#!/usr/bin/local/perl
##*****************************************************************************
##
##     File Name:	FileLock.pl
##         Owner:	InfoPlace
##     Copyright:	InfoPlace Inc., Pleasanton, CA, USA, 1996.
##     				Not to be copied, modified or redistributed without written
##     				permission of InfoPlace.
##
##
##*****************************************************************************
##                      R E V I S I O N   H I S T O R Y
##*****************************************************************************
##
##	  Date		Time	Author	Description
##	--------	-----	------	---------------------------------------------
##	09/15/96	12:51	N		Original version
##
##*****************************************************************************


# --------------------------------------------------------------------------------
# Globals and Constants
# 

# flock parameters
$Lock_SH = 1;
$Lock_EX = 2;
$Lock_NB = 4;
$Lock_UN = 8;

# fcntl parameters
$flk_t ="ssllLii";

$F_GETLK = 5;
$F_SETLK = 6;
$F_SETLKW = 7;

$F_RDLCK = 1;
$F_WRLCK = 2;
$F_UNLCK = 3;

$kIgnoreFileLock = 0;
$kUseFileLock = 1;  # otherwise use fcntl lock

# --------------------------------------------------------------------------------
# File locking utility functions
#

sub SetIgnoreFileLock 
{
	my($value) = @_;
	my($oldValue) = $kIgnoreFileLock;
	$kIgnoreFileLock = $value;
	return $oldValue;
}

sub GetReadLockOnFile
{
	(!$kIgnoreFileLock) || return 1;
	my($fileHandle) = @_;
	if ($kUseFileLock) {
		flock($fileHandle, $Lock_SH);
	}
	else {
		my(@ary) = (0, 0, 0, 0, 0, 0);
		$ary[0] = $F_RDLCK;  # lock type
		$ary[1] = $ary[2] = $ary[3] = 0;  # whole file
		my($flk) = pack($flk_t, @ary);
		# return fcntl($fileHandle, $F_SETLKW, $flk);
	}
	return 1;
}

sub GetWriteLockOnFile
{
	(!$kIgnoreFileLock) || return 1;
	my($fileHandle) = @_;
	if ($kUseFileLock) {
		flock($fileHandle, $Lock_EX);
	}
	else {
		my(@ary) = (0, 0, 0, 0, 0, 0);
		$ary[0] = $F_WRLCK;  # lock type
		$ary[1] = $ary[2] = $ary[3] = 0;  # whole file
		my($flk) = pack($flk_t, @ary);
		# return fcntl($fileHandle, $F_SETLKW, $flk);
	}
	return 1;
}

sub UnlockFile 
{
	(!$kIgnoreFileLock) || return 1;
	my($fileHandle) = @_;
	if ($kUseFileLock) {
		flock($fileHandle, $Lock_UN);
	}
	else {
		my(@ary) = (0, 0, 0, 0, 0, 0);
		$ary[0] = $F_UNLCK;
		my($flk) = pack($flk_t, @ary);
		# return fcntl($fileHandle, $F_SETLK, $flk);
	}
	return 1;
}

# --------------------------------------------------------------------------------
# Testing ...
#

1;
