# $Id: Import.pl,v 1.4 1998/07/09 02:24:28 naga Exp $
# Copyright InfoPlace Inc., Milpitas, CA, USA, 1996-1998
# Not to be copied, modified or redistributed without written
# permission of InfoPlace.
#
# --------------------------------------------------------------------------------

require "DocAPI.pl";
require "DocUtils.pl";
require "UMAPI.pl";
require "UMProfileAPI.pl";

# --------------------------------------------------------------------------------
sub ExtractVersion
{
	my($line) = @_;
	if ($line =~ /^\# Export Version\: (.*) \-\-/) {
		return $1;
	}
	return "";
}

# --------------------------------------------------------------------------------
# 
sub ReadCategories
{
    my($file, $rootid) = @_;
    
    open(CFILE, "<$file") || &LogFileOpenFailMsg($file);
    my($lineCount) = 0;
    my($line);
    local($/) = "\n";
	$line = <CFILE>;
	$lineCount++;
	chop($line);
	my($version) = &ExtractVersion($line);
	if ($version eq $kExportVersion1_1) {
		&ReadCategories1_1Format("CFILE", $lineCount, $rootid); 
	}
	elsif (($version eq  $kExportVersion) || ($version eq  $kExportVersion1_2)) {
		&ReadCategories1_2Format("CFILE", $lineCount, $rootid); 
	}
	else {
		&LogMsg("Error: File: $file Line: $lineCount - Invalid version information");
	}
	close(CFILE);
}

# --------------------------------------------------------------------------------
sub ReadCategories1_1Format
{
	my($fp, $lineCount, $rootid) = @_;
	my($line);
	my(@record) = ();
    while (!eof($fp)) {
		$line = <$fp>;
		$lineCount++;
		chop($line);
		if (!($line =~ /^#.*/)) {
			  @record = split($kImportExportFieldSeparator, $line);
			  if ($#record > 3) { 
				  &LogMsg("Warning: File: $file Line: $lineCount is ill formatted.");
			  }
			  else {
				  my($parent) = $record[1];
				  if (!defined($parent) || $parent eq "") {
					  $parent = $rootid;
				  }
				  else {
					  $parent = &GetCategoryIDFromName($parent);
				  }
				  if ($parent) {
					  if (&OPAddCategory($record[0], $parent, $record[2])) {
						  &LogMsg("Warning: File: $file Line: $lineCount Category create error.");
					  }
				  }
				  else {
					  &LogMsg("Warning: File: $file Line: $lineCount Category create error - could not resolve parent category.");
				  }
			  }
		}
	}
}

# --------------------------------------------------------------------------------
sub ReadCategories1_2Format
{
	my($fp, $lineCount, $rootid) = @_;
	my($line);
	my(@record) = ();
	my($rootpath) = &GetCategoryPath($rootid, 1, 0);
	if ($rootpath ne "") {
		$rootpath .= $kCatagoryPathSeparator;
	}
    while (!eof($fp)) {
		$line = <$fp>;
		$lineCount++;
		chop($line);
		if (!($line =~ /^#.*/)) {
			  @record = split($kImportExportFieldSeparator, $line);
			  if ($#record > 2) { 
				  &LogMsg("Warning: File: $file Line: $lineCount is ill formatted.");
			  }
			  else {
				  my($parent) = $record[1];
				  if (!defined($parent) || $parent eq "") {
					  $parent = "$rootid";
				  }
				  else {
					  $parent = &GetCategoryIDFromPath($rootpath . $parent);
				  }
				  if ($parent) {
					  if (&OPAddCategory($record[0], $parent, $record[2])) {
						  &LogMsg("Warning: File: $file Line: $lineCount Category create error.");
					  }
				  }
				  else {
					  &LogMsg("Warning: File: $file Line: $lineCount Category create error - could not resolve parent category.");
				  }
			  }
		}
	}
	close(CFILE);
}
	
# --------------------------------------------------------------------------------
sub ReadDocuments
{
    my($file, $rootid) = @_;
    
    open(DFILE, "<$file") || &LogFileOpenFailMsg($file);
    my($lineCount) = 0;
    my($line);
    local($/) = "\n";
	$line = <DFILE>;
	$lineCount++;
	chop($line);
	my($version) = &ExtractVersion($line);
	if ($version eq $kExportVersion) {
		&ReadDocuments1_3Format("DFILE", $lineCount, $rootid); 
	}
	elsif ($version eq $kExportVersion1_2) {
		&ReadDocuments1_2Format("DFILE", $lineCount, $rootid); 
	}
	elsif ($version eq $kExportVersion1_1) {
		&ReadDocuments1_1Format("DFILE", $lineCount, $rootid); 
	}
	else {
		&LogMsg("Error: File: $file Line: $lineCount - Invalid version information");
	}
	close(DFILE);
}

# --------------------------------------------------------------------------------
sub ReadDocuments1_1Format
{
	my($fp, $lineCount, $rootid) = @_;
	my($line);
	my(@record) = ();
    while (!eof($fp)) {
		$line = <$fp>;
		$lineCount++;
		chop($line);
		if (!($line =~ /^#.*/)) {
			  @record = split($kImportExportFieldSeparator, $line);
			  if ($#record > 7) { 
				  &LogMsg("Warning: File: $file Line: $lineCount is ill formatted.");
			  }
			  else {
				  my(@restargs) = ($record[1], $record[2], $record[3], $record[4], 1, 0, $record[7], $record[6]);
				  my($parents) = $record[5];
				  if (!defined($parents) || $parents eq "") {
					  # $parents = $rootid;  ??? root cannot have documents ...
				  }
				  else {
					  my(@plist) = split($kValueSeparator, $parents);
					  my($i) = 0;
					  foreach $parent (@plist) {
						  $plist[$i++] = &GetCategoryIDFromName($parent);
					  }
					  $parents = join($kValueSeparator, @plist);
				  }
				  if ($parents) {
					  if (&OPAddDocument($record[0], $parents, \@restargs)) {
						  &LogMsg("Warning: File: $file Line: $lineCount Document create error.");
						  my($rest) = join("|#|", @restargs);
						  &LogMsg("Document Name: $record[0] Parent: $parents Rest: $rest");
					  }
				  }
				  else {
					  &LogMsg("Warning: File: $file Line: $lineCount Document create error - could not resolve parent category.");
				  }
			  }
		}
	}
}

# --------------------------------------------------------------------------------
sub ReadDocuments1_2Format
{
	my($fp, $lineCount, $rootid) = @_;
	my($line);
	my(@record) = ();
	my($rootpath) = &GetCategoryPath($rootid, 1, 0);
	if ($rootpath ne "") {
		$rootpath .= $kCatagoryPathSeparator;
	}
    while (!eof($fp)) {
		$line = <$fp>;
		$lineCount++;
		chop($line);
		if (!($line =~ /^#.*/)) {
			  #&LogMsg("ReadDocuments1_2Format: $line");
			  @record = split($kImportExportFieldSeparator, $line);
			  if ($#record > 6) { 
				  &LogMsg("Warning: File: $file Line: $lineCount is ill formatted.");
			  }
			  else {
				  my(@restargs) = ($record[2], $record[3], 0, $record[4], 1, 0, $record[6], $record[5]);
				  my($parents) = $record[1];
				  if (!defined($parents) || $parents eq "") {
					  # $parents = $rootid; ??? root cannot have documents ...
				  }
				  else {
					  # workaround the export bug that makes Main the parent of docs that originally had multiple parents
					  if ($parents =~ /\,(.+)/) {
						$parents = $1;
					  }
					  my(@plist) = split($kValueSeparator, $parents);
					  my($i) = 0;
					  foreach $parent (@plist) {
						$plist[$i++] = &GetCategoryIDFromPath($rootpath . $parent);
					  }
					  $parents = join($kValueSeparator, @plist);
				  }
				  if ($parents) {
					  if (&OPAddDocument($record[0], $parents, \@restargs)) {
						  &LogMsg("Warning: File: $file Line: $lineCount Document create error.");
					  }
				  }
				  else {
					  &LogMsg("Warning: File: $file Line: $lineCount Document create error - could not resolve parent category.");
				  }
			  }
		}
	}
}

# --------------------------------------------------------------------------------
sub ReadDocuments1_3Format
{
	my($fp, $lineCount, $rootid) = @_;
	my($line);
	my(@record) = ();
	my($rootpath) = &GetCategoryPath($rootid, 1, 0);
	if ($rootpath ne "") {
		$rootpath .= $kCatagoryPathSeparator;
	}
    while (!eof($fp)) {
		$line = <$fp>;
		$lineCount++;
		chop($line);
		if (!($line =~ /^#.*/)) {
			  #&LogMsg("ReadDocuments1_3Format: $line");
			  @record = split($kImportExportFieldSeparator, $line);
			  if ($#record > 6) { 
				  &LogMsg("Warning: File: $file Line: $lineCount is ill formatted.");
			  }
			  else {
				  my(@restargs) = ($record[2], $record[3], 0, $record[4], 1, 0, $record[6], $record[5]);
				  my($parents) = $record[1];
				  if (!defined($parents) || $parents eq "") {
					  # $parents = $rootid; ??? root cannot have documents ...
				  }
				  else {
					  my(@plist) = split($kImportExportValueSeparator, $parents);
					  my($i) = 0;
					  foreach $parent (@plist) {
						  if($parent =~ /^$rootpath/) { #if already has path from root
							$plist[$i++] = &GetCategoryIDFromPath($parent);
						  }
						  else {
							  $plist[$i++] = &GetCategoryIDFromPath($rootpath . $parent);
						  }
						  my($ztemp) = $plist[$i-1];
					  }
					  $parents = join($kValueSeparator, @plist);
				  }
				  if ($parents) {
					  if (&OPAddDocument($record[0], $parents, \@restargs)) {
						  &LogMsg("Warning: File: $file Line: $lineCount Document create error.");
					  }
				  }
				  else {
					  &LogMsg("Warning: File: $file Line: $lineCount Document create error - could not resolve parent category.");
				  }
			  }
		}
	}
}

# --------------------------------------------------------------------------------
sub ReadGlossaries
{
    my($file) = @_;
    
    open(GFILE, "<$file") || &LogFileOpenFailMsg($file);
    my($lineCount) = 0;
    local(@record, $line);
    local($/) = "\n";
    while (!eof(GFILE)) {
		$line = <GFILE>;
		$lineCount++;
		chop($line);
		if (!($line =~ /^#.*/)) {
			  @record = split($kImportExportFieldSeparator, $line);
			  if ($#record > 1) { 
				  &LogMsg("Warning: File: $file Line: $lineCount is ill formatted.");
			  }
			  else {
				  &OPAddGlossary($record[0], $record[1]);
			  }
		}
    }
    close(GFILE);
}

# --------------------------------------------------------------------------------
sub ReadUsers
{
    my($file) = @_;
    
    open(UFILE, "<$file") || &LogFileOpenFailMsg($file);
    my($lineCount) = 0;
    local(@uRecord, $line);
    local($/) = "\n";
    while (!eof(UFILE)) {
		$line = <UFILE>;
		$lineCount++;
		chop($line);
		if (!($line =~ /^#.*/)) {
			  @uRecord = split($kImportExportFieldSeparator, $line);
			  if ($#uRecord > $#kUserDBFields) { 
				  &LogMsg("Warning: File: $file Line: $lineCount is ill formatted.");
			  }
			  else {
				  my($i) = 0;
				  for ($i = ($#uRecord + 1); $i <= $#kUserDBFields; $i++) {
					  $uRecord[$i] = ""; #mark end of record
				  }
				  &UMAddUser(*uRecord);
			  }
		}
	}
    close(UFILE);
}

# --------------------------------------------------------------------------------
sub ReadUserProfiles
{
    my($file) = @_;
    
    open(UFILE, "<$file") || &LogFileOpenFailMsg($file);
    my($lineCount) = 0;
    local(@pRecord, $line);
    local($/) = "\n";
    while (!eof(UFILE)) {
		$line = <UFILE>;
		$lineCount++;
		chop($line);
		if (!($line =~ /^#.*/)) {
			  @pRecord = split($kImportExportFieldSeparator, $line);
			  if ($#pRecord > $#kUserProfileDBFields) { 
				  &LogMsg("Warning: File: $file Line: $lineCount is ill formatted.");
			  }
			  else {
				  my($i) = 0;
				  for ($i = ($#pRecord + 1); $i <= $#kUserProfileDBFields; $i++) {
					  $pRecord[$i] = ""; #mark end of record
				  }
				  &UMAddUserProfile(*pRecord);
			  }
		}
    }
    close(UFILE);
}

# --------------------------------------------------------------------------------
sub ReadGroups
{
    my($file) = @_;
    
    open(GFILE, "<$file") || &LogFileOpenFailMsg($file);
    my($lineCount) = 0;
    local(@uRecord, $line);
    local($/) = "\n";
    while (!eof(GFILE)) {
		$line = <GFILE>;
		$lineCount++;
		chop($line);
		if (!($line =~ /^#.*/)) {
			  @uRecord = split($kImportExportFieldSeparator, $line);
			  if ($#uRecord > $#kGroupDBFields) { 
				  &LogMsg("Warning: File: $file Line: $lineCount is ill formatted.");
			  }
			  else {
				  $result = &UMAddGroup($uRecord[0]);
				  if ($result == $errNone) {
					  local(@uList) = split(",", $uRecord[1]); #users list if supplied
					  foreach $u (@uList) {
						  &UMAddUserToGroup($uRecord[0], $u, 0, 0);
					  }
				  }
			  }
	   }
    }
    close(GFILE);
}

1;
