#!/usr/bin/perl
##*****************************************************************************
##
##     File Name:	LogMsg.pl
##         Owner:	InfoPlace
##     Copyright:	InfoPlace Inc., Pleasanton, CA, USA, 1996.
##     				Not to be copied, modified or redistributed without written
##     				permission of InfoPlace.
##
##
##*****************************************************************************
##                      R E V I S I O N   H I S T O R Y
##*****************************************************************************
##
##	  Date		Time	Author	Description
##	--------	-----	------	---------------------------------------------
##	09/15/96	12:52	N		Original version
##
##*****************************************************************************


require "FileLock.pl";
require "ctime.pl";

# --------------------------------------------------------------------------------
# Define a message/error logging scheme
# The messages may be logged to a file or to stderr, 
# this could be controlled by the user at runtime.

$kLogFile = "error.log";
$kLogToFileEnabled = 0;

# --------------------------------------------------------------------------------
#
sub LogToFileEnable
{
	local($logfile) = @_;
	if (defined($logfile)) {
		&SetLogFile($logfile);
	}
	local($oldValue) = $kLogToFileEnabled; 
	$kLogToFileEnabled = 1;
	return $oldValue;
}

# --------------------------------------------------------------------------------
#
sub LogToFileDisable
{
	local($oldValue) = $kLogToFileEnabled; 
	$kLogToFileEnabled = 0;
	return $oldValue;
}

# --------------------------------------------------------------------------------
#
sub SetLogFile
{
	local($file) = @_;
	$kLogFile = $file;
}

# --------------------------------------------------------------------------------
#
sub ResetLogFile
{
	umask 0000;
	open(LOGFILE, ">$kLogFile") || die "Error: Could not open log file $kLogFile: $!\n";
	&GetWriteLockOnFile("LOGFILE");
	close(LOGFILE);
	&UnlockFile("LOGFILE");
}

# --------------------------------------------------------------------------------
#
sub LogMsg 
{
    local($msg) = @_;
    my($timeValue) = &GetTimeStamp();
    
    if ($kLogToFileEnabled) {
		umask 0000;
		open(LOGFILE, ">>$kLogFile") || die "Error: Could not open log file $kLogFile: $!\n";
		&GetWriteLockOnFile("LOGFILE");
		if ($msg =~ /T1/) {
			sleep(10);
		}
		print LOGFILE "${timeValue}: $msg\n";
		close(LOGFILE);
		&UnlockFile("LOGFILE");
    }
    else {
		print STDERR "${timeValue}: $msg\n";
    }
    return 1;
}

# --------------------------------------------------------------------------------
#
sub LogFileOpenFailMsg 
{
	local($filename) = @_;
	&LogMsg("Failed to open file ${filename}: $!");
	return 1;
}

# --------------------------------------------------------------------------------
#
sub GetTimeStamp
{
    my($timeValue) = time;
    $timeValue = &ctime($timeValue);
    chop($timeValue);
	return $timeValue;
}

1;
