# $Id: OPFind.pl,v 1.2 1998/05/26 18:10:10 thyag Exp $
# Copyright InfoPlace Inc., Milpitas, CA, USA, 1996-1998
# Not to be copied, modified or redistributed without written
# permission of InfoPlace.
#
# --------------------------------------------------------------------------------

require "DBOpen.pl";
require "DBFind.pl";
require "SList.pl";
	
# --------------------------------------------------------------------------------
# OPFindCategory(@criteriaValue, @compareType, @categoryList)
# Returns the category records that match the specified fields in the category record.
# Args:
#	@criteriaValue	(list): category record value as a list. Here the fields not 
#						needing to be matched should have 'undefined' as value.
#						Following fields may be specified:
#						(name, description, parentCategory) 
#	@compareType (list): each element indicates how to compare corresponding field
#						 value specified in criteria list.
#			for all values:- 0: equal, -1: not equal
#			for string values:- 1:partial string with wild card prefix (*str)
#							    2:partial string with wild card suffix (str*)
#			for numeral values:- 1:greater than, 2: less than
#	@categoryList (list): a list of category records which match the specified criteria.
#
#	Return Value(boolean): 0 if find successful, 1 if not.

sub OPFindCategory
{
	local(*criteriaValue, *compareType, *categoryList, $orConditions) = @_;
	my($result);
	my(@db) = ();
	$result = &DBOpen($kCategoryDB, $kDocDBPath, $kDBReadAccess, \@db);
	if ($result == $errNone) {
		$result = &DBFindRecords(\@db, *criteriaValue, *compareType, *categoryList, $orConditions);
		&DBClose(\@db);
	}
	return $result;
}

# --------------------------------------------------------------------------------
# OPFindDocument(@criteriaValue, @compareType, @documentList)
# Returns the document records that match the specified fields in the document record.
# Args:
#	@criteriaValue	(list): document record value as a list. Here the fields not 
#						needing to be matched should have 'undefined' as value.
#						Following fields may be specified:
#						(name, category, author, url, size, appType, versionID, 
#						 keyWordlist, desc)
#	@compareType (list): each element indicates how to compare corresponding field
#						 value specified in criteria list.
#			for all values:- 0: equal, -1: not equal
#			for string values:- 1:partial string with wild card prefix (*str)
#							    2:partial string with wild card suffix (str*)
#			for numeral values:- 1:greater than, 2: less than
#	@documentList (list): a list of document record IDs which match the specified criteria.

#	Return Value(boolean): 0 if find successful, 1 if not.
sub OPFindDocument
{
	local(*criteriaValue, *compareType, *documentList, $orConditions) = @_;
	my($result);
	my(@db) = ();
	$result = &DBOpen($kDocumentDB, $kDocDBPath, $kDBReadAccess, \@db);
	if ($result == $errNone) {
		$result = &DBFindRecords(\@db, *criteriaValue, *compareType, *documentList, $orConditions);
		&DBClose(\@db);
	}
	return $result;
}

# --------------------------------------------------------------------------------
sub GetStringCriteria
{
	my($criteria) = @_;
	if ($criteria eq "Equal") {
		return 0;
	}
	if ($criteria eq "Contains") {
		return 3;
	}
	if ($criteria eq "Starts With") {
		return 2;
	}
	if ($criteria eq "Ends With") {
		return 1;
	}
	return -1;
}

# --------------------------------------------------------------------------------
sub GetNumberCriteria
{
	my($criteria) = @_;
	if ($criteria eq "Equal") {
		return 0;
	}
	if ($criteria eq "Less Than") {
		return 2;
	}
	if ($criteria eq "Greater Than") {
		return 1;
	}
	return -1;
}

