# $Id: PreLogin.pl,v 1.3 1998/06/24 02:50:24 naga Exp $
# Copyright InfoPlace Inc., Milpitas, CA, USA, 1996-1998
# Not to be copied, modified or redistributed without written
# permission of InfoPlace.
#
# --------------------------------------------------------------------------------

require "LogMsg.pl";

# --------------------------------------------------------------------------------
sub GetDBOptionList
{
    local(*dbassoc, $selectItem) = @_;
    my($select) = "";
	my(@sortedlist) = sort { lc($a) cmp lc($b) } (keys %dbassoc);
	my($dbname);
    foreach $dbname (@sortedlist) {
		$option = ($selectItem eq $dbname) ? "<OPTION SELECTED>" : "<OPTION>";
		$select = $select . "${option}${dbname}\n";
    }
    return $select;
}

# --------------------------------------------------------------------------------
# Read the specified config file and return values in assoc list
sub ReadSpecifiedConfigFile
{
	my($configFile, $assoc) = @_; #$assoc is passed in as a reference
	if (!open(CONFIGFILE, "<$configFile")) {
		&LogFileOpenFailMsg($configFile);
		&LogMsg("ReadSpecifiedConfigFile::Configuration file could not be opened !!!");
	}
	else {
		&GetReadLockOnFile("CONFIGFILE") || &LogMsg("ReadSpecifiedConfigFile:: failed on read lock on file $configFile: $!");
		my($line);
		%$assoc = (); #$assoc is passed in as a reference
		while (!eof(CONFIGFILE)) {
			$line = <CONFIGFILE>;
			chop($line);
			if ($line !~ /^[\s]*#/) { #not a comment line
				my(@pairs) = split("=", $line);
				$pairs[0] = ($pairs[0] =~ /^[\s]*(.+?)[\s]*$/) ? $1 : $pairs[0];#remove preceding/following spaces
				$pairs[1] = ($pairs[1] =~ /^[\s]*(.+?)[\s]*$/) ? $1 : $pairs[1];#remove preceding/following spaces
				if($pairs[1] eq "{") { #assume multi-line value; start of block is "{"
					my($blockLine) = <CONFIGFILE>; 
					chop($blockLine);
					$pairs[1] = "";
					while (!eof(CONFIGFILE) && 
						   ($blockLine ne "}")) {
						$pairs[1] = $pairs[1] . "\n" . $blockLine;
					}#while not end of block
					}#if block value
				$assoc->{$pairs[0]} = $pairs[1];
			}#if
			}#while
				close(CONFIGFILE);
		&UnlockFile("CONFIGFILE") || &LogMsg("ReadSpecifiedConfigFile:: failed on unlock on file $configFile: $!");
	}
}

# --------------------------------------------------------------------------------
# Write to a config file
sub WriteSpecifiedConfigFile
{
	my($configFile, $assoc) = @_; #$assoc is passed in as a reference
	if (!open(CONFIGFILE, ">$configFile")) {
		&LogFileOpenFailMsg($configFile);
		&LogMsg("Configuration file could not be opened !!!");
	}
	else {
		&GetWriteLockOnFile("CONFIGFILE") || &LogMsg("WriteSpecifiedConfigFile: failed on write lock on file $configFile: $!");
		print CONFIGFILE "# Config Version: ${kConfigVersion} -- DO NOT EDIT THIS LINE\n";
		print CONFIGFILE "# Format: <key>=<value>, all in a single line\n";
		print CONFIGFILE "# or for multiline values:\n# <key> = {\n#\t<value>\n#\t<more value lines>\n# }\n#\n";
		# print the value pairs
		my($key, $value);
		while (($key, $value) = each %$assoc) {
			print CONFIGFILE $key, "=", $value, "\n";
		}
		close(CONFIGFILE);
		&UnlockFile("CONFIGFILE") || &LogMsg("WriteSpecifiedConfigFile: failed on unlock on file $configFile: $!");
	}	
}

# --------------------------------------------------------------------------------
# Append to a config file
sub AppendConfigFile
{
	my($configFile, $assoc) = @_; #$assoc is passed in as a reference
	if (!open(CONFIGFILE, ">>$configFile")) {
		&LogFileOpenFailMsg($configFile);
		&LogMsg("Configuration file could not be opened !!!");
	}
	else {
		&GetWriteLockOnFile("CONFIGFILE") || &LogMsg("AppendConfigFile: failed on write lock on file $configFile: $!");
		# print the value pairs
		my($key, $value);
		while (($key, $value) = each %$assoc) {
			print CONFIGFILE $key, "=", $value, "\n";
		}
		close(CONFIGFILE);
		&UnlockFile("CONFIGFILE") || &LogMsg("AppendConfigFile: failed on unlock on file $configFile: $!");
	}	
}

# --------------------------------------------------------------------------------
# Read the system.conf config file to get the infobase directories and
# get the infobase names from the respective infobase.conf files.
# system.conf has the following format:
# directory1=<dirpath to infobase 1>
# URL1=<url for infobase 1 docs dir>
# directory2=<dirpath to infobase 2>
# URL2=<url for infobase 2 docs dir>
# ...
# infobase.conf has the following format:
# name = full user readable name for infobase, can be multiple words with no quotes or slashes
# description = {
# This description also becomes the description for the base or main category.
# This value can be multiple lines block if needed.
# }
#
sub ReadDatabaseNames
{
    my($dbassoc, $dburl) = @_; # pass by reference
	my(%assoc) = ();
	# Read from the default Infobases dir first
	&ReadInfobasesDir($dbassoc, $dburl);

	# Now check for additional Infobases
#    &ReadSpecifiedConfigFile($kInfobasesConfigFile, \%assoc); #pass a reference to assoc
#	my(@keys) = keys %assoc;
#	while(@keys) {
#		my($dburlStr) = '';
#		my($dbPath) = '';
#		my($key) = pop(@keys);
#		if($key =~ /^[\w].*/) { #starts with an entry
#			if ($key =~ /^directory([0-9]+)$/) {
#				my($index) = $1;
#				$dbPath = $assoc{"directory" . $index};
#				my($dbName) = &ReadInfoBaseName($dbPath);
#				$dbassoc->{$dbName} = $dbPath;
#				$dburl->{$dbName} = $kDefaultDBURL;				
#			}
#		}
#	}
}#ReadDatabaseNames

sub ReadInfobasesDir
{
    my($dbassoc, $dburl) = @_; # pass by reference
    my($infobasesDir) = $kDefaultDBPath;
	unless (opendir(FILES, $infobasesDir)) {
		 &LogMsg("ReadInfobasesDir:: open $infobasesDir failed");
		 return;
	}
    my(@files);
    @files = readdir(FILES);
    closedir(FILES);
    my($i);
    my($filename);
    for ($i = 0; $i < $#files + 1; $i++) {
		$filename = $infobasesDir . "/". $files[$i];
		$dbPath = "$filename\/";
		#print "looking at: $filename...\n";
		if ((-d $filename) && ($files[$i] =~ /^DB[0-9]+$/) &&
			 (-d "$filename\/db") ) { #Real Infobase found
			 my($dbName) = &ReadInfoBaseName($dbPath);
			 $dbassoc->{$dbName} = $dbPath;
			 $dburl->{$dbName} = $kDefaultDBURL . "$filename\/";
		}
	}
}#ReadInfobasesDir
# --------------------------------------------------------------------------------
# Read the infobase.conf config file to get the infobase name
# infobase.conf has the following format:
# name = full user readable name for infobase, can be multiple words with no quotes or slashes
# description = {
# This description also becomes the description for the base or main category.
# This value can be multiple lines block if needed.
# }
#
sub ReadInfoBaseConfig {
    my($infobasePath, $assoc) = @_;# $assoc is a reference parameter
    &ReadSpecifiedConfigFile($infobasePath . $kIBInfobaseFileName, $assoc); #pass a reference to assoc
}

# --------------------------------------------------------------------------------
sub ReadInfoBaseName {
    my($infobasePath) = @_;
	my(%assoc) = ();
    &ReadSpecifiedConfigFile($infobasePath . $kIBInfobaseFileName, \%assoc); #pass a reference to assoc
	return $assoc{"name"} ? $assoc{"name"} : "";
}

# --------------------------------------------------------------------------------
sub ReadInfoBaseDescription {
    my($infobasePath) = @_;
	my(%assoc) = ();
    &ReadSpecifiedConfigFile($infobasePath . $kIBInfobaseFileName, \%assoc); #pass a reference to assoc
	return $assoc{"description"} ? $assoc{"description"} : "";
}

# --------------------------------------------------------------------------------
# Read the lastindex value from config/Infobases.conf
sub ReadLastDBDirIndex {
	my(%assoc) = ();
    &ReadSpecifiedConfigFile($kInfobasesConfigFile, \%assoc); #pass a reference to assoc
	return $assoc{"lastindex"};
}

# --------------------------------------------------------------------------------
sub ReadSystemConfigFile
{
	my(%sssoc) = ();
	&ReadSpecifiedConfigFile($kSystemConfigFile, \%assoc);

	$kDefaultDBPath = $assoc{"InfobasePath"} if $assoc{"InfobasePath"};
	$kDefaultDBURL = $assoc{"InfobaseURL"} if $assoc{"InfobaseURL"};
	$kDocExportBasePath = $assoc{"ExportPath"} if $assoc{"ExportPath"};	
	$kSystemDefaultDocColumns = $assoc{"DocumentColumns"} if $assoc{"DocumentColumns"};	 

}

# --------------------------------------------------------------------------------
sub ReadInfobaseConfigFile
{
	my(%assoc);
	&ReadSpecifiedConfigFile($kIBDefaultConfigFile, \%assoc);
	$kDefaultDocColumnNames = $assoc{"DocumentColumns"} if $assoc{"DocumentColumns"};
	$kUploadMaxFileSize = $assoc{"UploadMaxFileSize"} if $assoc{"UploadMaxFileSize"};
}

# --------------------------------------------------------------------------------
sub WriteInfobaseConfigFile
{
	my(%assoc);
	$assoc{"DocumentColumns"} = $kSystemDefaultDocColumns;
	$assoc{"UploadMaxFileSize"} = $kUploadMaxFileSize;
	&WriteSpecifiedConfigFile($kIBDefaultConfigFile, \%assoc);
}

# --------------------------------------------------------------------------------
sub EncodeCookieValue
{
  my($value) = @_;
  return unpack("h*", $value);
  #return $value;
}

# --------------------------------------------------------------------------------
sub DecodeCookieValue
{
  my($value) = @_;
  return pack("h*", $value);
  #return $value;
}

# --------------------------------------------------------------------------------
# Write version file
sub WriteVersionFile
{
	if (!open(VERSIONFILE, ">$kVersionFile")) {
		&LogFileOpenFailMsg($kVersionFile);
		&LogMsg("Version file could be opened !!!");
	}
	else {
		&GetWriteLockOnFile("VERSIONFILE") || &LogMsg("WriteVersionFile: failed on write lock on file $kVersionFile: $!");
		print VERSIONFILE "${kProductVersion}\n${kProductFullVersion}\n";
		close(VERSIONFILE);
		&UnlockFile("VERSIONFILE") || &LogMsg("WriteVersionFile: failed on unlock on file $kVersionFile: $!");
	}	
}

1;

