# $Id: Search.pl,v 1.1 1998/05/04 19:50:02 naga Exp $
# Copyright InfoPlace Inc., Milpitas, CA, USA, 1996-1998
# Not to be copied, modified or redistributed without written
# permission of InfoPlace.
#
#------------------------------------------------------------
# Requires

require "DocAPI.pl";
require "UMProfileAPI.pl";
require "ShowDocuments.pl";

#------------------------------------------------------------
# Globals

$kSETempDir       = 'C:/temp';
$kSEVerityBuilder = '../bin/BuildSearchIndex';
$kSEVerityFinder = '../bin/SearchQuery';
$kSECollectionName = $kDocDBRootPath . "searchinfo/index";
$kSEExternalDocsPath = $kDocDBRootPath . "searchinfo/externaldocs/";
$kSEInternalMap = $kDocDBRootPath . "searchinfo/InternalMap";
$kSEExternalMap = $kDocDBRootPath . "searchinfo/ExternalMap";
$kSEIndexBuilderLog = $kDocDBRootPath . "searchinfo/BuildSearchIndex.log";
$kSEMapSeperator = "\t";
$kSEFindResultPrefix = $kDocDBRootPath . "searchinfo/FindResult";

#------------------------------------------------------------
# Identifys if the given URL is internal document or external.
sub IsInternalURL 
{
    my($url) = @_;
    if ($url =~ m|^docs/|) {
		return 1;
    }
    return 0;
}

#------------------------------------------------------------
# Generates a unique file name
sub SEGenerateFileName
{
    my($path, $seed) = @_;
    my($tpid) = time() * $$;  # time * pid
    my($name) = "T${tpid}${seed}";
    if (-f "${path}${name}") {
	return &SEGenerateFileName($path, $seed+1);
    }
    return $name;
}

#------------------------------------------------------------
# This actually builds the index for the given set of docs. The
# docs are listed in the in-file, the first field in the file
# corresponds to the doc (the record is $kSEMapSeperator seperated).
# The out-file has the log of the activity.

sub SEBuildSearchIndexForDocs
{
    my($collectionName, $inFile, $outFile) = @_;
    if (!open(INFILE, "<$inFile")) {
	&LogFileOpenFailMsg($inFile);
	return 0;
    }
    my($docFile, $line);
    while (!eof(INFILE)) {
	$line = <INFILE>;
	chop($line);
	my(@elements) = split($kSEMapSeperator, $line);
	$docFile = $elements[0];
	my($result) = system("..\\bin\\bi ${collectionName} ${docFile} ${outFile}");
	if ($result) {
	    &LogMsg("Return value of system call in SEBuildSearchIndexForDocs: $result");
	    return 0;
	}
    }
    close(INFILE);
    return 1;
}


#------------------------------------------------------------
# For all the documents in the system, build internal 
# and external Maps which maps url to filename. In case of
# interal they are the same, for external, the filenames are
# dynamically generated for each unique urls.

sub SEBuildMaps
{
    my($result);
    my(%internalMaps, %externalMaps);
    local(@db) = ();
    $result = &DBOpen($kDocumentDB, $kDocDBPath, $kDBReadAccess, *db);
    if ($result == $errNone) {
	local(@doc) = ();
	my($i) = 0;
	$result = &DBReadFirstRecord(*db, *doc);
	while (!$result) {
	    my($url) = $doc[$kDocURLIndex];
	    if (&IsInternalURL($url)) {
		$internalMaps{$url} = $kDocDBRootPath . $url;
	    }
	    else {
		$externalMaps{$url} = $kSEExternalDocsPath . &SEGenerateFileName($kSEExternalDocsPath, $i);
	    }
	    $result = &DBReadNextRecord(*db, *doc);
	    $i++;
	}
    }
    &DBClose(*db);
    
    # Now, write out the maps ...
    if (!open(MAPFILE, ">$kSEInternalMap")) {
	&LogFileOpenFailMsg($kSEInternalMap);
	return 0;
    }
    foreach $key (keys(%internalMaps)) {
	print MAPFILE "$internalMaps{$key}${kSEMapSeperator}${key}\n";
    }
    close(MAPFILE);

    if (!open(MAPFILE, ">$kSEExternalMap")) {
	&LogFileOpenFailMsg($kSEExternalMap);
	return 0;
    }
    foreach $key (keys(%externalMaps)) {
	print MAPFILE "$externalMaps{$key}${kSEMapSeperator}${key}\n";
    }
    close(MAPFILE);
    return 1;
}

#------------------------------------------------------------
# Now, for each external url, get the document and put them
# into a temporary directory for the purposes of building
# the search index

sub SEGetExternalURLs
{
    if (!open(MAPFILE, "<$kSEExternalMap")) {
	&LogFileOpenFailMsg($kSEExternalMap);
	return 0;
    }
    my(%externalMaps);
    while (!eof(MAPFILE)) {
	$line = <MAPFILE>;
	chop($line);
	my(@elements) = split($kSEMapSeperator, $line);
	$externalMaps{$elements[1]} = $elements[0];
    }
    close(MAPFILE);

    foreach $key (keys(%externalMaps)) {
	# do the HTTP GET operation, on URL and output goes into a file ...
	# ????
    }
    return 1;
}

#------------------------------------------------------------
# Build the index for all documents

sub SECompleteSearchIndex
{
    my($result) = &SEBuildMaps();
    $result = $result && &SEGetExternalURLs();
    $result = $result && &SEBuildSearchIndexForDocs($kSECollectionName, $kSEInternalMap, $kSEIndexBuilderLog);
    $result = $result && &SEBuildSearchIndexForDocs($kSECollectionName, $kSEExternalMap, $kSEIndexBuilderLog);
    return $result;
}



#------------------------------------------------------------
# Read in a Map file
sub SEReadMapFile
{
    my($mapFile, $refMaps) = @_;

    if (!open(MAPFILE, "<$mapFile")) {
	&LogFileOpenFailMsg($mapFile);
	return 0;
    }
    while (!eof(MAPFILE)) {
	$line = <MAPFILE>;
	chop($line);
	my(@elements) = split($kSEMapSeperator, $line);
	$refMaps->{$elements[0]} = $elements[1];
    }
    close(MAPFILE);
}

#------------------------------------------------------------
# Build the index for all documents

sub SEFindMatchedDocs
{
    my($query, $refDocIDs, $refWeights) = @_;
    @docIDs = ();
    @weights = ();
    my($outFile) = "${kSEFindResultPrefix}$$";
    my($cmd) = "..\\bin\\sq \"${query}\" ${kSECollectionName}  ${outFile}";
    my ($result) = system($cmd);
    if ($result) {
	&LogMsg("Return value of system call in SEFindMatchedDocs: $result");
	return 0;
    }
    # read in the maps
    my(%internalMaps, %externalMaps);
    &SEReadMapFile($kSEInternalMap, \%internalMaps);
    &SEReadMapFile($kSEExternalMap, \%externalMaps);

    # parse the out-file, and get the corresponding doc-ids
    if (!open(OUTFILE, "<$outFile")) {
	&LogFileOpenFailMsg($outFile);
	return 0;
    }
    my($url, $wt, $file);
    my($firstTime) = 1;
    my($i) = 0;
    while (!eof(OUTFILE)) {
	if ($firstTime) {
	    $line = <OUTFILE>;
	    $line = <OUTFILE>;
	    $firstTime = 0;
	}
	$line = <OUTFILE>;
	if (!$line) {
	    close(OUTFILE);
	    return 1;
	}
	chop($line);
	my(@elements) = split($kSEMapSeperator, $line);
	$wt = $elements[0];
	$file = $elements[1];
	#  $file =~ tr/\\/\//;
	$url = $internalMaps{$file};
	if (!$url) {
	    # consider external map ...
	    $url = $externalMaps{$file};	    
	}
	if (!$url) {
	    &LogMsg("Found file '$file' from search result but could not file corresponding URL");
	}
	local(@docs) = ();
	my($count) = &OPGetDocsForURL($url, 0, *docs);
	foreach $doc (@docs) {
	   $refWeights->[$i] = $wt;
	   $refDocIDs->[$i] = $doc;
	   $i++;
	}
    }
    close(OUTFILE);
    return 1;
}

#------------------------------------------------------------
# List result of search ...
sub ShowFindResult
{
    local(*assocList, *documents, $userID) = @_;
    my($templateFile) = $kDocDocPagePath . $kDocFindDocumentResultTemplate;
    my($totalCells, $numCellsPerRow) = (0, 5);
    my($result);

   if ($#documents +1 == 0) {
		$assocList{"Result"} = "No documents matched the given search criteria.";
		$assocList{"NoDocuments"} = "";
		$assocList{"TABLEEXISTS"} = 0;
    }
    else {
		$assocList{"Result"} = "";
		# Look at user profile to determine columns
		local ($columns) = "";
		$result = &UMGetDocColumnsFromProfile($userID, *columns);
		if ($columns eq "") {
			$columns = $kDefaultDocColumnNames;
		}
		my(@columnList) = split("\,", $columns);
		foreach $c (@columnList) {
			$assocList{"COLUMN" . $c} = 1;
		}
		$numCellsPerRow = $#columnList + 1;
		my($docList) = join($kValueSeperator, @documents);
		&GenDocTableCells($docList, *assocList, "");
		$totalCells = $numCellsPerRow * ($#documents +1);
		$assocList{"Result"} = ($#documents + 1) . " documents found.";
		$assocList{"NoDocuments"} = "";
		$assocList{"TABLEEXISTS"} = 1;
    }
    &GenerateHTMLFromTemplate($templateFile, $totalCells, $numCellsPerRow, *assocList, $kDocDocPagePath);
}

#------------------------------------------------------------
# Testing code
if (0) {
    &SECompleteSearchIndex();
}
if (0) {
    @docs = ();
    @weights = ();
    &SEFindMatchedDocs("consulting", \@docs, \@weights);
    $c = $#docs + 1;
    print "Found $c docs\n\n";
}


1;



