# $Id: SetGlobals.pl,v 1.6 1998/07/14 22:03:55 naga Exp $
# Copyright InfoPlace Inc., Milpitas, CA, USA, 1996-1998
# Not to be copied, modified or redistributed without written
# permission of InfoPlace.
#
# --------------------------------------------------------------------------------

# NOTE: Any CGI which needs to execute without valid session, should 'require'
#       this file. This sets all Infobase-independent globals appropriately.
# --------------------------------------------------------------------------------
#
require "PreLogin.pl";
require "BaseDocGlobals.pl";

# --------------------------------------------------------------------------------
#
sub SetAllGlobals {
	my($ibroot, $iburl) = @_;
	&SetIBGlobals($ibroot, $iburl);
	&ReadInfobaseConfigFile() if (-f $kIBDefaultConfigFile);
}

# --------------------------------------------------------------------------------
# THIS FUNCTION IS CALLED AT THE END OF THIS FILE
sub SetNonIBGlobals {
	my($doroot) = @_;
	&SetDefaults();
	&SetDOGlobals($doroot);
	&ReadSystemConfigFile();
}

# --------------------------------------------------------------------------------
# The function sets ALL global variables appropriatele for a given 
# Infobase path

# EVERY CGI WILL CALL THIS FUNCTION, ONCE IT GETS THE INFOBASE PATH
# FROM THE COOKIE. OTHERS, WHO WANT TO SWITCH TO DIFFERENT INFOBASE
# COULD ALSO CALL IT.

sub SetIBGlobals {
	my($ibroot, $iburl) = @_;

	$kDocDBRootPath = $ibroot;
	$kDocDBRootURL = $iburl;
	$kErrorLogFile = $kDocDBRootPath . "error.log";
	$kIBAnonymousTemplatePath =  $kDocDBRootPath . 'templates/';
	&LogToFileEnable($kErrorLogFile);

	$kIBDefaultConfigFile = $kDocDBRootPath . $kIBDefaultFileName;
	$kIBInfobaseConfigFile = $kDocDBRootPath . $kIBInfobaseFileName;
	$kVersionFile = $kDocDBRootPath . $kVersionFileName;
	$kDocDBPath = $kDocDBRootPath . "db/";  
	$kDocUsrDocPath = $kDocDBRootPath . "docs/"; 
	$kUMDBPath = $kDocDBPath;

	# Lock File Names
	$kCatDocLockFile = $kDocDBPath . "CatDocLock";
	$kGlossaryLockFile = $kDocDBPath . "GlossaryLock";

	$kMasterSessionIDFile = $kUMDBPath . "MasterSessionID";
	$kMasterDocIDFile = $kDocDBPath . "MasterDocID";
	$kMasterCatIDFile = $kDocDBPath . "MasterCatID";
	$kMasterURLIDFile = $kDocDBPath . "MasterURLID";

	$kBlockSystem = $kDocDBRootPath . "BlockSystem";
}

# --------------------------------------------------------------------------------
# The function sets ALL global variables appropriatele for a given 
# code base path. These variables are independent of current
# Infobase

# EVERY CGI WILL CALL THIS FUNCTION TO SET ALL GLOBALS 
# WHICH DEPENDS ON THE LOCATION OF THE CODE. THE LOCATION OF
# THE CODE IS OF COURSE SET AT INSTALLATION TIME.

sub SetDOGlobals {
	my($doroot) = @_;
	
	$kDocRootPath = $doroot;
	$kUMRootPath = $doroot;
	$kDocPagePath = $kDocRootPath . 'pages/';
	$kDocTmpPath = $kDocRootPath . 'tmp/';
	$kDocErrPagePath = $kDocPagePath . 'errpages/';
	$kDocMsgPagePath = $kDocPagePath . 'msgpages/';
	$kDocDocPagePath = $kDocPagePath . 'docpages/';	
	$kAnonymousTemplatePath =  $kDocPagePath . 'templates/';
	$kInfobasesConfigFile = $kDocRootPath . '../config/infobases.conf';
	$kSystemConfigFile = $kDocRootPath . '../config/system.conf';
	$kDocExportBasePath = $kDocRootPath . '../exports/';

	# default infobases directory & url
	$kDefaultDBPath = $kDocRootPath . "../infobases/";
	$kDefaultDBURL = "/docorg/infobases/";

	# Mime types
	$kMimeTypesFile = $kDocRootPath . '../config/mime.types';
	$kGetDocCGIURL = '../cgi-bin/GetDocument.pl/docName/';
	$kGetDocDescCGIURL = '../cgi-bin/GetDocumentDesc.pl?DocID=';

	# set up the error file, until infobase resets it
	$kErrorLogFile = $kDocRootPath . "error.log";
	&LogToFileEnable($kErrorLogFile);
}

# --------------------------------------------------------------------------------
# The function sets ALL configurable global variables to 
# default values.

sub SetDefaults {
    # default max upload file size
	$kUploadMaxFileSize = 10*1024*1024; 
	# defult document columns to show
	$kDefaultDocColumnNames = $kSystemDefaultDocColumns;
	# mandatory document column
	$kMandatoryDocColumnNames = "Name";
	# default document url .. .. no more used ????
	$kDefaultDocumentURL = "";

	# Global Assoc settings ...
	%gAssocList = ();
    # Controls the template name shown on each page
	$gAssocList{"ShowTemplate"} = 0;
    # Product Name & version info
	$gAssocList{"ProductName"} = $kProductName;
	$gAssocList{"ProductVersion"} = $kProductVersion;
	$gAssocList{"ProductFullVersion"} = $kProductFullVersion;

	# Cookie settings ...
    # If Cookies are used set the following variable to the 
	# cookie string to be included in HTTP header (except 
	# the Set-Cookie part) example: $gUseCookie = 
	# "cookie_1 = value1; expires=Wdy, DD-Mon-YYYY HH:MM:SS GMT"

	# session cookie value 
	$gSessionCookie = "";
	# previous login cookie value 
	$gLoginCookie= "";

	# session cookie name
	$kLoadedSessionIDCookie = "CKID";
	# previous userid/dbname cookie name
	$kPreviousLoginCookie = "CKUserID";
	# used in gAssocList to associate current session id and user id
	$kSessionIDCookie = "CKsessionID";
	$kUserIDCookie = "CKuserID";
	$kUserAccessCookie = "CKuseraccessID";
	$kAnonymousUserCookie = "CKAnonymousUserID";
	$kLogoutURLCookie = "CKLogoutURLID";
	$kAnonymousLogoutCloseCookie = "CKAnonymousLogoutCloseID";
}

# --------------------------------------------------------------------------------
# Initialize all non-infobase related globals ...

# Define where the code is installed 
$kDocOrgInstallDirectory = "../";

&SetNonIBGlobals($kDocOrgInstallDirectory);

1;
