# $Id: ShowDocuments.pl,v 1.3 1998/07/09 02:30:43 thyag Exp $
# Copyright InfoPlace Inc., Milpitas, CA, USA, 1996-1998
# Not to be copied, modified or redistributed without written
# permission of InfoPlace.
#
# --------------------------------------------------------------------------------

require "LogMsg.pl";
require "CGIUtility.pl";
require "DocAPI.pl";
require "DocUtils.pl";
require "UMProfileAPI.pl";
require "Listing.pl";

# --------------------------------------------------------------------------------
# The following routine will generate an associated list to represent
# each document in the table view as a row.
# Each cell contains an attribute of a document.
# We have chosen attributes: Name, Size, AppType, Author, Description
# The %valueList is used to return the values for these with keywords using the cell#:
# eg. Name0 = Presentations; Size0 = 512K; 
#	  Author0 = "Winston Matthew"; 
# The Name string has a link: URL0 = /tmp/p1.htm
# The last argument $edit, indicates whether to present a editable view
sub GenDocTableCells
{	#cells are: Name, Size, AppType, Author
	# we need URL field for Link
	local($docList, *valueList, $whatsNewTime) = @_;
	my($count) = &SListLength($docList, $kValueSeperator);
	my($i) = 0; 
	my(@documentList);
	for ($i = 0; $i < $count; $i++) {
		my(@document) = ();
		my($docID) = &SListNthElement($docList, $i, $kValueSeperator);
		$result = &OPGetDocument($docID, \@document);
		if ($result != $errNone) {
			&LogMsg("Could not get the document for id: ${docID}");
		}
		$documentList[$i] = &SListMake(\@document, $kFieldSeperator);
	}
	@documentList = &SortRecordsOnField(\@documentList, $kDocNameIndex + 1);
	&AddDocumentsToTableCells(\@documentList, $count, 0, *valueList, $whatsNewTime);
}

sub ShowDocuments
{
	local($id, $edit, *assocList, $path, $userID, $templateFile, $includePath, $publishMode) = @_;
	my(@category) = ();
	if (!defined($templateFile)) {
		$templateFile = $kDocDocPagePath . $kDocCategoryDocsTemplate;
	}
	$includePath = $kDocDocPagePath if (!defined($includePath));
	if (!defined($id)) {
		$id = $kRootCategoryID;
	}
	my(@docList) = ();
	my($result) = &OPGetCategory($id, \@category);
	if($result == $errNone) {
		my($name) = $category[$kCategoryNameIndex];
		my($totalCells, $numCellsPerRow) = (1, 1);
		$assocList{"CategoryName"} = $name;
		$assocList{"EscapedCategoryName"} = &EncodeForURL($name);
		$assocList{"CategoryDescription"} = $category[$kCategoryDescIndex] || "";
		
		# Look at user profile to determine columns
		my($columns) = "";
		$result = &UMGetDocColumnsFromProfile($userID, \$columns);
		my($whatsNewTime) = &UMComputeUsersWhatsNewTime($userID);
		if ($columns eq "") {
			$columns = $kDefaultDocColumnNames;
		}
		my(@columnList) = split("\,", $columns);
		foreach $c (@columnList) {
			$assocList{"COLUMN" . $c} = 1;
		}
		my($docList) = $category[$kCategoryDocIDListIndex];
		&GenDocTableCells($docList, *assocList, $whatsNewTime);
		$numCellsPerRow = $#columnList + 1;
		$totalCells = $numCellsPerRow * &SListLength($docList, $kValueSeperator);
		$assocList{"CategoryParent"} = "";
		$assocList{"DocumentParent"} = $id;
		if ($totalCells <= 0) {
			$assocList{"CategoryName"} = $name;
			$assocList{"EscapedCategoryName"} = &EncodeForURL($name);
			$assocList{"NoDocuments"} = "No documents";
			$numCellsPerRow = 1;
		}
		else {
			$assocList{"NoDocuments"} = "";
			$assocList{"TABLEEXISTS"} = 1;
		}
		$assocList{"CategoryPath"} = ($publishMode ? &XGetCategoryLinkPath($id) : &GetCategoryLinkPath($id));
		&GenerateHTMLFromTemplate($templateFile, $totalCells, $numCellsPerRow, *assocList, $includePath);
	}
	elsif ($result == $errEntryNotFound) {  # category not found ...
		# it is possible that the category is stale, no more there ...
		&RedirectURL("../cgi-bin/StaleObject.pl?obj=Category");
		exit 0;
	}
	else {
		my($errPage) = $kDocErrPagePath . $kErrInternalErrorTemplate;
		my(%assocList) = ("Body", "Category ${name} not found!");
		&GenerateHTMLPage($errPage, \%assocList, $kDocErrPagePath);
	}
}

1;


