#!/usr/bin/perl
##*****************************************************************************
##
##     File Name:	TrackCoverage.pl
##         Owner:	InfoPlace
##     Copyright:	InfoPlace Inc., Pleasanton, CA, USA, 1996.
##     				Not to be copied, modified or redistributed without written
##     				permission of InfoPlace.
##
##
##*****************************************************************************
##                      R E V I S I O N   H I S T O R Y
##*****************************************************************************
##
##	  Date		Time	Author	Description
##	--------	-----	------	---------------------------------------------
##	04/21/98	12:52	N		Original version
##
##*****************************************************************************


require "FileLock.pl";
require "ctime.pl";

# --------------------------------------------------------------------------------
# Define a message/error logging scheme
# The messages may be logged to a file or to stderr, 
# this could be controlled by the user at runtime.

$kCoverageLogFile = "../coverage.log";
$kCoverageLogToFileEnabled = 0;

# --------------------------------------------------------------------------------
#
sub CoverageLogToFileEnable
{
	local($logFile) = @_;
	if (defined($logFile)) {
		&SetCoverageLog($logFile);
	}
	local($oldValue) = $kCoverageLogToFileEnabled; 
	$kCoverageLogToFileEnabled = 1;
	return $oldValue;
}

# --------------------------------------------------------------------------------
#
sub CoverageLogToFileDisable
{
	local($oldValue) = $kCoverageLogToFileEnabled; 
	$kCoverageLogToFileEnabled = 0;
	return $oldValue;
}

# --------------------------------------------------------------------------------
#
sub SetCoverageLog
{
	local($file) = @_;
	$kCoverageLogFile = $file;
	if (!(-e $file)) {
		&ResetCoverageLog;
	}
}

# --------------------------------------------------------------------------------
#
sub ResetCoverageLog
{
	umask 0000;
	open(COVERAGELOG, ">$kCoverageLogFile") || die "Error: Could not open log file $kCoverageLogFile: $!\n";
	&GetWriteLockOnFile("COVERAGELOG");
	close(COVERAGELOG);
	&UnlockFile("COVERAGELOG");
}

# --------------------------------------------------------------------------------
#
sub CoverageLogMsg 
{
    local($msg) = @_;
    my($timeValue) = &GetTimeStamp();
    
    if ($kCoverageLogToFileEnabled) {
		umask 0000;
		open(COVERAGELOG, ">>$kCoverageLogFile") || die "Error: Could not open log file $kCoverageLogFile: $!\n";
		&GetWriteLockOnFile("COVERAGELOG");
		if ($msg =~ /T1/) {
			sleep(10);
		}
		print COVERAGELOG "${timeValue}: $msg\n";
		close(COVERAGELOG);
		&UnlockFile("COVERAGELOG");
    }
    else {
		print STDERR "${timeValue}: $msg\n";
    }
    return 1;
}

# --------------------------------------------------------------------------------
#
sub getIncrementedEntry
{
	my($entryid, $cnt) = @_;
	return "$entryid = " . ($cnt + 1);
} 
# --------------------------------------------------------------------------------
#
sub AddToCoverageLog
{
    local($moduleName) = @_;
    my($timeValue) = &GetTimeStamp();
    
    if ($kCoverageLogToFileEnabled) {
		umask 0000;
		open(COVERAGELOG, "+<$kCoverageLogFile") || die "Error: Could not open log file $kCoverageLogFile: $!\n";
		&GetWriteLockOnFile("COVERAGELOG");
		if ($msg =~ /T1/) {
			sleep(10);
		}
		local($/);
		$data = <COVERAGELOG>; #read the file
		seek(COVERAGELOG, 0, 0);
		print COVERAGELOG "Last Updated:${timeValue}\n";
		
		$data =~ s/^Last Updated:.+?\n//im; # delete first line
		if($data =~ /${moduleName}\s+\=\s+([\d]+)/) {
			$data =~ s/${moduleName}\s+\=\s+([\d]+)/&getIncrementedEntry($moduleName, $1)/ie; 
			print COVERAGELOG $data;
		}
		else {
			print COVERAGELOG $data , "\n" , &getIncrementedEntry($moduleName, 0);
		}
		close(COVERAGELOG);
		&UnlockFile("COVERAGELOG");
    }
    else {
		print STDERR "${timeValue}: $msg\n";
    }
    return 1;
}

# --------------------------------------------------------------------------------
#
sub CoverageLogOpenFailMsg 
{
	local($filename) = @_;
	&CoverageLogMsg("Failed to open file ${filename}: $!");
	return 1;
}

# --------------------------------------------------------------------------------
#
sub GetTimeStamp
{
    my($timeValue) = time;
    $timeValue = &ctime($timeValue);
    chop($timeValue);
	return $timeValue;
}

1;
