# $Id: UMAPI.pl,v 1.3 1998/06/22 20:36:57 naga Exp $
# Copyright InfoPlace Inc., Milpitas, CA, USA, 1996-1998
# Not to be copied, modified or redistributed without written
# permission of InfoPlace.
#
# --------------------------------------------------------------------------------

# This file contains the APIs to support the User Management

require "CGIUtility.pl";
require "DBOpen.pl";
require "DBLookUp.pl";
require "DBRemove.pl";
require "DBIterate.pl";
require "DBAdd.pl";
require "DocUtility.pl";

# --------------------------------------------------------------------------------
# UMAddUser( *userRecord )
# Adds a user record with specified $userID to the User DB. This doesn't encrypt password.
# Args:
#	@userRecord	(list): UserDB record
#							
#	Return Value(boolean): 0 if add successful, $errDuplicateCategory if already 
#							user by the same userID exists, $errOPFailed otherwise
sub UMAddUser {
    local(*userRecord) = @_;
    my($result) = $errNone;
    local(@userDB) = ();
    $result = &DBOpen($kUserDB, $kUMDBPath, $kDBWriteAccess, *userDB);
    if ($result == $errNone) {
	$result = &DBAppendRecordIfNotExists(*userDB, *userRecord);
	&DBClose(*userDB);
    }
    return $result;	
}


# --------------------------------------------------------------------------------
# UMGetUser($userID, *userRecord)
# return the user record
# Args:
#	$userID	(string): User's ID for which record is needed
#	@userRecord	(list): UserDB record
#							
#	Return Value(boolean): 0 if add successful, $errOPFailed otherwise
sub UMGetUser {
    local($userID, *userRecord) = @_;
    local(@userDB);
    my($result) = $errNone;
    $result = &DBOpen($kUserDB, $kUMDBPath, $kDBReadAccess, *userDB);
    $result = $result || &DBRetreiveRecord(*userDB, $userID, *userRecord);
    &DBClose(*userDB);
    return $result;	
}

# --------------------------------------------------------------------------------
#
sub UMGetAllUsers
{
    local(*users) = @_;
    @users = ();
    my($result);
    local(@db) = ();
    $result = &DBOpen($kUserDB, $kDocDBPath, $kDBReadAccess, *db);
    if ($result == $errNone) {
	local(@record) = ();
	$result = &DBReadFirstRecord(*db, *record);
	while (!$result) {
	    push(@users, $record[$kUserUserIDIndex]);
	    $result = &DBReadNextRecord(*db, *record);
	}
    }
    &DBClose(*db);
    return $result;
}


# --------------------------------------------------------------------------------
# UMUpdateUser($userID, *userRecord)
# Update the user-record, only non-null fields are updated. This doesn't encrypt password.
# Args:
#	$userID	(string): User's ID for which update needs to be done
#	@userRecord	(list): UserDB record
#							
#	Return Value(boolean): 0 if add successful, $errOPFailed otherwise
sub UMUpdateUser {
    local($userID, *userRecord) = @_;
    local(@origRecord, @userDB);
    my($result) = $errNone;
    $userRecord[$kUserUserIDIndex] = "";  # one cannot change the userid
    $result = &DBOpen($kUserDB, $kUMDBPath, $kDBWriteAccess, *userDB);
    $result = $result || &DBRetreiveRecord(*userDB, $userID, *origRecord);
    if ($result == $errNone) {
	my($i, $size);
	$size = $#userRecord + 1;
	for ($i = 0; $i < $size; $i++) {
	    if ($userRecord[$i]) {
		$origRecord[$i] = $userRecord[$i];
	    }
	}
	$result = &DBReplaceRecord(*userDB, $userID, *origRecord);		
    }
    &DBClose(*userDB);
    return $result;	
}

# --------------------------------------------------------------------------------
# UMUpdateUserPassword( $userID, $password )
# Updates the user record with specified $userID, with an encrypted form
# of the $password in the User DB.
# Args:
#	$userID 		(string): the userID of the user.
#	$password 		(string): the unencrypted password of the user.
#							
#	Return Value(boolean):  $errNone if update successful, 
#							$errOPFailed otherwise

sub UMUpdateUserPassword {
	my($userID, $password) = @_;
	local(@record);
	$record[$kUserPasswordIndex] = &EncryptString($password);
	$result = &UMUpdateUser($userID, *record);
	return $result;
}

# --------------------------------------------------------------------------------
# UMRemoveUser( $userID )
# Removes a user record with specified $userID from the User DB and UserProfileDB.
# Args:
#	$userID 		(string): the userID of the user.
#							
#	Return Value(boolean): 0 if remove successful, $errOPFailed otherwise

sub UMRemoveUser {
	my($userID) = @_;
	local(@record, @userDB);
	my($result) = $errNone;
	$result = &DBOpen($kUserDB, $kUMDBPath, $kDBWriteAccess, *userDB);
	$result = $result || &DBRetreiveRecord(*userDB, $userID, *record);
	if ($result == $errNone) {
		# first remove user from all parent groups
		my($parents) = $record[$kUserGroupListIndex];#comma delimited
		my(@groupList) = split($kValueSeperator, $parents);
		my($group);
		local(@groupDB);
		$result = &DBOpen($kGroupDB, $kUMDBPath, $kDBWriteAccess, *groupDB);
		if ($result == $errNone) {
		# remove userID from each group it belongs
			foreach $group (@groupList) {
				local(@groupRecord) = ();
				$result = &DBRetreiveRecord(*groupDB, $group, *groupRecord);
				local($memberUsers) = $groupRecord[$kGroupUserIDListIndex];
				$memberUsers = &SListRemove($memberUsers, $userID, $kValueSeperator);
				$groupRecord[$kGroupUserIDListIndex] = $memberUsers;
				$result = &DBReplaceRecord(*groupDB, $group, *groupRecord);		
			}
			&DBClose(*groupDB);
		}
		# end all active user sessions
		local($userSessionList) = &UMGetUserSessionList($userID);
		&UMEndSessionList($userSessionList) if $userSessionList; 

		#now remove the user record
		$result = &DBRemoveRecord(*userDB, $userID);
		&DBClose(*userDB);
		local(@userPDB);
		# remove from the UserPRofile DB as well !!
		$result = &DBOpen($kUserProfileDB, $kUMDBPath, $kDBWriteAccess, *userPDB);
		$result = &DBRemoveRecord(*userPDB, $userID);
		&DBClose(*userPDB);
 	}
	return $result;
}

# --------------------------------------------------------------------------------
# UMGetUserSessionList( $userID )
# Get Session ID for specified $userID.
# Args:
#	$userID	(string): User's ID for which session IDs are to be retreived
#							
#	Return Value(string): Session ID to be used for the user, 0 if no Session
sub UMGetUserSessionList {
	my($userID) = @_;
	local(@userRecord, @userDB);
	my($result) = $errNone;
	$result = &DBOpen($kUserDB, $kUMDBPath, $kDBWriteAccess, *userDB);
	$result = $result || &DBRetreiveRecord(*userDB, $userID, *userRecord);
	if ($result == $errNone) {
		my($temp) = $userRecord[$kUserSessionIDIndex];
		return ($temp ? $temp : 0);
	}
	&DBClose(*userDB);
	return 0;	
}


# --------------------------------------------------------------------------------
# UMSetUserSession( $userID, $sessionID, $userType)
# append a new Session in the user record for specified $userID.
# Args:
#	$userID	(string): User's ID for which session needs to be set
#	$sessionID	(string): Session ID to be used for the user
#	$userType	(integer): output, the user type
#							
#	Return Value(boolean): 0 if add successful, $errOPFailed otherwise
sub UMSetUserSession {
	local($userID, $sessionID, *userType) = @_;
	local(@userRecord, @userDB);
	my($result) = $errNone;
	$result = &DBOpen($kUserDB, $kUMDBPath, $kDBWriteAccess, *userDB);
	$result = $result || &DBRetreiveRecord(*userDB, $userID, *userRecord);
	if ($result == $errNone) {
		my($sessionIDtemp) = $userRecord[$kUserSessionIDIndex];
		$userRecord[$kUserSessionIDIndex] = ($sessionIDtemp ? 
											  $sessionIDtemp . $kValueSeperator . $sessionID :
											  $sessionID);
		$userType =  $userRecord[$kUserTypeIndex];
		if (!$userType) {
			$userType = $kReadOnlyMode;
		}
		$result = &DBReplaceRecord(*userDB, $userID, *userRecord);		
	}
	&DBClose(*userDB);
	return $result;	
}


# --------------------------------------------------------------------------------
# UMAddGroup( $groupName )
# Adds a users Group record with specified group name to the GroupDB.
# Args:
#	$groupName 		(string): the group name.
#							
#	Return Value(boolean): 0 if add successful, $errDuplicateCategory if already 
#							a group by the same name exists, $errOPFailed otherwise
sub UMAddGroup {
    my($groupName) = @_;
    my($result) = $errNone;
    local(@groupDB) = ();
    $result = &DBOpen($kGroupDB, $kUMDBPath, $kDBWriteAccess, *groupDB);
    if ($result == $errNone) {
	local (@record) = ();
	$record[$kGroupNameIndex] = $groupName;
	my($i) = 0;
	for ($i = ($#record + 1); $i <= $#kGroupDBFields; $i++) {
	    $record[$i] = ""; #mark end of record
	}
	$record[$kGroupRefCountIndex] = 0;
	$result = &DBAppendRecordIfNotExists(*groupDB, *record);
	&DBClose(*groupDB);
    }
    return $result;	
}

# --------------------------------------------------------------------------------
# 
sub UMGetGroup {
    local($group, *groupRecord) = @_;
    local(@db);
    my($result) = $errNone;
    $result = &DBOpen($kGroupDB, $kUMDBPath, $kDBReadAccess, *db);
    $result = $result || &DBRetreiveRecord(*db, $group, *groupRecord);
    &DBClose(*db);
    return $result;	
}

# --------------------------------------------------------------------------------
# 
sub UMUpdateGroup {
    local($group, *groupRecord) = @_;
    local(@origRecord, @db);
    my($result) = $errNone;
    $groupRecord[$kGroupNameIndex] = "";  # one cannot change the group name
    $result = &DBOpen($kGroupDB, $kUMDBPath, $kDBWriteAccess, *db);
    $result = $result || &DBRetreiveRecord(*db, $group, *origRecord);
    if ($result == $errNone) {
	my($i, $size);
	$size = $#groupRecord + 1;
	for ($i = 0; $i < $size; $i++) {
	    if ($groupRecord[$i]) {
		$origRecord[$i] = $groupRecord[$i];
	    }
	}
	$result = &DBReplaceRecord(*db, $group, *origRecord);		
    }
    &DBClose(*db);
    return $result;	
}


# --------------------------------------------------------------------------------
# UMRemoveGroup( $groupName )
# Removes a users Group record with specified group name from the GroupDB.
# Args:
#	$groupName 		(string): the group name.
#							
#	Return Value(boolean): 0 if remove successful, $errOPFailed otherwise

sub UMRemoveGroup {
	my($groupName) = @_;
	local(@record, @groupDB);
	my($result) = $errNone;
	$result = &DBOpen($kGroupDB, $kUMDBPath, $kDBWriteAccess, *groupDB);
	$result = $result || &DBRetreiveRecord(*groupDB, $groupName, *record);
	if ($result == $errNone) {
		# first remove occurence of the group from its member user records
		my($users) = $record[$kGroupUserIDListIndex];#comma delimited
		my(@userIDList) = split($kValueSeperator, $users);
		my($userID);
		local(@userDB);
		$result = &DBOpen($kUserDB, $kUMDBPath, $kDBWriteAccess, *userDB);
		if ($result == $errNone) {
		# remove group from each user it belongs
			foreach $userID (@userIDList) {
				local(@userRecord) = ();
				$result = &DBRetreiveRecord(*userDB, $userID, *userRecord);
				my($groups) = $userRecord[$kUserGroupListIndex];
				$groups = &SListRemove($groups, $groupName, $kValueSeperator);
				$userRecord[$kUserGroupListIndex] = $groups;
				$result = &DBReplaceRecord(*userDB, $userID, *userRecord);		
			}
			&DBClose(*userDB);
		}
		#now remove the group record
		$result = &DBRemoveRecord(*groupDB, $groupName);
		&DBClose(*groupDB);
 	}
	return $result;	
}


# --------------------------------------------------------------------------------
# UMAddUserToGroup( $groupName, $userID )
# Add a user to the Group record with specified group name in the GroupDB.
# Args:
#	$groupName 	(string): the group name.
#	$userID 	(string): the userID of the user.
#       $ignoreUserRecord (bool): if false then update user record to indicate the new
#                                 group else ignore update to user record
#							
#	Return Value(boolean): 0 if add successful, $errDuplicateCategory if already 
#							a group by the same name exists, $errOPFailed otherwise
sub UMAddUserToGroup {
    my($groupName, $userID, $ignoreGroupRecord, $ignoreUserRecord) = @_;
    local(@groupDB, @groupRecord);
    my($result) = $errNone;

    if (!$ignoreGroupRecord) {
	$result = &DBOpen($kGroupDB, $kUMDBPath, $kDBWriteAccess, *groupDB);
	$result = $result || &DBRetreiveRecord(*groupDB, $groupName, *groupRecord);
	if ($result == $errNone) {
	    # first add the userID to the group record
	    my($userIDList) = $groupRecord[$kGroupUserIDListIndex];#comma delimited
		$userIDList = &SListAppend($userIDList, $userID, $kValueSeperator);
	    $groupRecord[$kGroupUserIDListIndex] = $userIDList;
	    $result = &DBReplaceRecord(*groupDB, $groupName, *groupRecord);
	    &DBClose(*groupDB);
	}
    }
    if (!$ignoreUserRecord && ($result == $errNone)) {
	#now add group to the user's record as well
	local(@userDB, @userRecord);
	$result = &DBOpen($kUserDB, $kUMDBPath, $kDBWriteAccess, *userDB);
	$result = $result || &DBRetreiveRecord(*userDB, $userID, *userRecord);
	if ($result == $errNone) {
	    my($groupList) = $userRecord[$kUserGroupListIndex];
	    $groupList = &SListAppend($groupList, $groupName, $kValueSeperator);
	    $userRecord[$kUserGroupListIndex] = $groupList;
	    $result = &DBReplaceRecord(*userDB, $userID, *userRecord);
	}
	&DBClose(*userDB);	
    }
}

# --------------------------------------------------------------------------------
# UMRemoveUserFromGroup( $groupName, $userID )
# Removes a user from the Group record with specified group name from the GroupDB.
# Args:
#	$groupName 		(string): the group name.
#	$userID 	(string): the userID of the user.
#							
#	Return Value(boolean): 0 if remove successful, $errOPFailed otherwise

sub UMRemoveUserFromGroup {

    my($groupName, $userID, $ignoreGroupRecord, $ignoreUserRecord) = @_;
    local(@groupDB, @groupRecord);
    my($result) = $errNone;
    if (!$ignoreGroupRecord) {
	$result = &DBOpen($kGroupDB, $kUMDBPath, $kDBWriteAccess, *groupDB);
	$result = $result || &DBRetreiveRecord(*groupDB, $groupName, *groupRecord);
	if ($result == $errNone) {
	    # first remove the userID from the group record
	    my($userIDList) = $groupRecord[$kGroupUserIDListIndex];
	    $userIDList = &SListRemove($userIDList, $userID, $kValueSeperator);
	    $groupRecord[$kGroupUserIDListIndex] = $userIDList;
	    $result = &DBReplaceRecord(*groupDB, $groupName, *groupRecord);
	}
	&DBClose(*groupDB);
    }
    if (!$ignoreUserRecord) {
	#now remove the group from the user's record as well
	local(@userDB, @userRecord);
	$result = &DBOpen($kUserDB, $kUMDBPath, $kDBWriteAccess, *userDB);
	$result = $result || &DBRetreiveRecord(*userDB, $userID, *userRecord);
	if ($result == $errNone) {
	    my($groupList) = $userRecord[$kUserGroupListIndex];
	    $groupList = &SListRemove($groupList, $groupName, $kValueSeperator);
	    $userRecord[$kUserGroupListIndex] = $groupList;
	    $result = &DBReplaceRecord(*userDB, $userID, *userRecord);
	}
	&DBClose(*userDB);
    }
}

# --------------------------------------------------------------------------------
# UMGetAllGroupNames(@groups)
#
sub UMGetAllGroupNames
{
    local(*groups) = @_;
    @groups = ();
    my($result);
    local(@db) = ();
    $result = &DBOpen($kGroupDB, $kDocDBPath, $kDBReadAccess, *db);
    if ($result == $errNone) {
	local(@record) = ();
	$result = &DBReadFirstRecord(*db, *record);
	while (!$result) {
	    push(@groups, $record[$kGroupNameIndex]);
	    $result = &DBReadNextRecord(*db, *record);
	}
    }
    &DBClose(*db);
    return $result;
}

# --------------------------------------------------------------------------------
# UMVerifyUser( $userID, $password, $anonymous )
# Verifies whether a user record with specified $userID and password
# exists in the User DB.
# Args:
#	$userID 		(string): the userID of the user.
#	$password 		(string): the password of the user.
#							
#	Return Value(boolean):  $errNone if user with matching password found, 
#							$errInvalidPassword if password doesn't match
#							$errOPFailed otherwise

sub UMVerifyUser {
	my($userID, $password, $anonymous) = @_;

	local(@userDB, @userRecord);
	my($result) = $errNone;

	$result = &DBOpen($kUserDB, $kUMDBPath, $kDBWriteAccess, *userDB);
	$result = $result || &DBRetreiveRecord(*userDB, $userID, *userRecord);
	if ($result == $errNone) {
		&DBClose(*userDB);
		if ($anonymous && ($kAnonymousUserID eq $userID)) {
			return $errNone;
		}
		if (&EncryptString($password) eq $userRecord[$kUserPasswordIndex]) {
			return $errNone;
		}
#		elsif ($password eq $userRecord[$kUserPasswordIndex]) {#oldstyle no encoding
#			&LogMsg("UMVerifyUser:Old password format for user $userID, updating to new encrypted format");
#			#update with encoded password for future
#			&UMUpdateUserPassword($userID, $password);
#			return $errNone;
#		}
		else {
#			&LogMsg("UMVerifyUser:user $userID, password:$password, encpassword:" . &EncryptString($password));
#			&LogMsg("UMVerifyUser:db password:" . $userRecord[$kUserPasswordIndex] . " encpassword:" . &EncryptString($password));
#			&LogMsg("UMVerifyUser:" . ($userRecord[$kUserPasswordIndex] eq &EncryptString($password)));
			return $errInvalidPassword;
		}
	}
	return $result;
}

# --------------------------------------------------------------------------------
# UMNewSession( $userID )
# Start a new Session for specified $userID.
# Args:
#	$userID	(string): session for the specified user
#	*newSessionID	(string): used to return the new session ID
#							
#	Return Value(boolean): 0 if add successful, $errOPFailed otherwise
sub UMNewSession {
	local($userID, *newSessionID) = @_;
	my($result) = $errNone;
	local(@sessionDB) = ();
	local(@sessionRecord) = ();
	my($sessionID) = 0;
	$newSessionID = 0;
	$result = &DBOpen($kSessionDB, $kUMDBPath, $kDBWriteAccess, *sessionDB);
	if ($result == $errNone) {
	    $sessionID = &GenerateSessionID(&GetAndIncrementMasterID($kMasterSessionIDFile));
		local($userType);
		$result = $result || &UMSetUserSession($userID, $sessionID, *userType);
		if ($result == $errNone) {
			$sessionRecord[$kSessionSessionIDIndex] = $sessionID;
			local($currentTime) = time;
			$sessionRecord[$kSessionStartTimeIndex] = $currentTime;
			$sessionRecord[$kSessionUserIDIndex] = $userID;
			$sessionRecord[$kSessionModeIndex] = $userType;
			$sessionRecord[$kSessionMenuControlIndex] = $kDefaultMenuControlValue;
			$result = &DBAppendRecordIfNotExists(*sessionDB, *sessionRecord);
			&DBClose(*sessionDB);
			$newSessionID = $sessionID;
		}
		else {
			$newSessionID = 0;
		}
 	}
	return $result;	
}

# --------------------------------------------------------------------------------
# UMEndSession( $sessionID, $userID )
# End an on-going Session for specified $sessionID and set the $userID.
# Args:
#	$sessionID	(string): current session-id for the specified user
#	*userID	(string): used to return the userid for the given session ID
#							
#	Return Value(boolean): 0 if add successful, $errDuplicateCategory if already 
#							user by the same userID exists, $errOPFailed otherwise
sub UMEndSession {
	local($sessionID, *userID) = @_;
	my($result) = $errNone;
	local(@sessionDB) = ();
	local(@userDB) = ();
	local(@userRecord) = ();
	local(@sessionRecord) = ();

    $result = &DBOpen($kSessionDB, $kUMDBPath, $kDBWriteAccess, *sessionDB);
	$result = $result || &DBRetreiveRecord(*sessionDB, $sessionID, *sessionRecord);
	if ($result == $errNone) {
		$userID = $sessionRecord[$kSessionUserIDIndex];
		$result = &DBRemoveRecord(*sessionDB, $sessionID);
		&DBClose(*sessionDB);
		if ($result == $errNone) {
		    $result = &DBOpen($kUserDB, $kUMDBPath, $kDBWriteAccess, *userDB);
	        $result = $result || &DBRetreiveRecord(*userDB, $userID, *userRecord);
	        if ($result == $errNone) { # update the user record ....
#				if ($userRecord[$kUserLastLoginTimeIndex] < $sessionRecord[$kSessionStartTimeIndex]) {
						$userRecord[$kUserLastLoginTimeIndex] = $sessionRecord[$kSessionStartTimeIndex];
						$userRecord[$kUserSessionIDIndex] =~ s/${sessionID}${kValueSeperator}//; #remove if comma spearated
						$userRecord[$kUserSessionIDIndex] =~ s/${sessionID}//;#if last entry (no comma)
						$result = &DBReplaceRecord(*userDB, $userID, *userRecord);
#				}
				&DBClose(*userDB);				
            }
			&DBClose(*sessionDB);
		}
    }
	return $result;
}

# --------------------------------------------------------------------------------
# UMGetUserIDFromSessionID( $sessionID, $userID )
# Start a new Session for specified $userID.
# Args:
#	$sessionID	(string): current session-id for the specified user
#	*userID	(string): used to return the userid for the given session ID
#							
#	Return Value(boolean): 0 if add successful, $errOPFailed otherwise
sub UMGetUserIDFromSessionID {
    local($sessionID, *userID) = @_;
    my($result) = $errNone;
    local(@sessionDB) = ();
    local(@sessionRecord) = ();
    $result = &DBOpen($kSessionDB, $kUMDBPath, $kDBReadAccess, *sessionDB);
    $result = $result || &DBRetreiveRecord(*sessionDB, $sessionID, *sessionRecord);
    if ($result == $errNone) {
		$userID = $sessionRecord[$kSessionUserIDIndex];
    }
    &DBClose(*sessionDB);
    return $result;	
}

# --------------------------------------------------------------------------------
# UMValidateUserSession( $userID, $sessionID)
# Validates whether a user record with specified $userID has an acceptable sessionID
# Args:
#	$userID 		(string): the userID of the user.
#	$sessionID 		(string): the sessionID of the current session.
#							
#	Return Value(boolean):  $errNone if user with matching sessionID found, 
#							$errOPFailed otherwise

sub UMValidateUserSession {
    my($userID, $sessionID) = @_;
    local(@userDB, @userRecord);
    my($result) = $errNone;
    $result = &DBOpen($kUserDB, $kUMDBPath, $kDBWriteAccess, *userDB);
    $result = $result || &DBRetreiveRecord(*userDB, $userID, *userRecord);
    if ($result == $errNone) {
		&DBClose(*userDB);
		my($userSessionIDList) = $userRecord[$kUserSessionIDIndex];
		if (($sessionID eq $userSessionIDList) || 
			($userSessionIDList =~ /\,$sessionID/)) { #comma delimited string
			return $errNone;
		}#if
		else {
			return $errInvalidSession;
		}#else
    }#if
    return $result;
}

# --------------------------------------------------------------------------------
# UMEndSessionList( $sessionIDList )
# End on-going Sessions for specified sessionIDs. 
# NOTE: doesn't update corresponding User records!!!.
# Args:
#	$sessionIDList	(string): $kValueSeperator seperated session-id list
#							
#	Return Value(boolean): 0 if successful, $errOPFailed or other DB errors otherwise
sub UMEndSessionList {
	my($sessionIDList) = @_;
	my($result) = $errNone;
	local(@sessionDB) = ();
	my(@allSessions) = split($kValueSeperator, $sessionIDList);
	my($sessionID);
    $result = &DBOpen($kSessionDB, $kUMDBPath, $kDBWriteAccess, *sessionDB);
	if($result == $errNone) {
		foreach $sessionID (@allSessions) {
			local(@sessionRecord) = ();
			$result = &DBRetreiveRecord(*sessionDB, $sessionID, *sessionRecord);
			if ($result == $errNone) {
				$result = &DBRemoveRecord(*sessionDB, $sessionID);
			}
		}#foreach
		&DBClose(*sessionDB);
    }#if
	return $result;
}#UMEndSessionList

# --------------------------------------------------------------------------------
# UMEndObsoleteSessions( $userID )
# End old Sessions for specified user. 
# NOTE: Also updates corresponding User record!!!.
# Args:
#	$userID	(string): User's ID for which old sessions need to be deleted
#							
#	Return Value(boolean): 0 if successful, $errOPFailed or other DB errors otherwise
sub UMEndObsoleteSessions {
	my($userID) = @_;
	my($sessionIDList) = &UMGetUserSessionList($userID);
	my($result) = $errNone;
	if($sessionIDList && ($sessionIDList ne "")) {
		local(@sessionDB) = ();
		my(@allSessions) = split($kValueSeperator, $sessionIDList);
		my($sessionID);
		my($undeletedSessions) = "";
		$result = &DBOpen($kSessionDB, $kUMDBPath, $kDBWriteAccess, *sessionDB);
		if($result == $errNone) {
			foreach $sessionID (@allSessions) {
				local(@sessionRecord) = ();
				$result = &DBRetreiveRecord(*sessionDB, $sessionID, *sessionRecord);
				if ($result == $errNone) {
					my($sessionDate) = $sessionRecord[$kSessionStartTimeIndex];
					if ((time - $kMaxSessionLife*3600) > $sessionDate) { #if old enough to delete
						$result = &DBRemoveRecord(*sessionDB, $sessionID);
					}#if old enough
					else {
						$undeletedSessions = ($undeletedSessions ? 
												$undeletedSessions . $kValueSeperator .  $sessionID :
												$sessionID);
					}#else accumalate undeleted session ids
				}#if no error
			}#foreach
			&DBClose(*sessionDB);
			# now update the user record to contain any undeleted sessions only
			local(@userRecord) = ();
			$userRecord[$kUserSessionIDIndex] = $undeletedSessions;
			$result = &UMUpdateUser($userID, *userRecord);
		}#if DBOpen succeeded
		else{
			&LogMsg("UMEndObsoleteSessions::DBOpen failed");
		}
	}#if sessions exist
	return $result;
}#UMEndObsoleteSessions


1;
