#
##*****************************************************************************
##
##     File Name:	UMDB.pl
##         Owner:	InfoPlace
##     Copyright:	InfoPlace Inc., Pleasanton, CA, USA, 1996.
##     				Not to be copied, modified or redistributed without written
##     				permission of InfoPlace.
##
##
##*****************************************************************************
##                      R E V I S I O N   H I S T O R Y
##*****************************************************************************
##
##	  Date		Time	Author	Description
##	--------	-----	------	---------------------------------------------
##	09/15/96	12:51	N		Original version
##
##*****************************************************************************


# --------------------------------------------------------------------------------
# This file creates all Toy related DBs.
# --------------------------------------------------------------------------------

require "CGIUtility.pl";
require "DBCreate.pl";
require "DBOpen.pl";
require "DBAdd.pl";
require "UMGlobals.pl";

# --------------------------------------------------------------------------------
# Define UserDB - This keeps the User/Password records
#
sub DefineUserDB
{
    if (&DBCreate($kUserDB, $kUMDBPath, *kUserDBFields, *kUserDBFieldsTypes, $kMaxUserCount, 1)) {
	&LogMsg("Error: Failed to create the DB: $kUserDB");
	return 1;
    }
    # add the admin user ...
    local(@db) = ();
    local(@record) = ();
    my($result) = &DBOpen($kUserDB, $kUMDBPath, $kDBWriteAccess, *db);
    if (!$result) {
	$record[$kUserUserIDIndex] = $kAdminUserID;
	$record[$kUserPasswordIndex] = &EncryptString($kAdminUserID);
	$record[$kUserTypeIndex] = $kSuperAdminMode;
	$record[$kUserGroupListIndex] = $kAdminGroupID;
	$record[$kUserSessionIDIndex] = "";
	$record[$kUserLastLoginTimeIndex] = "";
	$result = &DBAppendRecord(*db, *record);
    }
    return $result;
}

# --------------------------------------------------------------------------------
# Initialize MasterSessionID to 1 in its file
#
sub InitMasterSessionID
{
    if (!open(SESSIONIDFILE, ">$kMasterSessionIDFile")) {
	&LogMsg("Error: Failed to create the file for master session-id index");
	return 1;
    }
    print SESSIONIDFILE "1\n";
    close(SESSIONIDFILE);
    return 0;
}

# --------------------------------------------------------------------------------
# Define GroupDB - This keeps the access control records
#
sub DefineGroupDB
{
    if (&DBCreate($kGroupDB, $kUMDBPath, *kGroupDBFields, *kGroupDBFieldsTypes, $kMaxGroupCount, 1)) {
	&LogMsg("Error: Failed to create the DB: $kGroupDB");
	return 1;
    }
    # add the admin user ...
    local(@db, @record);
    my($result) = &DBOpen($kGroupDB, $kUMDBPath, $kDBWriteAccess, *db);
    if (!$result) {
	$record[$kGroupNameIndex] = $kAdminGroupID;
	$record[$kGroupUserIDListIndex] = $kAdminUserID;
	$record[$kGroupRefCountIndex] = 1;
	$result = &DBAppendRecord(*db, *record);
    }
    return $result;
}


# --------------------------------------------------------------------------------
# Define SessionDB - This keeps the session related info records
#
sub DefineSessionDB
{
	if (&DBCreate($kSessionDB, $kUMDBPath, *kSessionDBFields, *kSessionDBFieldsTypes, $kMaxGroupCount, 1)) {
		&LogMsg("Error: Failed to create the DB: $kSessionDB");
    	return 1;
	}
    return 0;
}


# --------------------------------------------------------------------------------
# Define UserProfileDB - This keeps the user profile info records
#
sub DefineUserProfileDB
{
    if (&DBCreate($kUserProfileDB, $kUMDBPath, *kUserProfileDBFields, *kUserProfileDBFieldsTypes, $kMaxUserCount, 1)) {
	&LogMsg("Error: Failed to create the DB: $kUserProfileDB");
	return 1;
    }
    # add the admin user profile ...
    local(@db, @record);
    my($result) = &DBOpen($kUserProfileDB, $kUMDBPath, $kDBWriteAccess, *db);
    if (!$result) {
	$record[$kUserProfileUserIDIndex] = $kAdminUserID;
	$record[$kUserProfileNameIndex] = $kAdminName;;
	$record[$kUserProfileEmailIndex] = $kAdminEmail;
	$record[$kUserProfileOtherPrefsIndex] = "";
	$result = &DBAppendRecord(*db, *record);
    }
    return $result;
}

# --------------------------------------------------------------------------------
# Defines all UM DBs
#
sub DefineAllUMDBs
{
   local($result);
   $result = &DefineUserDB();
   $result = $result ||	&DefineGroupDB();
   $result = $result ||	&DefineSessionDB();
   $result = $result ||	&DefineUserProfileDB();
   $result = $result ||	&InitMasterSessionID();
   return $result;
}

# --------------------------------------------------------------------------------
# Delete all UM DBs
#
sub DeleteAllUMDBs
{
	local($result);
	$result = &DBDelete($kUserDB, $kUMDBPath);
	$result = $result || &DBDelete($kGroupDB, $kUMDBPath);
	$result = $result || &DBDelete($kSessionDB, $kUMDBPath);
	$result = $result || &DBDelete($kUserProfileDB, $kUMDBPath);
	return $result;
}
1;
