#
##*****************************************************************************
##
##     File Name:	UMGlobals.pl
##         Owner:	InfoPlace
##     Copyright:	InfoPlace Inc., Pleasanton, CA, USA, 1996.
##     				Not to be copied, modified or redistributed without written
##     				permission of InfoPlace.
##
##
##*****************************************************************************
##                      R E V I S I O N   H I S T O R Y
##*****************************************************************************
##
##	  Date		Time	Author	Description
##	--------	-----	------	---------------------------------------------
##	09/15/96	12:51	N		Original version
##
##*****************************************************************************


require "DBGlobals.pl";

# --------------------------------------------------------------------------------
# Define various User Management (UM) constants - REAL CONSTANTS (non-configurable)
# --------------------------------------------------------------------------------

# --------------------------------------------------------------------------------
# Constants

$kMaxUserCount = 1000;
$kMaxGroupCount = 100;
$kMaxSessionCount = 100;
$kMaxSessionLife = 8; # in hours
# --------------------------------------------------------------------------------
# DB Names

$kUserDB = "UserDB";
$kGroupDB = "GroupDB";
$kSessionDB = "SessionDB";
$kUserProfileDB = "UserProfileDB";

# --------------------------------------------------------------------------------
# DB Record Fields and Indices
#
# UserDB Fields:
@kUserDBFields = ("UserID", "Password", "Type", "GroupList", "SessionID", "LastLoginTime");
@kUserDBFieldsTypes = ($kStringTypeName, $kStringTypeName, $kNumericTypeName, $kListOfStringTypeName, 
						$kStringTypeName, $kNumericTypeName);
$kUserUserIDIndex = 0;
$kUserPasswordIndex = $kUserUserIDIndex + 1;
$kUserTypeIndex = $kUserPasswordIndex + 1;
$kUserGroupListIndex = $kUserTypeIndex + 1;
$kUserSessionIDIndex = $kUserGroupListIndex + 1;
$kUserLastLoginTimeIndex = $kUserSessionIDIndex + 1;

# -----------------------------------------------------
# GroupDB Fields:
@kGroupDBFields = ("Name", "UserIDList", "RefCount");

@kGroupDBFieldsTypes = ($kStringTypeName, $kListOfStringTypeName, $kNumericTypeName);

$kGroupNameIndex = 0; 
$kGroupUserIDListIndex = $kGroupNameIndex + 1; 
$kGroupRefCountIndex = $kGroupUserIDListIndex + 1;

# -----------------------------------------------------
# SessionDB Fields:
@kSessionDBFields = ("SessionID", "StartTime", "UserID", "Mode", "MenuControl");
@kSessionDBFieldsTypes = ($kStringTypeName, $kNumericTypeName, $kStringTypeName, 
							$kStringTypeName, $kNumericTypeName);
$kSessionSessionIDIndex = 0;
$kSessionStartTimeIndex = $kSessionSessionIDIndex + 1;
$kSessionUserIDIndex = $kSessionStartTimeIndex + 1;
$kSessionModeIndex = $kSessionUserIDIndex + 1;
$kSessionMenuControlIndex = $kSessionModeIndex + 1;

# defines the MenuControl field
$kMenuControlUserMode  = (1 << 0);
$kMenuControlPreferences = (1 << 1);
$kMenuControlModifications = (1 << 2);
$kMenuControlBrowse = (1 << 3);
$kMenuControlMoreBrowse = (1 << 4);
$kMenuControlAdmin = (1 << 5);
$kMenuControlCurrentIB = (1 << 6);
$kMenuControlAllIB = (1 << 7);

# indicates groups which by default needs to be collapsed
$kDefaultMenuControlValue = $kMenuControlCurrentIB | $kMenuControlAllIB;

# -----------------------------------------------------
# UserProfileDB Fields: (Note: The last column OtherPrefs is overloaded, 
# it is a place holder for other preferences too, this allows us to
# keep the same schema and add new preferences).
@kUserProfileDBFields = ("UserID", "Name", "Email", "DocColumns", "OtherPrefs");
@kUserProfileDBFieldsTypes = ($kStringTypeName, $kStringTypeName, $kStringTypeName, $kListOfStringTypeName, $kStringTypeName);
$kUserProfileUserIDIndex = 0;
$kUserProfileNameIndex = $kUserProfileUserIDIndex + 1;
$kUserProfileEmailIndex = $kUserProfileNameIndex + 1;
$kUserProfileDocColumnsIndex = $kUserProfileEmailIndex + 1;
$kUserProfileOtherPrefsIndex = $kUserProfileDocColumnsIndex + 1;

$kUMOtherPrefSeparator = '%';
$kUMWhatsNewIndex = 0;
$kUMDocumentViewIndex = 1;

# --------------------------------------------------------------------------------
# Misc. globals

$kRootSessionID = 1;

$kDefaultDocViewType = 1;
$kDefaultWhatsNewHours = 24;
$kUMDefaultOtherPrefs = $kDefaultWhatsNewHours . $kUMOtherPrefSeparator . $kDefaultDocViewType;

# Modes, also used for user type
$kReadOnlyMode = 1;
$kEditMode = 3;
$kAdminMode = 7;
$kSuperAdminMode = 15;

# Admin userid and group etc
$kAdminUserID = "admin"; #also the Super Admin User ID
$kAdminGroupID = "administrator";
$kAdminName = "Administrator";
$kAdminEmail = "admin\@infoplace.com";
$kSpecialGuestUser = "specialguest";

# --------------------------------------------------------------------------------
# Operations error code

$errUMNone = 0;
$errUMUndefinedUserID = 1;
$errUMUndefinedGroup = 2;
$errUMInvalidPassword = 3;
$errUMInvalidSessionID = 4;
$errUMSessionIDNotFound = 5;
$errDuplicateUser = 2;

# --------------------------------------------------------------------------------
# 

1;
