# $Id: UMProfileAPI.pl,v 1.1 1998/05/04 19:50:02 naga Exp $
# Copyright InfoPlace Inc., Milpitas, CA, USA, 1996-1998
# Not to be copied, modified or redistributed without written
# permission of InfoPlace.
#
# --------------------------------------------------------------------------------

# This file contains the APIs to support the User Profile Management

require "UMGlobals.pl";
require "CGIUtility.pl";
require "DBOpen.pl";
require "DBLookUp.pl";
require "DBRemove.pl";
require "DBAdd.pl";

# --------------------------------------------------------------------------------
# UMAddUserProfile( *userRecord )
# Adds a user record with specified $userID to the UserProfileDB.
# Args:
#	@userRecord	(list): UserProfileDB record
#							
#	Return Value(boolean): 0 if add successful, $errDuplicateCategory if already 
#							user by the same userID exists, $errOPFailed otherwise
sub UMAddUserProfile {
	local(*userRecord) = @_;
	my($result) = $errNone;
	local(@userDB) = ();
	$result = &DBOpen($kUserProfileDB, $kUMDBPath, $kDBWriteAccess, *userDB);
	if ($result == $errNone) {
		$result = &DBAppendRecordIfNotExists(*userDB, *userRecord);
		&DBClose(*userDB);
 	}
	return $result;	
}
 
# --------------------------------------------------------------------------------
# UMGetUserProfile( $userID, *record )
# Get from user record with specified $userID in the UserProfileDB, the Document Columns the user wants to view.
# Args:
#	$userID	(string): Id for user whose profile needs changed
#   *record (list): user profile record for the userID
#							
#	Return Value(boolean): 0 if got successfully
#			       $errOPFailed otherwise
sub UMGetUserProfile {
	local($userID, *record) = @_;
	my($result) = $errNone;
	local(@db) = ();
	$result = &DBOpen($kUserProfileDB, $kUMDBPath, $kDBReadAccess, *db);
	if ($result == $errNone) {
		$result = &DBRetreiveRecord(*db, $userID, *record);
		&DBClose(*db);
 	}
	return $result;	
}

# --------------------------------------------------------------------------------
# UMSetUserProfile( $userID, @record )
# Set the record to the given values for the given userid
# Args:
#	$userID	(string): Id for user whose profile needs changed
#       @record (list): user profile record
#							
#	Return Value(boolean): 0 if set successful, $errDuplicateCategory if already 
#		     user by the same userID exists, $errOPFailed otherwise
sub UMSetUserProfile {
  local($userID, *record) = @_;
  my($result) = $errNone;
  local(@userPDB) = ();
  local(@userPRecord) = ();
  $result = &DBOpen($kUserProfileDB, $kUMDBPath, $kDBWriteAccess, *userPDB);
  if ($result == $errNone) {
    $result = &DBRetreiveRecord(*userPDB, $userID, *userPRecord);
    if ($result == $errNone) {
      my($size) = $#record + 1;
      my($i) = 1; # ignore index 0, for userid
	for (; $i < $size; $i++) {
	  if (defined($record[$i])) {
	    $userPRecord[$i] = $record[$i];
	  }
	}
      $result = &DBReplaceRecord(*userPDB, $userID, *userPRecord);
    }
    &DBClose(*userDB);
  }
  return $result;	
}

# --------------------------------------------------------------------------------
# UMAddDocColumnsToProfile( $userID, $docColumns )
# Adds to user record with specified $userID in the UserProfileDB, the Document Columns the user wants to view.
# Args:
#	$userID	(string): Id for user whose profile needs changed
#       $docColumns (string): comma delimited list of column names
#							
#	Return Value(boolean): 0 if add successful, $errDuplicateCategory if already 
#							user by the same userID exists, $errOPFailed otherwise
sub UMAddDocColumnsToProfile {
	local($userID, $docColumns) = @_;
	my($result) = $errNone;
	local(@userPDB) = ();
	local(@userPRecord) = ();
	$result = &DBOpen($kUserProfileDB, $kUMDBPath, $kDBWriteAccess, *userPDB);
	if ($result == $errNone) {
		$result = &DBRetreiveRecord(*userPDB, $userID, *userPRecord);
		if ($result == $errNone) {
		  $userPRecord[$kUserProfileDocColumnsIndex] = $docColumns;
		  $result = &DBReplaceRecord(*userPDB, $userID, *userPRecord);
		}
		&DBClose(*userDB);
 	}
	return $result;	
}

# --------------------------------------------------------------------------------
# UMGetDocColumnsFromProfile( $userID, *docColumns )
# Get from user record with specified $userID in the UserProfileDB, the Document Columns the user wants to view.
# Args:
#	$userID	(string): Id for user whose profile needs changed
#       *docColumns (string): comma delimited list of column names
#							
#	Return Value(boolean): 0 if got successfully
#			       $errOPFailed otherwise
sub UMGetDocColumnsFromProfile {
	local($userID, *docColumns) = @_;
	my($result) = $errNone;
	local(@userPDB) = ();
	local(@userPRecord) = ();
	$result = &DBOpen($kUserProfileDB, $kUMDBPath, $kDBReadAccess, *userPDB);
	if ($result == $errNone) {
		$result = &DBRetreiveRecord(*userPDB, $userID, *userPRecord);
		if ($result == $errNone) {
		  $docColumns = $userPRecord[$kUserProfileDocColumnsIndex];
		}
		&DBClose(*userDB);
 	}
	return $result;	
}

# --------------------------------------------------------------------------------
#
sub UMGetUsersWhatsNewHours
{
    my($userid) = @_;
    my($whatsNewHours) = $kDefaultWhatsNewHours;
    my($result);
    local(@record);
    $result = &UMGetUserProfile($userid, *record);
    if ($result == $errNone) {
		$whatsNewHours = &UMGetOtherPrefs($record[$kUserProfileOtherPrefsIndex], $kUMWhatsNewIndex) || $kDefaultWhatsNewHours;
    }
    return $whatsNewHours;
}

# --------------------------------------------------------------------------------
#
sub UMComputeUsersWhatsNewTime
{
    my($userid, $hours) = @_;
    my($whatsNewHours) = (defined($hours)) ? $hours : &UMGetUsersWhatsNewHours($userid);
    return time() - $whatsNewHours*60*60;
}

# --------------------------------------------------------------------------------
#
sub UMGetOtherPrefsFromUserID
{
	my($userID, $index) = @_;
	my($result);
    local(@record);
    $result = &UMGetUserProfile($userid, *record);
    if ($result == $errNone) {
		return &UMGetOtherPrefs($record[$kUserProfileOtherPrefsIndex], $index);
	}
	return 0;
}

# --------------------------------------------------------------------------------
#
sub UMGetDocViewTypePrefFromUserID
{
	# for now we always force to new window 
	return "popup";
	my($userID) = @_;
	my($value) = &UMGetOtherPrefsFromUserID($userID, $kUMDocumentViewIndex);
	return ($value) ? "popup" : "reuse";
}

# --------------------------------------------------------------------------------
#
sub UMGetOtherPrefs
{
	my($values, $index) = @_;

	# EFFECTIVELY WE ARE DISABLING THE PREFERENCE TO
	# VIEW THE DOCUMENT IN THE SAME WINDOW, THIS HAS
	# JAVASCRIPT SECURITY PROBLEM.
	if ($index == $kUMDocumentViewIndex) {
		return 1;
	}
	my(@l) = split($kUMOtherPrefSeparator, $values);
	return $l[$index];
}

# --------------------------------------------------------------------------------
#
sub UMSetOtherPrefs
{
	my($values, $index, $value) = @_;
	my(@l) = split($kUMOtherPrefSeparator, $values);
	$l[$index] = $value;
	return join($kUMOtherPrefSeparator, @l);
}
