# $Id: replace.pl,v 1.1 1998/05/22 01:24:09 thyag Exp $
# author InfoPlace Inc.
# copyright 1997 InfoPlace, Inc.  All Rights Reserved.
# --------------------------------------------------------------
# For a given directory tree, do string replacement
# This is platform independent, 100% pure perl code,
# no "system" calls!
#
# --------------------------------------------------------------
# requires

# --------------------------------------------------------------
# globals

# --------------------------------------------------------------
# 
sub ReadFileIntoString
{
	my($file) = @_;
	my($data);
	if (!open(XFILE, "<$file")) {
		die "Failed to open file: $file";
	}
	local($/);
	$data = <XFILE>;
	close(XFILE);
	return $data;
}

# --------------------------------------------------------------
# 
sub DoFileReplace
{
  my($filename, $searchPattern, $replacePattern, $multiline) = @_;
  my($data) = &ReadFileIntoString($filename);

  my($result) = ($multiline) ? ($data =~ s/$searchPattern/$replacePattern/sg) 
	  : ($data =~ s/$searchPattern/$replacePattern/g);

  if ($result) {
	  my($savedfile) = $filename . ".tmp";
	  open(FILEOUT, ">$savedfile")  || die "Could not open file $savedfile : $!\n";
	  print FILEOUT $data;
	  close(FILEOUT);
	  if (rename($filename, "${filename}.save")) {
		  rename($savedfile, $filename);
	  }
	  else {
		  die "Failed to rename file $filename to ${filename}.save\n";
	  }
  }
  else {
	  print "File $filename is unchanged.\n";
  }
}

# --------------------------------------------------------------
#
sub DoReplace
{
	my($dirname, $level, $searchPattern, $replacePattern, $multiline) = @_;
	
    # read all file names from the given directory ...
    opendir(FILES, $dirname);
    my(@files);
    @files = readdir(FILES);
    closedir(FILES);

    my($filename, $i);

    # skip "." and "..", so start index from 2
    for ($i = 2; $i < $#files + 1; $i++) {
		$filename = $dirname . "/". $files[$i];
		if (-f $filename) {
			&DoFileReplace($filename, $searchPattern, $replacePattern, $multiline);
		}
		elsif ($level > 0) {
			&DoReplace($filename, $level - 1, $searchPattern, $replacePattern, $multiline);
		}
    }
}

# --------------------------------------------------------------
# 
sub printUsageAndExit
{
	print "Usage: $0: [-recurse <level>] <directory> <search-string>/file <replace-string>/file [-multi]\n";
	print "           Use -multi option if your search pattern can span multiple lines.\n";
	exit 0;
}

# --------------------------------------------------------------
# 
$multiline = 0;
if ($#ARGV >= 2 || $#ARGV <= 5) {
	my($dir, $level, $searchPattern, $replacePattern);
	if ($#ARGV == 2 || $#ARGV == 3) {
		$dir = $ARGV[0];
		$level = 0;
		$searchPattern = $ARGV[1];
		$replacePattern = $ARGV[2];
		if ($ARGV[3] eq "-multi") {
			$multiline = 1;
		}
	}
	else {
		if ($ARGV[0] ne "-recurse") {
			&printUsageAndExit();
		}
		$level = $ARGV[1];
		$dir = $ARGV[2];
		$searchPattern = $ARGV[3];
		$replacePattern = $ARGV[4];
		if ($ARGV[5] eq "-multi") {
			$multiline = 1;
		}
	}
	$sFile = "./" . $searchPattern;
	if (-f $sFile) { # searchPattern is a local filename
		# file contains the search string
		#local($/); # do not read entire file; just the first line
		open(XIN, "<$sFile")  || die "Could not open file $sFile : $!\n";
		$searchPattern = <XIN>;
		close(XIN);
	}
	$rFile = "./" . $replacePattern;
	if (-f $rFile) { # replacePattern is a local filename
		# file contains the replacement string
		local($/);
		open(XIN, "<$rFile")  || die "Could not open file $rFile : $!\n";
		$replacePattern = <XIN>;
		close(XIN);
	}
	print "Search Pattern: $searchPattern\n\nReplace Pattern: $replacePattern\n\n";
	(-d $dir) || die "Error: Directory $dir does not exists.";
	&DoReplace($dir, $level, $searchPattern, $replacePattern, $multiline);
} else {
	&printUsageAndExit();
}
exit 0;
